#include <TSystem.h>
#include <TTime.h>
#include <TMath.h>
#include <TROOT.h>

//#include <H5Cpp.h>

#include <iomanip>
#include <math.h>

#include "TH5Dataset.h"

ClassImp(TH5Dataset)

TH5Dataset::TH5Dataset(const char *fname, 
		       const Int_t subsampRate,
		       const EH5MemOpt memOpt,
		       TGFrame * parentFrame) :
  fMemOpt(memOpt),
  fSubsampRate(subsampRate), 
  fParentFrame(parentFrame)
{
  Int_t OK = Construct(fname);
  if (!OK)
    h5Error(this, "TH5Dataset", fParentFrame, "Dude! Could not create TH5Dataset!");
}

TH5Dataset::TH5Dataset(const TString & fnamestr, 
                       const Int_t subsampRate,
		       const EH5MemOpt memOpt,
                       TGFrame * parentFrame) :
  fMemOpt(memOpt),
  fSubsampRate(subsampRate), 
  fParentFrame(parentFrame)
{
  Int_t OK = Construct(fnamestr.Data());
  if (!OK)
    h5Error(this, "TH5Dataset",fParentFrame, "Dude! Could not create TH5Dataset!");
}

Int_t TH5Dataset::Construct(const char *fname)

{ // the real constructor!
  // extract file names; use standard strings for these manipulations
  // (rewrite with TStrings later?)

  string sFilename = string(fname);
  string sFullFilename(sFilename);

  if (sFilename.find("/") != string::npos)
    sFilename.erase(0, sFilename.find_last_of("/") + 1);

  string sFilenameNoExt(sFilename);

  if (sFilenameNoExt.find(".") != string::npos)
    sFilenameNoExt.erase(sFilenameNoExt.find_last_of("."));

  fFullFilename = TString(sFullFilename.c_str());
  fFilename = TString(sFilename.c_str());
  fFilenameNoExt = TString(sFilenameNoExt.c_str());

  // here check for file existence!!
  //  fH5File = H5PartOpenFile(fFullFilename.Data(), H5PART_READ);
  fH5File = NULL;
  if (!fH5File) {
    h5Error(this, "Construct", fParentFrame, "Dude!! The file %s does not exist!",fFullFilename.Data());
    return(0);
  }

  //  H5PartSetStep(fH5File, 0);
  //  Int_t nDataset = H5PartGetNumDatasets(fH5File);
  Int_t nDataset = 100;

  //----------------------------------------------------------------------
  // read in the variable/attribute names
  //----------------------------------------------------------------------

  const Int_t maxLength(64);
  char name[maxLength];

  for (Int_t i = 0; i < 9; i++) {
    fPartVarName[i] = TString("");
    fPartVarUnit[i] = TString("");
  }

  if (nDataset > 0) {
    for (Int_t i = 0; i < nDataset; i++) {
      //      H5PartGetDatasetName(fH5File, i, name, maxLength);
      TString varName = TString(name);
      if (varName == "x")
        fPartVarName[0] = varName;
      else if (varName == "y")
        fPartVarName[1] = varName;
      else if (varName == "z")
        fPartVarName[2] = varName;
      else if (varName == "phiz")
        fPartVarName[3] = varName;
      else if (varName == "px")
        fPartVarName[4] = varName;
      else if (varName == "py")
        fPartVarName[5] = varName;
      else if (varName == "pz")
        fPartVarName[6] = varName;
      else if (varName == "enez")
        fPartVarName[7] = varName;
      else if (varName == "id")
        fPartVarName[8] = varName;
      else
        h5Error(this, "Construct",fParentFrame,
                "Unrecognized variable name \"%s\" in file \"%s\", dude!",
                varName.Data(), fFilename.Data());
    }
  }

  TTime t0 = gSystem->Now();
  //  fNStep = H5PartGetNumSteps(fH5File);
  fNStep = 100;
  TTime t1 = gSystem->Now();
  TTime dt = t1;
  dt -= t0;

  fWidthStep = static_cast<Int_t>(ceil(log(fNStep)/log(10.)+0.01));

  cout << "\n\tOpened " << fFilename << ", contains "
      << fNStep << " steps, " << nDataset << " data sets.\n" << endl;

  //  cout << "\t GetNumSteps took " << dt.AsString() << " ms\n" << endl;

  //  fNStepAttr = H5PartGetNumStepAttribs(fH5File);
  //  fNFileAttr = H5PartGetNumFileAttribs(fH5File);
  fNStepAttr = 12;
  fNFileAttr = 13;

  // list file attributes:
  char an[32];
  TString attrName;
  //  h5part_int64_t *nElem = (h5part_int64_t *) new int;
  Int_t *nElem = new int;

  if (fNFileAttr > 0) {
    printf("\tFound %d file attributes:\n", fNFileAttr);
    for (Int_t n = 0; n < fNFileAttr; n++) {

      //      H5PartGetFileAttribInfo(fH5File, n, an, 32, 0, nElem);
      attrName = TString(an);
      fFileAttr.AddLast(new TObjString(attrName));

      // nice print out:
      if (n == fNFileAttr - 1) 
        cout << attrName << endl << endl;
      else if (n == 0) 
        cout << "\t" << attrName << ", ";
      else {
        cout << attrName << ", ";
        if (!((n + 1) % 5))
          cout << endl << "\t";
      }
    }
  } else {
    printf("\tFound no file attributes.\n");
  }

  printf("\tFound %d step attributes:\n", fNStepAttr);

  // retrieve step attributes for step 0 (assume they're the same for all steps!)
  //H5PartSetStep(fH5File, 0);
  TString sposName;

  fNScalarAttr = 0;
  fNVectorAttr = 0;

  // initialize names:
  for (Int_t n = 0; n < fNStepAttr; ++n) {
    //H5PartGetStepAttribInfo(fH5File, n, an, 32, 0, nElem);
    //    printf("Attrib name nr %d is \"%s\", it has %d elements \n",n,an,*nElem);
    attrName = TString(an);
    if ((attrName == "SPOS" || attrName == "sPOS" || attrName == "Spos"
         || attrName == "spos" || attrName == "s")
        && *nElem == 1) {
      sposName = attrName;
    }

    TString type("?");
    if (*nElem == 1) {          // scalar variable
      fScalarName.AddLast(new TObjString(attrName));
      fNScalarAttr++;
      type = TString(" (scalar)");
    } else if (*nElem == 3) {   // vector variable
      fVectorName.AddLast(new TObjString(attrName));
      fNVectorAttr++;
      type = TString(" (vector)");
    } else {
      type = TString(" (");
      type += *nElem;
      type += TString(" elements)");
      h5Warning(this, "Construct",fParentFrame,
                "Step attribute %s with %d elements will be ignored, dude!",
                an, *nElem);
    }
    TString nameAndType = attrName + type;
    if (n == fNStepAttr - 1) {
      cout << nameAndType << endl << endl;
    } else if (n == 0) {
      cout << "\t" << nameAndType << ", ";
    } else {
      cout << nameAndType << ", ";
      if (!((n + 1) % 3))
        cout << endl << "\t";
    }
  }

  //----------------------------------------------------------------------
  // read in the units
  //----------------------------------------------------------------------

  for (Int_t i = 0; i < 9; i++)
    fPartVarUnit[i] = GetUnit(fPartVarName[i]);
  for (Int_t i = 0; i < fNScalarAttr; i++)
    fScalarUnit.AddLast(new TObjString(GetUnit(static_cast<TObjString*>(fScalarName.At(i))->GetString())));
  for (Int_t i = 0; i < fNVectorAttr; i++)
    fVectorUnit.AddLast(new TObjString(GetUnit(static_cast<TObjString*>(fVectorName.At(i))->GetString())));

  //----------------------------------------------------------------------
  // read in the data
  //----------------------------------------------------------------------

  // initialize (clones) arrays
  fSpos = new TArrayD(fNStep);
  // add filename to histogram name to avoid memory leak warning
  fScalar = new TH2D((TString("fScalar-") + fFilenameNoExt).Data(),
                     "scalar values", fNStep, 0., 1., fNScalarAttr, 0.,
                     1.);
  fVector = new TClonesArray("TClonesArray", fNStep);
  for (Int_t step = 0; step < fNStep; ++step) {
    TClonesArray & fV = *fVector;
    new(fV[step]) TClonesArray("TVector3", fNVectorAttr);
  }
  
  fNtuple = new TClonesArray("TNtupleD", fNStep);
  
  fSigma =
    new TH3D((TString("fSigma-") + fFilenameNoExt).Data(),
	     "sigma matrices", fNStep, 0., 1., 8, 0., 1., 8, 0., 1.);
  
  fMean = new TH2D((TString("fMean-") + fFilenameNoExt).Data(),
                   "variable means", fNStep, 0., 1., 8, 0., 1.);
  
  fSigmaSl =
    new TH3D((TString("fSigmaSl-") + fFilenameNoExt).Data(),
	     "sigma matrices (slice)", fNStep, 0., 1., 8, 0., 1., 8, 0.,
	     1.);
  
  fMeanSl = new TH2D((TString("fMeanSl-") + fFilenameNoExt).Data(),
                     "variable means (slice)", fNStep, 0., 1., 8, 0., 1.);
  
  fSigmaSc =
    new TH3D((TString("fSigmaSc-") + fFilenameNoExt).Data(),
	     "sigma matrices for screen projection", fNStep, 0., 1., 8, 0., 1., 8, 0., 1.);
  
  fMeanSc = new TH2D((TString("fMeanSc-") + fFilenameNoExt).Data(),
			     "variable means for screen projection", fNStep, 0., 1., 8, 0., 1.);
  
  fSliceCut = new TClonesArray("TCut", fNStep);
  fSlicePos = new TArrayD(fNStep);

  TH5ProgressBar *progBar = NULL;

  // Loop over steps, fill ntuples and vectors (one per step)
  Double_t val;
  Double_t spos;
  Double_t arr[3];

  const char *progBarMsg = "loading steps...";
  if (fMemOpt == kPreloadAll)
    progBarMsg = "preloading all steps into memory... patience dude!";

  for (Int_t step = 0; step < fNStep; step++) {

    //H5PartSetStep(fH5File, step);
    //    unsigned long n = H5PartGetNumParticles(fH5File);
    unsigned long n = 10000;

    // read in scalar attributes
    for (int i = 0; i < fNScalarAttr; ++i) {
      TObjString *s = static_cast<TObjString*>(fScalarName.At(i));
      //      H5PartReadStepAttrib(fH5File,
      //                     const_cast<char*>(s->GetString().Data()),
      //                     &val);
      if (TMath::Abs(val) < 1.e-20) {
        if (step != 0 && val != 0.) {
          Warning("Construct",
                  "Read in very small value %e for attribute %s, will set to 0.",
                  val, const_cast<char*>(s->GetString().Data()));
        }
        val = 0.;
      }
      fScalar->SetBinContent(step, i, val);
    }

    // separately read in spos values:
    //  H5PartReadStepAttrib(fH5File, const_cast<char*>(sposName.Data()),&spos);
    (*fSpos)[step] = spos;

    // read in vector attributes
    for (int i = 0; i < fNVectorAttr; ++i) {
      TObjString *s = static_cast<TObjString*>(fVectorName.At(i));
      //H5PartReadStepAttrib(fH5File,
      //                           const_cast<char*>(s->GetString().Data()),
      //                     arr);
      TClonesArray *a = static_cast<TClonesArray*>(fVector->At(step));
      if (TMath::Abs(arr[0]) < 1.e-20 && TMath::Abs(arr[1]) < 1.e-20
          && TMath::Abs(arr[2]) < 1.e-20) {
        Warning("Construct",
                "Read in very small values %e, %e, %e for attribute %s, will set to 0.",
                arr[0], arr[1], arr[2],
                const_cast<char*>(s->GetString().Data()));
        arr[0] = 0.;
        arr[1] = 0.;
        arr[2] = 0.;
      }
      (*a)[i] = new TVector3(arr);
    }

    // demonstrate how to read elements from fVector:
    if (fNVectorAttr > 0) {
      TClonesArray *fV = static_cast<TClonesArray*>(fVector->At(step));
      TVector3 *v = static_cast<TVector3*>(fV->At(0));
      cout << "\tStep " << step << ": n = " << n << ", Spos = " <<
          (*fSpos)[step] << ",\t" << static_cast<TObjString*>(fVectorName.At(0))->GetString() << 
          ": " << v->x() << " " << v->y() << " " << v->z() << endl;
    }
    // if preload option, load all steps right here:
    if (fMemOpt == kPreloadAll) {
      LoadStep(step);

      // inform user with a progress bar
      if (fParentFrame) {
	if (0 == step) {
	  progBar = new
            TH5ProgressBar(gClient->GetRoot(), this, 400, 200, progBarMsg);
	  progBar->Reset();
	}
	progBar->IncrementStep(step, fNStep);
	if (progBar->ReceivedCancel()) return (0);
	gSystem->ProcessEvents();
      } 
    }
    // initialize sigma matrices:
    for (Int_t i = 0; i < 8; i++) {
      for (Int_t j = 0; j < 8; j++)
        fSigma->SetBinContent(step, i, j, EMPTY);
      for (Int_t j = 0; j < 8; j++)
        fSigmaSl->SetBinContent(step, i, j, EMPTY);

      for (Int_t j = 0; j < 8; j++)
        fSigmaSc->SetBinContent(step, i, j, EMPTY);

      // ... and mean arrays:
      fMean->SetBinContent(step, i, EMPTY);
      fMeanSl->SetBinContent(step, i, EMPTY);
      fMeanSc->SetBinContent(step, i, EMPTY);
    }

    // initialize slice cut array:
    TClonesArray & fS = *fSliceCut;
    new(fS[step]) TCut("1");

    (*fSlicePos)[step] = EMPTY;
  }                             // end for loop over steps

  if (progBar) {
    delete progBar;
    progBar = NULL;
  }
  // set file active:
  fFileState = kActive;

  // make first step current ntuple:
  fActiveStep = 1;
  fActiveNt = GetNtuple(0);

  fNPart0 = fActiveNt->GetEntries();

  fPlotCount = 0;

  // default slice parameters:
  fSlicePosOpt = kMeanZ;
  fSlicePercent = 20.;
  fSlicePartPerBin = 100;

  fSliceExtentOpt = kLength;
  fSliceLength = 1.0;
  fSliceLengthUnit = kmm;
  fSliceSigma = 0.0;
  fSliceSigmaUnit = kmm;

  return (1);
}


TH5Dataset::~TH5Dataset()
{

  delete[]fH5File;

  delete fSpos;
  delete fScalar;
  delete fVector;
  delete fNtuple;

  delete fMean;
  delete fSigma;
  delete fMeanSl;
  delete fSigmaSl;
  delete fMeanSc;
  delete fSigmaSc;

  delete fSliceCut;
  delete fSlicePos;

}                               // Destructor


Double_t TH5Dataset::GetSpos(const Int_t step) const 
{

  if (step < fNStep) {
    return (fSpos->At(step));
  } else {
    Warning("GetSpos",
            "Step %u is requested, but there are only %u steps!", step,
            fNStep);
    return (fSpos->At(fNStep - 1));
  }
}

Int_t TH5Dataset::GetStep(const Double_t spos) const
{

  if (fNStep == 0)
    return (0);
  Int_t step = 0;

  while (fSpos->At(step) < spos && step < fNStep) {
    step++;
    if (step == fNStep)
      break;
  }
  step -= 1;
  return (step);

}

Double_t TH5Dataset::GetStepVar(const TString* stepVarName, 
				const Int_t step, 
				const EH5VarType vtype) {

  if (vtype==kScalar) return(GetScalarStepVar(stepVarName, step));
  if (vtype==kVectorX) return(GetVectorStepVar(stepVarName, step).x());
  if (vtype==kVectorY) return(GetVectorStepVar(stepVarName, step).y());
  if (vtype==kVectorZ) return(GetVectorStepVar(stepVarName, step).z());
  if (vtype==kPart||vtype==kPartSlice||vtype==kPartScreen) return(GetPartStepVar(stepVarName, step, vtype));
  return(0.);
}

Double_t TH5Dataset::GetScalarStepVar(const TString * stepVarName,
                                      const Int_t step) const
{

  Int_t st = step;
  if (st >= fNStep) {
    Warning("GetScalarStepVar",
            "Step %u is requested, but there are only %u steps!", step,
            fNStep);
    st = fNStep - 1;
  }
  Double_t val(0.);
  for (Int_t i = 0; i < fNScalarAttr; ++i) {
    TObjString *s = static_cast<TObjString*>(fScalarName.At(i));
    if (s->GetString() == *stepVarName)
      val = fScalar->GetBinContent(st, i);
  }
  return (val);
}

TVector3 TH5Dataset::GetVectorStepVar(const TString * stepVarName,
                                      const Int_t step) const 
{

  Int_t st = step;
  if (st >= fNStep) {
    Warning("GetVectorStepVar",
            "Step %u is requested, but there are only %u steps!", step,
            fNStep);
    st = fNStep - 1;
  }
  TVector3 vec = TVector3(0., 0., 0.);
  for (Int_t i = 0; i < fNVectorAttr; ++i) {
    TObjString *s = static_cast<TObjString*>(fVectorName.At(i));
    if (s->GetString() == *stepVarName) {
      TClonesArray *fV = static_cast<TClonesArray*>(fVector->At(st));
      TVector3 *pv = static_cast<TVector3*>(fV->At(i));
      vec = *pv;
    }
  }
  return (vec);

}

Double_t TH5Dataset::GetPartStepVar(const TString* stepVarName, 
				    const Int_t step,
				    const EH5VarType vtype) { 
  // note: this method cannot be const, since ntuple must be loaded if needed!

  TString s = *stepVarName;

  // check for correct variable start:
  TString start = s(0,4);
  if (!(start==TString("#mu(")||start==TString("#sig")||start==TString("#var"))) { 
    h5Error(this,"GetPartStepVar",fParentFrame,"Unexptected variable name \"%s\", dude!",*stepVarName->Data());
    return(0.);
  }
  
  // recover x and y variable names from stepVarName (between brackets):
  Ssiz_t L = s.First("(");
  Ssiz_t M = s.First(",");
  
  TString funName = s(0,L);

  TString varName1;
  TString varName2;
  if (M > 0) {
    varName1 = s(L+1,M-L-1);
    varName2 = s(M+1,s.Length()-M-2);
  } else {
    varName1 = s(L+1,s.Length()-L-2);
    varName2 = varName1;
  }

  Double_t var(0.);

  if (funName==TString("#mu")) 
    var=GetPartMean(&varName1, step, vtype);
  else if (funName==TString("#sigma")) { 
    // return square root of covariance
    var=GetPartCov(&varName1, &varName2, step, vtype);
    var=sqrt(TMath::Abs(var)); }
  else if (funName==TString("#varepsilon")) 
    var=GetPartEmit(&varName1, &varName2, step, vtype);
  else {
    h5Error(this,"GetPartStepVar",fParentFrame,"Unexptected variable name \"%s\" ",*stepVarName->Data());
    return(0.);
  }
    
  return(var);
}

TString TH5Dataset::GetPartVarName(const Int_t index) const
{
  if (index >= 0 && index < 9) {
    return (fPartVarName[index]);
  } else {
    return (TString(""));
  }
}

Int_t TH5Dataset::GetPartVarIndex(const TString * varName) const
{
  Int_t index(-1);
  while (index < 9) {
    index++;
    if (fPartVarName[index] == *varName)
      return (index);
  }
  return (index);
}

TString TH5Dataset::GetScalarStepVarName(const Int_t index) const
{

  return (static_cast<TObjString*>(fScalarName.At(index))->GetString());
}

TString TH5Dataset::GetVectorStepVarName(const Int_t index) const
{

  return (static_cast<TObjString*>(fVectorName.At(index))->GetString());
}

TString TH5Dataset::GetUnit(const char* vname) const
{
  return GetUnit(TString(vname));
}

TString TH5Dataset::GetUnit(const TString varName) const
{

  // check if the variable is derived from particle statistics
  // (mean, covariance, emittance,...):
  if (varName.Contains("(") && varName.Contains(")"))
    return (GetPartUnit(varName));

  TString unit("");
  TString varNameU = varName;
  varNameU += TString("Unit");

  // loop over file attributes, look for entry var+Unit:
  for (Int_t i = 0; i < fNFileAttr; i++) {

    TObjString *s = static_cast<TObjString*>(fFileAttr.At(i));
    if (s->GetString() == varNameU) {
      char u[32];
      //H5PartReadFileAttrib(fH5File,
      //                           const_cast<char*>(s->GetString().Data()),
      //                     &u);
      unit = TString(u);
    }
  }
  return (unit);
}

TString TH5Dataset::GetPartUnit(const char* vname) const
{
  return GetPartUnit(TString(vname));
}

TString TH5Dataset::GetPartUnit(const TString varName) const
{

  TString unit("");

  // deconstruct varName:
  TString s = varName;
  Ssiz_t L = s.First("(");
  Ssiz_t M = s.First(",");

  TString funName = s(0, L);

  TString varName1;
  TString varName2;
  if (M > 0) {
    varName1 = s(L + 1, M - L - 1);
    varName2 = s(M + 1, s.Length() - M - 2);
  } else {
    varName1 = s(L + 1, s.Length() - L - 2);
    varName2 = varName1;
  }

  TString unit1 = GetUnit(varName1);
  TString unit2 = GetUnit(varName2);

  if (funName == TString("#mu")) 
    unit = unit1;
  else if (funName == TString("#sigma"))
    if (unit1 == unit2) unit = unit1;
    else unit = "#sqrt{" + unit1 + " " + unit2 + "}";
  else  // (funName == TString("#varepsilon"))
    if (unit1 == unit2) unit = unit1 + TString("^{2}");
    else unit = unit1 + " " + unit2;

  return (unit);
}

TNtupleD* TH5Dataset::GetNtuple(Int_t step)
{

  if (step >= fNStep) {
    Error("GetNtuple", "File only has %d steps, will use last step",
          fNStep);
    step = fNStep - 1;
  }

  if (step == fActiveStep) {
    // same step, just return current one
    return (fActiveNt);
  } else {
    // need to load new step, delete old one if requested
    if (fMemOpt == kLoadAndDump) fNtuple->RemoveAt(fActiveStep);
    fActiveStep = step;
    if (!fNtuple->At(step)) LoadStep(step);
    TNtupleD *nt = static_cast<TNtupleD*>(fNtuple->At(step));
    nt->SetEstimate(nt->GetEntries()); // needed for GetV1 etc. operations
    fActiveNt = nt;
    return (fActiveNt);
  }
}

void TH5Dataset::DumpNtuple(const Int_t step)
{
  fNtuple->RemoveAt(step);
}

Double_t TH5Dataset::GetPartMean(const TString * varName, 
				 const Int_t step,
                                 const EH5VarType vtype)
{
  // for Gaussian-weighted slice, delegate to special method:
  if (vtype==kPartSlice && fSliceSigma > 0.) return(GetGaussSliceStepVar(varName, step, kMean));
  
  // for screen-projected mean, delegate to special method:
  if (vtype==kPartScreen) return (GetScreenProjectedStepVar(varName, step, kMean));

  TCut sliceCut("1");
  if (vtype==kPartSlice) sliceCut = *(GetSliceCut(step));

  // check if element is already in array, else compute from ntuple:
  Int_t index = GetPartVarIndex(varName);
  Double_t mean;

  if (vtype==kPartSlice) {
    mean = fMeanSl->GetBinContent(step, index);
  } else {
    mean = fMean->GetBinContent(step, index);
  }
  if (mean == EMPTY) {
    TString vname = *varName;
    TNtupleD *nt = GetNtuple(step);
    nt->Draw(vname + TString(":") + vname, sliceCut, "goff");
    TGraph *graphXX =
        new TGraph(nt->GetSelectedRows(), nt->GetV2(), nt->GetV1());
    mean = graphXX->GetMean(1);
    delete graphXX;
    if (vtype==kPartSlice) {
      fMeanSl->SetBinContent(step, index, mean);
    } else {
      fMean->SetBinContent(step, index, mean);
    }
  }
  return (mean);
}

Double_t TH5Dataset::GetPartCov(const TString * varNameX,
                                const TString * varNameY, 
				const Int_t step,
                                const EH5VarType vtype)
{
  // for Gaussian-weighted slice, delegate to special method:
  if ( vtype==kPartSlice && fSliceSigma > 0.) 
    return(GetGaussSliceStepVar(varNameX, varNameY, step, kCov));
  
  // for screen-projected covariance, delegate to special method:
  if ( vtype==kPartScreen) return (GetScreenProjectedStepVar(varNameX, varNameY, step, kCov));

  TCut sliceCut("1");
  if (vtype==kPartSlice)
    sliceCut = *(GetSliceCut(step));

  // check if element is already in array, else compute from ntuple:
  Int_t ix = GetPartVarIndex(varNameX);
  Int_t iy = GetPartVarIndex(varNameY);
  Double_t cov;
  
  if (vtype==kPartSlice) {
    cov = fSigmaSl->GetBinContent(step, ix, iy);
  } else {
    cov = fSigma->GetBinContent(step, ix, iy);
  }

  if (cov == EMPTY) {
    TString vxname = *varNameX;
    TString vyname = *varNameY;
    TNtupleD *nt = GetNtuple(step);
    nt->Draw(vyname + TString(":") + vxname, sliceCut, "goff");
    TGraph *graphXY =
        new TGraph(nt->GetSelectedRows(), nt->GetV2(), nt->GetV1());
    cov = graphXY->GetCovariance();
    delete graphXY;
    if (vtype==kPartSlice) {
      fSigmaSl->SetBinContent(step, ix, iy, cov);
    } else {
      fSigma->SetBinContent(step, ix, iy, cov);
    }
  }

  return (cov);
}

Double_t TH5Dataset::GetPartEmit(const TString * varNameX,
                                 const TString * varNameY,
                                 const Int_t step, 
				 const EH5VarType vtype)
{
  // for Gaussian-weighted slice, delegate to special method:
  if ( vtype==kPartSlice && fSliceSigma > 0.) 
    return(GetGaussSliceStepVar(varNameX, varNameY, step, kEmit));
 
  Double_t covXX, covXY, covYY;
  covXX = GetPartCov(varNameX, varNameX, step, vtype);
  covXY = GetPartCov(varNameX, varNameY, step, vtype);
  covYY = GetPartCov(varNameY, varNameY, step, vtype);

  Double_t eps = covXX * covYY - covXY * covXY;
  eps = sqrt(eps);
  
  return (eps);
}

Double_t TH5Dataset::GetPartPeak(const TString * varName, 
				 const Int_t step,
                                 const Int_t nBin)
{
  // find the peak of the variable distribution in a histogram with nBins bins

  Double_t min, max;
  Range(varName, step, min, max);
  TH1F *histVar = Histo1d(varName, step, min, max, nBin);
  Double_t peakLocation = histVar->GetBinCenter(histVar->GetMaximumBin());
  return (peakLocation);

}

void TH5Dataset::SetSliceOptions(const EH5SlicePosOpt posOpt,
				 const Int_t partPerBin,
				 const EH5SliceExtentOpt extOpt,
				 const Double_t percent,
				 const Double_t length,
				 const EH5SliceUnit lengthUnit,
				 const Double_t sigmaValue,
				 const EH5SliceUnit sigmaUnit)
{
  fSlicePosOpt = posOpt;
  if (posOpt == kPeakZ)
    fSlicePartPerBin = partPerBin;
  
  fSliceExtentOpt = extOpt;
  if (extOpt == kPercent)
    fSlicePercent = percent;
  if (extOpt == kLength) {
    fSliceLength = length;
    fSliceLengthUnit = lengthUnit;
  }

  fSliceSigma = sigmaValue;
  fSliceSigmaUnit = sigmaUnit;

  // reset array of slice cuts (to make sure cuts are re-evaluated)
  for (Int_t step = 0; step < fNStep; step++) {
    TCut *slice = static_cast<TCut*>(fSliceCut->At(step));
    *slice = TCut("1");
    //    *(fSliceCut->At(step)) = TCut("1");
    (*fSlicePos)[step] = EMPTY;
  }

  // reset all slice quantities (need to be recomputed with new settings!)
  for (Int_t step = 0; step < fNStep; step++) {
    for (Int_t i = 0; i < 8; i++) {
      fMeanSl->SetBinContent(step, i, EMPTY);
      for (Int_t j = 0; j < 8; j++)
        fSigmaSl->SetBinContent(step, i, j, EMPTY);
    }
  }

}

void TH5Dataset::SetSlicePosOpt(const EH5SlicePosOpt posOpt,
				const Int_t partPerBin)
{
  SetSliceOptions(posOpt,partPerBin,fSliceExtentOpt,fSlicePercent,
		  fSliceLength,fSliceLengthUnit,fSliceSigma,fSliceSigmaUnit);
}

void TH5Dataset::SetSlicePosition(const char* pos,
				  const Int_t partPerBin)
{
  TString p(pos);
  EH5SlicePosOpt posOpt;
  if (p.Contains("mean", TString::kIgnoreCase) )
    posOpt = kMeanZ;
  else if (p.Contains("median", TString::kIgnoreCase) )
    posOpt = kMedianZ;
  else if (p.Contains("half", TString::kIgnoreCase) )
    posOpt = kHalfZ;
  else if (p.Contains("start", TString::kIgnoreCase) || 
	   p.Contains("begin", TString::kIgnoreCase) ) 
    posOpt = kStartZ;
  else if (p.Contains("end", TString::kIgnoreCase) )
    posOpt = kEndZ;
  else if (p.Contains("peak", TString::kIgnoreCase) ||
           p.Contains("max", TString::kIgnoreCase) )
    posOpt = kPeakZ;
  else 
    Error("SetSlicePosition", "Incomprehensible slice position request: %s",
          pos);
  SetSlicePosOpt(posOpt, partPerBin);
}

void TH5Dataset::SetSlicePartPerBin(const Int_t partPerBin)
{
  SetSlicePosOpt(kPeakZ,partPerBin);
}

void TH5Dataset::SetSliceLength(const Double_t length, 
				const EH5SliceUnit unit)
{
  SetSliceOptions(fSlicePosOpt,fSlicePartPerBin,kLength,0.,
		  length,unit,fSliceSigma,fSliceSigmaUnit);
}

void TH5Dataset::SetSliceLength(const Double_t length, 
				const char* unit)
{
  EH5SliceUnit lengthUnit = kmm;
  TString su(unit);
  if (su.Contains("mm", TString::kIgnoreCase) ||
      su.Contains("millimeter", TString::kIgnoreCase) ||
      su.Contains("millimetre", TString::kIgnoreCase) )
    lengthUnit = kmm;
  else if (su.Contains("ps", TString::kIgnoreCase) ||
	   su.Contains("picosec", TString::kIgnoreCase))
    lengthUnit = kps;

  SetSliceLength(length, lengthUnit);
}

void TH5Dataset::SetSliceLengthUnit(const char* unit)
{
  SetSliceLength(fSliceLength,unit);
}

void TH5Dataset::SetSliceLengthUnit(const EH5SliceUnit unit)
{
  SetSliceLength(fSliceLength,unit);
}

void TH5Dataset::SetSliceSigma(const Double_t sigma, 
			       const EH5SliceUnit unit)
{
  SetSliceOptions(fSlicePosOpt,fSlicePartPerBin,fSliceExtentOpt,fSlicePercent,
		  fSliceLength,fSliceLengthUnit,sigma,unit);
}

void TH5Dataset::SetSliceSigma(const Double_t sigma, 
			       const char* unit)
{
  EH5SliceUnit sigmaUnit = kmm;
  TString su(unit);
  if (su.Contains("mm", TString::kIgnoreCase) ||
      su.Contains("millimeter", TString::kIgnoreCase) ||
      su.Contains("millimetre", TString::kIgnoreCase) )
    sigmaUnit = kmm;
  else if (su.Contains("ps", TString::kIgnoreCase) ||
	   su.Contains("picosec", TString::kIgnoreCase))
    sigmaUnit = kps;

  SetSliceSigma(sigma, sigmaUnit);
}


void TH5Dataset::SetSliceSigmaUnit(const char* unit)
{
  SetSliceSigma(fSliceSigma,unit);
}

void TH5Dataset::SetSliceSigmaUnit(const EH5SliceUnit unit)
{
  SetSliceSigma(fSliceSigma,unit);
}

void TH5Dataset::SetSlicePercent(const Double_t percent)
{
  SetSliceOptions(fSlicePosOpt,fSlicePartPerBin,kPercent,percent,
		  fSliceLength,fSliceLengthUnit,fSliceSigma,fSliceSigmaUnit);
}

TCut *TH5Dataset::GetSliceCut(const Int_t step)
{
  // get cut from array, check if it has been determined already:
  TCut *sliceCut = static_cast<TCut*>(fSliceCut->At(step));
  if (TString(*sliceCut) == TString("1")) { // nope, need to construct it
    TString sliceVar("z");
    if (fSliceExtentOpt == kPercent)
      sliceCut = GetSliceCutPercent(&sliceVar, step, fSlicePercent, 
				    fSlicePosOpt, fSlicePartPerBin);
    else if (fSliceExtentOpt == kLength)
      sliceCut = GetSliceCutLength(&sliceVar, step, fSliceLength, fSliceLengthUnit, 
				   fSlicePosOpt, fSlicePartPerBin);
    else {
      cout << "GetSliceCut: unknown slice extent option: " << fSliceExtentOpt << endl;
      sliceCut = static_cast<TCut*>(fSliceCut->At(step));
    }
  }
  return (sliceCut);
}

TCut *TH5Dataset::GetSliceCutPercent(const TString * varName, 
				     const Int_t step,
				     const Double_t slicePercent,
				     const EH5SlicePosOpt posOpt,
				     const Int_t slicePartPerBin)
{
  TCut *sliceCut = static_cast<TCut*>(fSliceCut->At(step));
  Double_t slicePos = GetSlicePos(varName, step, posOpt, slicePartPerBin);
  
  // The following cut needed for gun simulations: 
  // non-emitted particles all have z=0...
  TCut positiveZ("z>0.0"); 
  
  TNtupleD *nt = GetNtuple(step);
  Int_t nPart = nt->GetEntries();
  Int_t nPartFrac = TMath::FloorNint(0.005 * slicePercent * nPart);   // factor 0.5 for +/-
  
  if (posOpt == kStartZ) {
    slicePos = 1.e20;
    nPartFrac = TMath::FloorNint(0.01 * slicePercent * nPart);
  } else if (posOpt == kEndZ) {
    slicePos = -1.e20;
    nPartFrac = TMath::FloorNint(0.01 * slicePercent * nPart);
  }
  
  // get array with entries above position:
  TString sel = *varName;
  sel += TString(">");
  sel += slicePos;
  TCut selection(sel.Data());
  nt->Draw(*varName, selection&&positiveZ, "goff");
  TGraph *above = new TGraph(nt->GetSelectedRows(), nt->GetV1(), nt->GetV1());
  Int_t nAbove = above->GetN();
  //  Double_t* arrayAbove = nt->GetV1();
  
  // ...and array with entries below position:
  sel = *varName;
  sel += TString("<");
  sel += slicePos;
  selection = sel.Data();
  nt->Draw(*varName, selection&&positiveZ, "goff");
  TGraph *below = new TGraph(nt->GetSelectedRows(), nt->GetV1(), nt->GetV1());
  Int_t nBelow = below->GetN();
  //  Double_t* arrayBelow = nt->GetV1();
  
  // now find m-th smallest element in above array
  // (for the time being, use TGraph's Sort method, later perhaps speed up
  // with dedicated selection routine?)
  
  above->Sort();
  below->Sort();
  
  Double_t smin, smax, dummy;
  if (nAbove > 0) {
    if (nAbove < nPartFrac) {
      above->GetPoint(nAbove - 1, smax, dummy);
    } else {
      above->GetPoint(nPartFrac - 1, smax, dummy);
    }
  } else {                    // no points in above array -> get highest from below array
    below->GetPoint(nBelow - 1, smax, dummy);
  }
  
  // and m-th largest element in below array
  if (nBelow > 0) {
    if (nBelow < nPartFrac) {
      below->GetPoint(0, smin, dummy);
    } else {
      below->GetPoint(nBelow - nPartFrac, smin, dummy);
    }
  } else {                    // no points in below array -> get lowest from above array
    above->GetPoint(0, smin, dummy);
  }
  
  delete above;
  delete below;
  
  // if at the start or end of bunch, also change the slice center accordingly
  if (posOpt == kStartZ || posOpt == kEndZ) {
    (*fSlicePos)[step] = (smax - smin)/2.;
  }
  
  TString cut(*varName);
  cut += ">";
  cut += smin;
  cut += "&&";
  cut += *varName;
  cut += "<";
  cut += smax;
  *sliceCut = TCut(cut.Data());

  return (sliceCut);
}

TCut *TH5Dataset::GetSliceCutLength(const TString * varName, 
				    const Int_t step,
				    const Double_t sliceLength,
				    const EH5SliceUnit sliceLengthUnit,
				    const EH5SlicePosOpt posOpt,
				    const Int_t slicePartPerBin)
{
  // get slice cut for a given slice lenght
  // Note: ignores Gauss sigma that might be set (could think about cutting slice
  // length to some number of sigma's to safe computing time, but let that be the
  // caller's responsibility!

  TCut *sliceCut = static_cast<TCut*>(fSliceCut->At(step));
  Double_t slicePos = GetSlicePos(varName, step, posOpt, slicePartPerBin);
  
  const Double_t half = 0.5;  // factor for +/- 
  const Double_t mm = 1.0e-3; // convert mm to m

  Double_t sliceL(sliceLength);

  if (sliceLengthUnit == kps) sliceL = ps2mm(sliceL,step);
  
  Double_t halfLength = sliceL * half * mm;
  //modify slicePos according to "posOpt", to get full slice when at the edge of bunch
  if (posOpt == kStartZ) {
    slicePos -= halfLength;
    (*fSlicePos)[step] = slicePos;
  }
  if (posOpt == kEndZ) {
    slicePos += halfLength;
    (*fSlicePos)[step] = slicePos;
  }
  TString cut(*varName);
  cut += ">";
  Double_t smin = slicePos - halfLength;
  cut += smin;
  cut += "&&";
  cut += *varName;
  cut += "<";
  Double_t smax = slicePos + halfLength;
  cut += smax;
  *sliceCut = TCut(cut.Data());

  return (sliceCut);
}

Double_t TH5Dataset::GetSlicePos(const TString* varName, 
				 const Int_t step,
                                 const EH5SlicePosOpt posOpt,
                                 const Int_t slicePartPerBin)
{
  Double_t slicePos = (*fSlicePos)[step];

  if (slicePos == EMPTY) {
    // slice position has not been computed yet, do it now:
    TNtupleD *nt = GetNtuple(step);
    Int_t nPart = nt->GetEntries();

    // determine position of slice according to position option:
    if (posOpt == kMeanZ) {
      // find mean of ntuple column (via TGraph...)
      nt->Draw(*varName, "1", "goff");
      TGraph *column =
          new TGraph(nt->GetSelectedRows(), nt->GetV1(), nt->GetV1());
      slicePos = column->GetMean(1);
      delete column;
    } else if (posOpt == kMedianZ) {
      // find median of ntuple column (via TGraph...)
      nt->Draw(*varName, "1", "goff");
      TGraph *column =
          new TGraph(nt->GetSelectedRows(), nt->GetV1(), nt->GetV1());
      column->Sort();
      Int_t nPartHalf = TMath::FloorNint(0.5 * nPart);
      Double_t xp, yp;
      column->GetPoint(nPartHalf, xp, yp);
      slicePos = xp;
      delete column;
    } else if (posOpt == kHalfZ) {
      Double_t min = nt->GetMinimum(*varName);
      Double_t max = nt->GetMaximum(*varName);
      slicePos = 0.5 * (min + max);
    } else if (posOpt == kStartZ) {
      slicePos = nt->GetMaximum(*varName);
    } else if (posOpt == kEndZ) {
      slicePos = nt->GetMinimum(*varName);
    } else if (posOpt == kPeakZ) {
      Double_t min = nt->GetMinimum(*varName);
      Double_t max = nt->GetMaximum(*varName);
      Int_t nBin = (slicePartPerBin > 0) ? nPart / slicePartPerBin : 100;
      TCut positiveZ("z>0.0");
      TH1F *histVar = Histo1d(varName, step, min, max, nBin, positiveZ);
      slicePos = histVar->GetBinCenter(histVar->GetMaximumBin());
    }
    (*fSlicePos)[step] = slicePos;
  }
  return slicePos;
}

Int_t TH5Dataset::LoadStep(const Int_t step)
{
  char ntupleName[16];
  sprintf(ntupleName, "ntuple%u", step);
  char ntupleDesc[32];
  sprintf(ntupleDesc, "data from step %u", step);

  //H5PartSetStep(fH5File, step);
  //  unsigned long n = H5PartGetNumParticles(fH5File);
  unsigned long n = 10000;

  TClonesArray & fN = *fNtuple;
  new(fN[step]) TNtupleD(ntupleName, ntupleDesc,
                        "x:y:z:phiz:px:py:pz:enez:id");

  double *x = new double[n];
  double *y = new double[n];
  double *z = new double[n];
  double *px = new double[n];
  double *py = new double[n];
  double *pz = new double[n];

  Int_t *id = new Int_t[n];

  double *phiz = new double[n];
  double *enez = new double[n];

  // H5PartReadParticleStep(fH5File,step,x,y,z,px,py,pz,id);

  if (fPartVarName[0] == "x") {
    //    H5PartReadDataFloat64(fH5File, "x", x);
  } else {
    for (unsigned long i = 0; i < n; i++)
      x[i] = 0.;
  }

  if (fPartVarName[1] == "y") {
    //    H5PartReadDataFloat64(fH5File, "y", y);
  } else {
    for (unsigned long i = 0; i < n; i++)
      y[i] = 0.;
  }

  if (fPartVarName[2] == "z") {
    //    H5PartReadDataFloat64(fH5File, "z", z);
  } else {
    for (unsigned long i = 0; i < n; i++)
      z[i] = 0.;
  }

  if (fPartVarName[4] == "px") {
    //    H5PartReadDataFloat64(fH5File, "px", px);
  } else {
    for (unsigned long i = 0; i < n; i++)
      px[i] = 0.;
  }

  if (fPartVarName[5] == "py") {
    //    H5PartReadDataFloat64(fH5File, "py", py);
  } else {
    for (unsigned long i = 0; i < n; i++)
      py[i] = 0.;
  }

  if (fPartVarName[6] == "pz") {
    //    H5PartReadDataFloat64(fH5File, "pz", pz);
  } else {
    for (unsigned long i = 0; i < n; i++)
      pz[i] = 0.;
  }

  if (fPartVarName[3] == "phiz") {
    //    H5PartReadDataFloat64(fH5File, "phiz", phiz);
  } else {
    for (unsigned long i = 0; i < n; i++)
      phiz[i] = 0.;
  }

  if (fPartVarName[7] == "enez") {
    //    H5PartReadDataFloat64(fH5File, "enez", enez);
  } else {
    for (unsigned long i = 0; i < n; i++)
      enez[i] = 0.;
  }

  if (fPartVarName[8] == "id") {
    //    H5PartReadDataInt64(fH5File, "id", id);
  } else {
    for (unsigned long i = 0; i < n; i++)
      id[i] = 0;
  }

  for (unsigned long i = 0; i < n; i++) {
    // subsampling: only fill every n-th particle into ntuple:
    if (!(i % fSubsampRate)) {
      static_cast<TNtupleD*>(fNtuple->At(step))->
          Fill(x[i], y[i], z[i], phiz[i], px[i], py[i], pz[i], enez[i],
               static_cast<double>(id[i]));
    }
  }

  delete[]x;
  delete[]y;
  delete[]z;
  delete[]px;
  delete[]py;
  delete[]pz;
  delete[]id;
  delete[]phiz;
  delete[]enez;

  return (0);

}

TH1F *TH5Dataset::Histo1d(const TString * varName, 
			  const Int_t step,
                          const Double_t min, 
			  const Double_t max,
                          const Int_t nBin, 
			  const TCut cut, 
			  const Bool_t keep)
{
  TString vname = *varName;
  TNtupleD *nt = GetNtuple(step);
  TString hname = vname + TString(" distribution");

  // delete previous histogram to avoid memory leak
  TH1F *histVar = (TH1F *) gROOT->FindObject("histVar");
  if (histVar)
    delete histVar;

  TString histName("histVar");
  if (keep) {                   // keep histogram in memory: create a unique name for it
    histName += fPlotCount++;
  }

  histVar = new TH1F(histName.Data(), hname, nBin, min, max);
  histVar->SetXTitle(vname);
  histVar->SetYTitle("entries");

  if (nt)
    nt->Draw(vname + TString(">>") + histName);

  return (histVar);
}

TH1F *TH5Dataset::Histo1d(const char *varName, 
			  const Int_t step,
                          const Double_t min, 
			  const Double_t max,
                          const Int_t nBin, 
			  const TCut cut)
{
  TString vname(varName);
  return (Histo1d(&vname, step, min, max, nBin, cut, kTRUE));        // kTRUE: keep histogram in memory
}

TH2F *TH5Dataset::Histo2d(const TString * varNameX,
                          const TString * varNameY, 
			  const Int_t step,
                          const Double_t minX, 
			  const Double_t maxX,
                          const Double_t minY, 
			  const Double_t maxY,
                          const Int_t nBinX, 
			  const Int_t nBinY, 
			  const TCut cut,
                          const Bool_t keep)
{
  TString vxname = *varNameX;
  TString vyname = *varNameY;
  TNtupleD *nt = GetNtuple(step);
  TString hname =
      vxname + TString("-") + vyname + TString(" distribution");

  // delete previous histogram to avoid memory leak
  TH2F *histXY = static_cast<TH2F*>(gROOT->FindObject("histXY"));
  if (histXY)
    delete histXY;

  TString histName("histXY");
  if (keep) {                   // keep histogram in memory: create a unique name for it
    histName += fPlotCount++;
  }

  histXY =
      new TH2F(histName.Data(), hname, nBinX, minX, maxX, nBinY, minY,
               maxY);
  histXY->SetXTitle(vxname);
  histXY->SetYTitle(vyname);

  if (nt)
    nt->Draw(vyname + TString(":") + vxname + TString(">>") + histName);

  return (histXY);
}

TH2F *TH5Dataset::Histo2d(const char *varNameX, 
			  const char *varNameY,
                          const Int_t step, 
			  const Double_t minX,
                          const Double_t maxX, 
			  const Double_t minY,
                          const Double_t maxY, 
			  const Int_t nBinX,
                          const Int_t nBinY, 
			  const TCut cut)
{
  TString vxname(varNameX);
  TString vyname(varNameY);
  return (Histo2d
          (&vxname, &vyname, step, minX, maxX, minY, maxY, nBinX, nBinY,
           cut, kTRUE));

}

TH1F *TH5Dataset::Scatter(const TString * varNameX,
                          const TString * varNameY, 
			  TCanvas * canv,
                          const Int_t step, 
			  const Double_t minX,
                          const Double_t maxX, 
			  const Double_t minY,
                          const Double_t maxY, 
			  const TCut cut)
{
  TString vxname = *varNameX;
  TString vyname = *varNameY;
  TNtupleD *nt = GetNtuple(step);
  TString hname =
      vxname + TString("-") + vyname + TString(" distribution");

  TH1F *scatPlot = canv->DrawFrame(minX, minY, maxX, maxY);
  scatPlot->SetXTitle(vxname);
  scatPlot->SetYTitle(vyname);

  Int_t origStyle = nt->GetMarkerStyle();
  nt->SetMarkerStyle(1);
  if (nt)
    nt->Draw(vyname + TString(":") + vxname, "", "same");
  nt->SetMarkerStyle(origStyle);

  return (scatPlot);
}

TH1F *TH5Dataset::Scatter(const char *varNameX, 
			  const char *varNameY,
                          const Int_t step,
                          const Double_t minX, 
			  const Double_t maxX,
                          const Double_t minY, 
			  const Double_t maxY,
			  const TCut cut)
{

  TString vxname(varNameX);
  TString vyname(varNameY);
  TCanvas *c1 = new TCanvas();
  return (Scatter(&vxname, &vyname, c1, step, minX, maxX, minY, maxY, cut));
}

TH3F *TH5Dataset::Histo3d(const TString * varNameX,
                          const TString * varNameY,
                          const TString * varNameZ, 
			  const Int_t step,
                          const Double_t minX, 
			  const Double_t maxX,
                          const Double_t minY, 
			  const Double_t maxY,
                          const Double_t minZ, 
			  const Double_t maxZ,
                          const Int_t nBinX, 
			  const Int_t nBinY,
                          const Int_t nBinZ, 
			  const TCut cut, 
			  const Bool_t keep)
{
  TString vxname = *varNameX;
  TString vyname = *varNameY;
  TString vzname = *varNameZ;
  TNtupleD *nt = GetNtuple(step);
  TString hname =
      vxname + TString("-") + vyname + TString("-") + vzname +
      TString(" distribution");

  // delete previous histogram to avoid memory leak
  TH3F *histXYZ = static_cast<TH3F*>(gROOT->FindObject("histXYZ"));
  if (histXYZ)
    delete histXYZ;

  TString histName("histXYZ");
  if (keep) {                   // keep histogram in memory: create a unique name for it
    histName += fPlotCount++;
  }

  histXYZ = new TH3F(histName.Data(), hname,
                     nBinX, minX, maxX, nBinY, minY, maxY, nBinZ, minZ,
                     maxZ);
  histXYZ->SetXTitle(vxname);
  histXYZ->SetYTitle(vyname);
  histXYZ->SetZTitle(vzname);

  if (nt)
    nt->Draw(vzname + TString(":") + vyname + TString(":") + vxname +
             TString(">>") + histName);

  return (histXYZ);
}

TH3F *TH5Dataset::Histo3d(const char *varNameX, 
			  const char *varNameY,
                          const char *varNameZ, 
			  const Int_t step,
                          const Double_t minX, 
			  const Double_t maxX,
                          const Double_t minY, 
			  const Double_t maxY,
                          const Double_t minZ, 
			  const Double_t maxZ,
                          const Int_t nBinX, 
			  const Int_t nBinY,
                          const Int_t nBinZ, 
			  const TCut cut)
{
  TString vxname(varNameX);
  TString vyname(varNameY);
  TString vzname(varNameZ);
  return (Histo3d
          (&vxname, &vyname, &vzname, step, minX, maxX, minY, maxY, minZ,
           maxZ, nBinX, nBinY, nBinZ, cut, kTRUE));
}

Double_t TH5Dataset::GetMinimum(const char *varName, 
				Int_t step)
{
  return (GetMinimum(TString(varName),step));
}

Double_t TH5Dataset::GetMinimum(const TString varName, 
				Int_t step)
{
  TNtupleD *nt = GetNtuple(step);
  return (nt->GetMinimum(varName));
}

Double_t TH5Dataset::GetMaximum(const char *varName, 
				Int_t step)
{
  return (GetMaximum(TString(varName),step));
}

Double_t TH5Dataset::GetMaximum(const TString varName, 
				Int_t step)
{
  TNtupleD *nt = GetNtuple(step);
  return (nt->GetMaximum(varName));
}

Double_t TH5Dataset::GetGlobalMinimum(const char *varName)
{
  return (GetGlobalMinimum(TString(varName)));
}

Double_t TH5Dataset::GetGlobalMinimum(const TString varName)
{
  Double_t min= 1.e20;
  for (Int_t step = 0; step < fNStep; ++step) 
    min = std::min(min,GetMinimum(varName,step));
  return (min);
}

Double_t TH5Dataset::GetGlobalMaximum(const char *varName)
{
  return (GetGlobalMaximum(TString(varName)));
}

Double_t TH5Dataset::GetGlobalMaximum(const TString varName)
{
  Double_t max= -1.e20;
  for (Int_t step = 0; step < fNStep; ++step) 
    max = std::max(max,GetMaximum(varName,step));
  return (max);
}

Double_t TH5Dataset::PlotRange(const TString * varName, 
			       const Int_t step,
                               Double_t & plotMin, 
			       Double_t & plotMax)
{
  TNtupleD *nt = GetNtuple(step);
  plotMin = nt->GetMinimum(*varName);
  plotMax = nt->GetMaximum(*varName);
  return (AddMargin(plotMin, plotMax));
}

Double_t TH5Dataset::Range(const TString * varName, 
			   const Int_t step,
                           Double_t & min, 
			   Double_t & max)
{
  TNtupleD *nt = GetNtuple(step);
  min = nt->GetMinimum(*varName);
  max = nt->GetMaximum(*varName);
  return (max - min);
}

Int_t TH5Dataset::GetStepVarArray(const TString stepVarName,
				  Double_t var[], 
				  const EH5VarType vtype) 
{
  // retrieve an array of variables, given a dataset;
  // pop up a progress bar if calculation takes long

  // cannot do this for multiplot variable type
  if (vtype==kPartProjAndSlice   || 
      vtype==kPartProjAndScreen  || 
      vtype==kPartSliceAndScreen || 
      vtype==kPartProjAndSliceAndScreen) { 
    h5Error(this,"GetStepVarArray", fParentFrame,
	    "Cannot return variable array for multi plot variable type!");    
    return(0);
  }

  TH5ProgressBar *progBar = NULL;
  TString start = stepVarName(0,4);

  // short cut with GetScreenProjectedStepVarArray for screen-projected vars
  // (also works without, but much slower!)
  if (vtype == kPartScreen) {
    // recover x and y variable names from stepVarName (between brackets):
    TString s(stepVarName);
    Ssiz_t L = s.First("(");
    Ssiz_t M = s.First(",");
    
    TString funName = s(0, L);
    
    TString varName1;
    TString varName2;
    if (M > 0) {
      varName1 = s(L + 1, M - L - 1);
      varName2 = s(M + 1, s.Length() - M - 2);
    } else {
      varName1 = s(L + 1, s.Length() - L - 2);
      varName2 = varName1;
    }

    if (start == TString("#mu(")) 
      return GetScreenProjectedStepVarArray(&varName1, &varName2, var, kMean);
    else if (start == TString("#sig")) 
      return GetScreenProjectedStepVarArray(&varName1, &varName2, var, kCov);
    else if (start == TString("#var"))
      return GetScreenProjectedStepVarArray(&varName1, &varName2, var, kEmit);
    else {
      h5Error(this, "GetStepVarArray", fParentFrame,"Unexptected variable name \"%s\" ",
              stepVarName.Data());
      return(0);
    }
  }

  if ( (start == TString("#mu(") || start == TString("#sig")
	//      || start == TString("#var")) && fParentFrame) {
      || start == TString("#var")) ) {
    // if time to compute one step exceeds 2 millisec, show a progress bar:
    TTime t0 = gSystem->Now();
    GetStepVar(&stepVarName, fNStep-1, vtype);
    TTime t1 = gSystem->Now();
    TTime dt = t1;
    dt -= t0;

    if (atoi(dt.AsString()) > 2) {  // variable retrieval takes a long time
      TString progBarMsg = TString("Computing ");
      if (vtype == kPart)
        progBarMsg += TString("projected ");
      if (vtype == kPartSlice)
        progBarMsg += TString("slice ");
      if (vtype == kPartScreen)
        progBarMsg += TString("screen ");
      if (start == TString("#mu(")) {
        progBarMsg += TString("means for file ");
      } else {
        progBarMsg += TString("covariances for file ");
      }
      progBarMsg += GetFilename();
      progBarMsg += TString("...");
      for (Int_t step = 0; step < fNStep; step++) {

	cout << setw(fWidthStep) << step+1 << "/" << setw(fWidthStep) << fNStep << flush;

        var[step] = GetStepVar(&stepVarName, step, vtype);

        // add an option to display progressbar in interactive environment
	if (fParentFrame) {
	  if (0 == step) {
	    progBar = new
              TH5ProgressBar(gClient->GetRoot(), this, 400, 200,
                             progBarMsg.Data());
	    progBar->Reset();
	  }
	  progBar->IncrementStep(step, fNStep);
	  if (progBar->ReceivedCancel()) {
	    cout << endl;
	    return (0);
	  }
	  gSystem->ProcessEvents();
	}

	for (Int_t j=0; j<fWidthStep*2+1; ++j) cout << "\b";
	cout << flush;
      }

      if (progBar) {
	delete progBar;
	progBar = NULL;
      }
 
    } else {    // variable retrieval is quick, show no progress bar

      for (Int_t step = 0; step < fNStep; step++) {
        var[step] = GetStepVar(&stepVarName, step, vtype);
      }
    }
  } else {  // retrieve simple variables without showing a progress bar
    for (Int_t step = 0; step < fNStep; step++) {
      var[step] = GetStepVar(&stepVarName, step, vtype);
    }
  }
  return (1);
}

TGraph *TH5Dataset::GetGraphXY(const char *varNameX, 
			       const char *varNameY,
                               const char *coordY,
			       const Double_t scaleY)
{
  /*
     Usage of TH5Dataset::GetGraphXY
     -------------------------------

     GetGraphXY returns a TGraph containing the values of two specified step values,
     which are either stored as step variables in the file, or computed from the 
     particle distributions.

     Examples:

     TH5Dataset* data = new TH5Dataset("example.h5");

     1) Retrieve two scalar step variables from the data file, e.g. SPOS and ENERGY:

     TGraph* gr = data->GetGraphXY("SPOS","ENERGY");

     2) Retrieve a scalar and a vector step variables from the data file, e.g. SPOS and 
     RMSP, y-comp.:

     TGraph* gr = data->GetGraphXY("SPOS","RMSP","y");

     3) Retrieve two vector step variables from the data file, e.g. RMSP, x and y components:

     TGraph* gr = data->GetGraphXY("RMSP","x","RMSP","y");

     4) Retrieve a scalar and a particle-derived step variable from the data file, e.g. SPOS
     and emittance from x and px distributions:

     TGraph* gr = data->GetGraphXY("SPOS","emittance(x,px)");

     the argument is case-insensitive and rather tolerant otherwise, for instance,

     TGraph* gr = data->GetGraphXY("SPOS","EMIT(x,px)");

     will produce the same TGraph. Other possibilities are "mean(x)" and "covariance(x,px)" 
     etc.

     5) Retrieve a scalar and a slice particle-derived step variable from the data file, 
     e.g. SPOS and slice emittance from x and px distributions:

     TGraph* gr = data->GetGraphXY("SPOS","emittance(x,px)","slice");

     6) Retrieve two slice particle-derived step variables from the data file, e.g.
     slice x- and y-emittances:

     TGraph* gr = data->GetGraphXY("emit(x,px)","slice","emit(y,py)","slice");

     or any combination of the above!

   */

  // overload to GetGraphXY for interactive use
  return GetGraphXY(TString(varNameX), TString(""), TString(varNameY),
                    TString(coordY), scaleY);
}

TGraph *TH5Dataset::GetGraphXY(const char *varNameX, 
			       const char *coordX,
                               const char *varNameY, 
			       const char *coordY,
			       const Double_t scaleY)
{
  return GetGraphXY(TString(varNameX), TString(coordX), 1.0, 
		    TString(varNameY), TString(coordY),scaleY);
}

TGraph *TH5Dataset::GetGraphXY(const char *varNameX, 
			       const char *coordX,
			       const Double_t scaleX,
                               const char *varNameY, 
			       const char *coordY,
			       const Double_t scaleY)
{
  return GetGraphXY(TString(varNameX), TString(coordX), scaleX, 
		    TString(varNameY), TString(coordY), scaleY);
}

TGraph *TH5Dataset::GetGraphXY(const TString vnamex, 
			       const TString crdx,
                               const TString vnamey, 
			       const TString crdy,
			       const Double_t scaleY)
{
  return GetGraphXY(vnamex, crdx, 1.,vnamey, crdy,scaleY);
}

TGraph *TH5Dataset::GetGraphXY(TString vnamex, 
			       const TString crdx,
			       const Double_t scaleX,
                               TString vnamey, 
			       const TString crdy,
			       const Double_t scaleY)
{

  Double_t varX[fNStep];
  Double_t varY[fNStep];

  EH5VarType vtypex(kScalar);
  if (crdx == TString("x") || crdx == TString("X"))
    vtypex = kVectorX;
  if (crdx == TString("y") || crdx == TString("Y"))
    vtypex = kVectorY;
  if (crdx == TString("z") || crdx == TString("Z"))
    vtypex = kVectorZ;

  EH5VarType vtypey(kScalar);
  if (crdy == TString("x") || crdy == TString("X"))
    vtypey = kVectorX;
  if (crdy == TString("y") || crdy == TString("Y"))
    vtypey = kVectorY;
  if (crdy == TString("z") || crdy == TString("Z"))
    vtypey = kVectorZ;

  // analyze variable names
  if (DecodeString(vnamex)) {
    if (crdx.Contains("slic", TString::kIgnoreCase)) 
      vtypex = kPartSlice;
    else  if (crdx.Contains("scree", TString::kIgnoreCase)) 
      vtypex = kPartScreen;
    else 
      vtypex = kPart;
  }
  if (DecodeString(vnamey)) {
    if (crdy.Contains("slic", TString::kIgnoreCase)) 
      vtypey = kPartSlice;
    else  if (crdy.Contains("scree", TString::kIgnoreCase)) 
      vtypey = kPartScreen;
    else 
      vtypey = kPart;
  }

  if (!GetStepVarArray(vnamex, varX, vtypex)) return (0);
  if (!GetStepVarArray(vnamey, varY, vtypey)) return (0);

  if (TMath::Abs(scaleX-1.)>1.e-20)
    for (Int_t step = 0; step < fNStep; ++step) varX[step] *= scaleX;

  if (TMath::Abs(scaleY-1.)>1.e-20)
    for (Int_t step = 0; step < fNStep; ++step) varY[step] *= scaleY;

  TGraph *graphXY = new TGraph(fNStep, varX, varY);
  return graphXY;

}


Int_t TH5Dataset::DecodeString(TString & s)
{
  // try to decode intention of user when producing a particle line plot,
  // anything containing eps, emit,... --> emittance plot
  // anything containing cov, sig...   --> sigma (covariance) plot
  // anything containing mu, mean...   --> mean plot

  Int_t f = s.First('(');
  if (f > 0) {
    TString firstPart = s(0, f);
    TString lastPart = s(f, s.Length());

    if (firstPart.Contains("emit", TString::kIgnoreCase)
        || firstPart.Contains("eps", TString::kIgnoreCase))
      s = TString("#varepsilon") + lastPart;
    if (firstPart.Contains("cov", TString::kIgnoreCase) ||
	firstPart.Contains("sig", TString::kIgnoreCase))
      s = TString("#sigma") + lastPart;
    if (firstPart.Contains("mu", TString::kIgnoreCase) ||
        firstPart.Contains("mean", TString::kIgnoreCase))
      s = TString("#mu") + lastPart;
    return (1);
  }
  return (0);
}


TH1F *TH5Dataset::HistoSliceEmit(const TString* varNameX,
				 const TString* varNameY,
				 const Int_t step,
				 const Bool_t keep)
// return a histogram containing slice variables along particle bunch
{
  // use internal values for slicelength = slice spacing, unit and Gauss sigma...
  return HistoSliceEmit(varNameX,varNameY,step,fSliceLength,fSliceLength,fSliceLengthUnit,fSliceSigma,keep);
}


TH1F *TH5Dataset::HistoSliceEmit(const TString* varNameX,
				 const TString* varNameY,
				 const Int_t step,
				 const Double_t sliceLength,  // in mm or ps according to sliceUnit!!
				 const Double_t sliceSpacing, // in mm or ps, according to sliceUnit!!
				 const EH5SliceUnit sliceUnit,
				 const Double_t gaussSigma,  // if = 0, uniform weighting
				 const Bool_t keep)
// return a histogram containing slice variables along particle bunch
{
  TString vxname = *varNameX;
  TString vyname = *varNameY;
  TNtupleD *nt = GetNtuple(step);
  TString hname =
      vxname + TString("-") + vyname + TString(" slice emittance distribution");

  // delete previous histogram to avoid memory leak
  TH1F *histSliceEmit = (TH1F *) gROOT->FindObject("histSliceEmit");
  if (histSliceEmit) delete histSliceEmit;

  TString histName("histSliceEmit");
  if (keep) histName += fPlotCount++;  // keep histogram in memory: create a unique name for it

  TString svar("z");
  Double_t min = nt->GetMinimum(svar);
  Double_t max = nt->GetMaximum(svar);

  const Double_t mm = 1.e-3;
  Double_t length(sliceLength);
  Double_t space(sliceSpacing);
  if (space<=0.) space = length;
  if (sliceUnit == kps) {
    length = ps2mm(length,step);
    space = ps2mm(length,step);
  }
//   if (bGaussian) {    // think about requiring length < 6*sigma or so...
//     length *= 6.;     // (but that requirement could be earlier
//   }
  length *= mm;
  space *= mm;

//   Double_t sx(0.),sy(0.),sxx(0.),syy(0.),sxy(0.);
  Double_t spos = GetSpos(step);

  Int_t nBinUpper = static_cast<Int_t>(1.2*ceil((max-spos)/space));
  Int_t nBinLower = static_cast<Int_t>(1.2*ceil((spos-min)/space));
  Int_t nBin = nBinLower+nBinUpper;
  Double_t histMin = spos-nBinLower*space;
  Double_t histMax = spos+nBinUpper*space;

  histSliceEmit = new TH1F(histName.Data(), hname, nBin, histMin, histMax);
  histSliceEmit->SetXTitle("z");
  TString yTitle("#varepsilon_{slice}(");
  yTitle += *varNameX;
  yTitle += ",";
  yTitle += *varNameY;
  yTitle += ")";
  histSliceEmit->SetYTitle(yTitle);  

  // step through histogram bins and fill them
  Double_t s=0.;
  cout << "computing slice emittances for step " << step << endl;
  for (Int_t iBin = 1; iBin <= nBin; iBin++) {
    // get slice emittance
    s = histSliceEmit->GetBinCenter(iBin);
    Double_t emit = GetSliceEmit(varNameX,varNameY,step,s,length,gaussSigma);
    histSliceEmit->SetBinContent(iBin,emit);
    // histSliceEmit->SetBinError(iBin,0.);  // this screws up bar option...
  }

  return histSliceEmit;
}

TH1F *TH5Dataset::HistoSliceEmit(const char* varNameX,
				 const char* varNameY,
				 const Int_t step,
				 const Double_t sliceLength,  // in mm or ps according to sliceUnit!!
				 const Double_t sliceSpacing, // in mm or ps, according to sliceUnit!!
				 const EH5SliceUnit sliceUnit,
				 const Double_t gaussSigma)  // if = 0, uniform weighting
{
  TString vxname(varNameX);
  TString vyname(varNameY);
  Double_t sl = fSliceLength;
  if (sliceLength > 0.) sl = sliceLength;
  return (HistoSliceEmit(&vxname, &vyname, step, sl, sliceSpacing, sliceUnit, gaussSigma, kTRUE));
}


Double_t TH5Dataset::GetSliceEmit(const TString* varNameX,
				  const TString* varNameY,
				  const Int_t step,
				  const Double_t s0,          // s position of slice
				  const Double_t sliceLength, // in m!
				  const Double_t gaussSigma)  // if = 0, uniform weighting
// within a timestep (step), return the slice emittance at arbitrary position s0
{
  Double_t sliceEmit(0.);
  Double_t halfLength = 0.5*sliceLength;
  //construct selection string
  TString sliceVar("z");
  TString sel("(");
  sel += sliceVar;
  sel += TString(">");
  sel += s0-halfLength;
  sel += TString(") && (");
  sel += sliceVar;
  sel += TString("<");
  sel += s0+halfLength;
  sel += TString(")");

  TCut sliceCut(sel.Data());

// construct expression to get weight information as well as x and px
// gaussian weighted or uniform weighted
  TString exp;

  if (gaussSigma>0.) {
    exp = TString("TMath::Gaus(z,");
    exp += s0;
    exp += ",";
    exp += gaussSigma;
    exp += "):";
  } else {
    exp = TString("1:");
  }
  
  exp += *varNameX;
  exp += ":";
  exp += *varNameY;

  TNtupleD *nt = GetNtuple(step);

  nt->Draw(exp, sliceCut, "goff");
  Int_t nPoints = nt->GetSelectedRows();

  if (nPoints > 0) {
    Double_t *v1 = nt->GetV1();
    Double_t *v2 = nt->GetV2();
    Double_t *v3 = nt->GetV3();

    Double_t wtot = 0.;
    Double_t sXX = 0.;
    Double_t sYY = 0.;
    Double_t sXY = 0.;

    for (Int_t i = 0; i < nPoints; i++) {
      wtot += *(v1 + i);
      sXX += (*(v1 + i)) * (*(v2 + i)) * (*(v2 + i));
      sYY += (*(v1 + i)) * (*(v3 + i)) * (*(v3 + i));
      sXY += (*(v1 + i)) * (*(v2 + i)) * (*(v3 + i));
    }
    sliceEmit = sXX * sYY - sXY * sXY;
    if (sliceEmit < 0) {
      if (TMath::Abs(sliceEmit) < 1e-20) {
        sliceEmit = 0.;
      } else {
        cout << "error: negative slice emittance: " << sliceEmit << endl;
        return 0.;
      }
    }
    if (wtot <= 0) {
      cout << "error: weight less than 0" << endl;
      return 0.;
    }
    sliceEmit = sqrt(sliceEmit) / wtot;
  }

  return sliceEmit;
}

Double_t TH5Dataset::GetGaussSliceStepVar(const TString* varName,
					  const Int_t step,
					  const EH5PartVarType varType)
{
  return(GetGaussSliceStepVar(varName,varName,step,varType));
}

Double_t TH5Dataset::GetGaussSliceStepVar(const TString* varNameX,
					  const TString* varNameY,
					  const Int_t step,
					  const EH5PartVarType pvtype)
// return slice variable with Gaussian weighting according to slice options
{
  Int_t ix = GetPartVarIndex(varNameX);
  Int_t iy = GetPartVarIndex(varNameY);

  Double_t v0(EMPTY);
  if (pvtype == kMean) 
    v0 = fMeanSl->GetBinContent(step, ix);
  else if (pvtype == kCov) 
    v0 = fSigmaSl->GetBinContent(step, ix, iy);
  else if (pvtype==kEmit) {
    Double_t covXX = GetGaussSliceStepVar(varNameX,varNameX,step,kCov);
    Double_t covYY = GetGaussSliceStepVar(varNameY,varNameY,step,kCov);
    Double_t covXY = GetGaussSliceStepVar(varNameX,varNameY,step,kCov);
    v0 = covXX*covYY-covXY*covXY;
    v0 = sqrt(v0);
    return(v0);
  } else 
    cout << "GetGaussSliceStepVar: unknown part variable option" << endl;

  if (v0 == EMPTY) { // not computed yet
    TCut sliceCut = *(GetSliceCut(step));
    
    TString vxname = *varNameX;
    TString vyname = *varNameY;
    // compute one more paired variable even if not requested, to save time
    // for example: if only x requested, compute x and px
    if (ix == iy) {
      iy = (ix < 4) ? (ix + 4) : (ix - 4);
      vyname = GetPartVarName(iy);
    }
    Double_t sigmaZ = fSliceSigma;
    if (fSliceSigmaUnit == kps) sigmaZ = ps2mm(sigmaZ,step);
    const Double_t mm = 1.0e-3; // convert mm to m
    sigmaZ *= mm;

    Double_t center = (*fSlicePos)[step]; 
    // construct expression to get weight information as well as x and y
    TString exp;
    exp = TString("TMath::Gaus(z,");
    exp += center;
    exp += ",";
    exp += sigmaZ;
    exp += "):";
    exp += vxname;
    exp += ":";
    exp += vyname;
    //  cout << "string exp: " << exp << endl; // typical: TMath::Gaus(z,0.0127508831,   0.0003):x:px

    Double_t meanX(0.), meanY(0.), covXY(0.), covXX(0.), covYY(0.);

    TNtupleD *nt = GetNtuple(step);

    nt->Draw(exp, sliceCut, "goff");
    Int_t nParts = nt->GetSelectedRows();

    if (nParts > 0) {
      Double_t *v1 = nt->GetV1(); // weights
      Double_t *v2 = nt->GetV2(); // X values
      Double_t *v3 = nt->GetV3(); // Y values

      Double_t totalWeight(0.);
      Double_t sumX(0.);
      Double_t sumY(0.);
      Double_t sumXX(0.);
      Double_t sumYY(0.);
      Double_t sumXY(0.);

      for (Int_t i = 0; i < nParts; i++) {
        totalWeight += *(v1 + i);
        sumX  += (*(v1 + i)) * (*(v2 + i));                 // w*x
        sumY  += (*(v1 + i)) * (*(v3 + i));                 // w*y
        sumXX += (*(v1 + i)) * (*(v2 + i)) * (*(v2 + i));   // w*x*x
        sumYY += (*(v1 + i)) * (*(v3 + i)) * (*(v3 + i));   // w*y*y
        sumXY += (*(v1 + i)) * (*(v2 + i)) * (*(v3 + i));   // w*x*y
      }
      if (totalWeight <= 0.) totalWeight = 1.e20;
      meanX = sumX / totalWeight;
      meanY = sumY / totalWeight;
    
      // definition of weighted covariance according to GNU Scientific Library
      // http://www.gnu.org/software/gsl/manual
      Double_t wxy(0.),ww(0.),wxx(0.),wyy(0.);
      for (Int_t i = 0; i < nParts; i++) {
        Double_t w  = (*(v1+i)) / totalWeight;
        Double_t dx = (*(v2+i)) - meanX;
        Double_t dy = (*(v3+i)) - meanY;
        wxx += w * dx * dx;
        wyy += w * dy * dy;
        wxy += w * dx * dy;
        ww += w * w;
      }
      covXX = wxx / (1-ww);
      covYY = wyy / (1-ww);
      covXY = wxy / (1-ww);
    }

    fMeanSl->SetBinContent(step, ix, meanX);
    fMeanSl->SetBinContent(step, iy, meanY);
    fSigmaSl->SetBinContent(step, ix, iy, covXY);
    fSigmaSl->SetBinContent(step, ix, ix, covXX);
    fSigmaSl->SetBinContent(step, iy, iy, covYY);

    return(GetGaussSliceStepVar(varNameX,varNameY,step,pvtype));
  }
  return v0;
}

Double_t TH5Dataset::GetScreenProjectedStepVar(const TString * varName,
					       const Int_t step,
					       const EH5PartVarType pvtype)
{
  return(GetScreenProjectedStepVar(varName,varName,step,pvtype));
}

Double_t TH5Dataset::GetScreenProjectedStepVar(const TString * varNameX,
					       const TString * varNameY,
					       const Int_t step,
					       const EH5PartVarType pvtype)
{
  // check if element is already in array, else compute from ntuple:
  Int_t ix0 = GetPartVarIndex(varNameX);
  Int_t iy0 = GetPartVarIndex(varNameY);
  Double_t v0;
  
  if (pvtype==kMean) 
    v0 = fMeanSc->GetBinContent(step, ix0);
  else if (pvtype==kCov) {
    // return square root of covariance
    v0 = fSigmaSc->GetBinContent(step, ix0, iy0);
    if (v0!=EMPTY) v0 = sqrt(TMath::Abs(v0));
  }
  else if (pvtype==kEmit) {
    Double_t covXX = GetScreenProjectedStepVar(varNameX,varNameX,step,kCov);
    Double_t covYY = GetScreenProjectedStepVar(varNameY,varNameY,step,kCov);
    Double_t covXY = GetScreenProjectedStepVar(varNameX,varNameY,step,kCov);
    v0 = covXX*covYY-covXY*covXY;
    v0 = sqrt(v0);
    return(v0);
  } else 
    cout << "GetScreenProjectedStepVar: unknown part variable option" << endl;

  if (v0==EMPTY) { // not yet computed, need to do it here

    TString vxname = *varNameX;
    TString vyname = *varNameY;
    Int_t ix(ix0);
    Int_t iy(iy0);
    // compute one more paired variable even if not requested, to save time
    // for example: if only x requested, compute x and px
    if (ix == iy) {
      iy = (ix < 4) ? (ix + 4) : (ix - 4);
      vyname = GetPartVarName(iy);
    }

    Double_t sumX(0.),sumY(0.),sumXX(0.),sumYY(0.),sumXY(0.);
    TString varNameZ("z");
    
    Double_t zStart = GetSpos(step);
    Double_t zEnd = (step == fNStep - 1) ? zStart*2-GetSpos(fNStep-2) : GetSpos(step+1);
    
    TNtupleD *nt = GetNtuple(step);
    Int_t nParts0 = nt->GetEntries();
    Double_t smin = nt->GetMinimum(varNameZ);
    Double_t smax = nt->GetMaximum(varNameZ);
    Double_t bunchLength = smax-smin;
    
    Int_t iStartStep = GetStep(zStart-bunchLength) ; // get first step that includes particles for this slice
    if (iStartStep < 0) iStartStep = 0;
    Int_t iEndStep = GetStep(zStart+bunchLength) ; // get last step that includes particles for this slice
    if (iEndStep >= fNStep) iEndStep = fNStep - 1;
    
    cout << iStartStep << "\t" << iEndStep << endl;
    
    Int_t i1 = (iStartStep == 0)? 0: iStartStep-1;
    nt = GetNtuple(i1);
    smax = nt->GetMaximum(varNameZ);
    Int_t i2 = (iEndStep == fNStep - 1)? fNStep-1: iEndStep+1;
    nt = GetNtuple(i2);
    smin = nt->GetMinimum(varNameZ);
    //cout << "smax at iStart-1: " << smax << " smin at iEnd+1: " << smin << endl;
    
    //if (iEndStep < fNStep - 1) {
    //    nt = GetNtuple(iEndStep+1);
    //    smax = nt->GetMaximum(varNameZ);
    //    iEndStep = GetStep(smax);
    //  } 
    //cout << iStartStep << "\t" << iEndStep << endl;
    
    Int_t nPartsSum(0);
    TString sel("(");
    sel += varNameZ;;
    sel += TString(">=");
    sel += zStart;
    sel += TString(") && (");
    sel += varNameZ;;
    sel += TString("<");
    sel += zEnd;
    sel += TString(")");
    TCut sliceCut(sel.Data());

    // loop over relevant time steps and sum up the quantities:
    for (Int_t istep = iStartStep; istep <= iEndStep; ++istep) { 
      //cout << "\t step " << step << ":spos: " << (*fSpos)[istep] << " is included" << endl;
      TNtupleD *nt = GetNtuple(istep);
      nt->Draw(vxname + TString(":") + vyname, sliceCut, "goff");
      Int_t nPartsInSlice = nt->GetSelectedRows();
      cout << "particle number in this slice: " << nPartsInSlice << endl;
      nPartsSum += nPartsInSlice;
      if (nPartsInSlice > 0) { 
	Double_t *v1 = nt->GetV1();
	Double_t *v2 = nt->GetV2();
	for (Int_t i = 0; i < nPartsInSlice; i++) { //sum all particles in this slice
          sumX  += (*(v1 + i));
          sumY  += (*(v2 + i));
	  sumXX += (*(v1 + i)) * (*(v1 + i));
	  sumXY += (*(v1 + i)) * (*(v2 + i));
	  sumYY += (*(v2 + i)) * (*(v2 + i));
	} //end of loop of particles
      } // end of if (nParts > 0) 
    } //end of loop for each step
    
    if ( TMath::Abs(nParts0/nPartsSum - 1.) > 0.001 )
      cout << "Warning: inconsistent particle numbers for step "  
	   << right << setw(fWidthStep) << step
	   << " (spos = " << scientific << setprecision(3) << setw(9) 
	   << (*fSpos)[step]*1.e3
	   << " mm): " 
	   << nParts0 << " vs. " << nPartsSum << endl;

    Double_t meanX = 0.;
    Double_t meanY = 0.;
    Double_t covXX = 0.;
    Double_t covYY = 0.;
    Double_t covXY = 0.;

    if (nPartsSum > 0) {

      meanX = sumX/nPartsSum;    
      meanY = sumY/nPartsSum;    
      
      covXY = sumXY - sumX*sumY/nPartsSum; 
      covXY /= nPartsSum; 
      covXX = sumXX - sumX*sumX/nPartsSum; 
      covXX /= nPartsSum; 
      covYY = sumYY - sumY*sumY/nPartsSum; 
      covYY /= nPartsSum; 
      
    }

    Double_t eps = covXX * covYY - covXY * covXY;
    eps = sqrt(eps);

    fMeanSc->SetBinContent(step, ix, meanX);
    fMeanSc->SetBinContent(step, iy, meanY);
    fSigmaSc->SetBinContent(step, ix, ix, covXX);
    fSigmaSc->SetBinContent(step, iy, iy, covYY);
    fSigmaSc->SetBinContent(step, ix, iy, covXY);

    return(GetScreenProjectedStepVar(varNameX,varNameY,step,pvtype));

  }
  return(v0);
}


Int_t TH5Dataset::GetScreenProjectedStepVarArray(const TString * varNameX,
						 const TString * varNameY,
						 Double_t var[], 
						 const EH5PartVarType pvtype)
{
  Int_t ix0 = GetPartVarIndex(varNameX);
  Int_t iy0 = GetPartVarIndex(varNameY);
  
  Double_t v0(EMPTY);
  if (pvtype == kMean) 
    v0 = fMeanSc->GetBinContent(0, ix0);
  else if (pvtype == kCov) 
    v0 = fSigmaSc->GetBinContent(0, ix0, iy0);
  else if (pvtype == kEmit) {
    // emittance: check that all relevant arrays are filled:
    Double_t cxx0 = fSigmaSc->GetBinContent(0, ix0, ix0);
    Double_t cyy0 = fSigmaSc->GetBinContent(0, iy0, iy0);
    Double_t cxy0 = fSigmaSc->GetBinContent(0, ix0, iy0);
    if (cxx0==EMPTY||cyy0==EMPTY||cxy0==EMPTY) v0 = EMPTY;
    else v0 = sqrt(cxx0*cyy0-cxy0*cxy0);
  } else 
    cout << "GetScreenProjectedStepVarArray: unknown part variable option" << endl;

  if (v0 == EMPTY) { // not computed yet (at least for step 0...)
    TString vxname = *varNameX;
    TString vyname = *varNameY;

    Int_t ix(ix0);
    Int_t iy(iy0);
    // compute one more paired variable even if not requested, to save time
    // for example: if only x requested, compute x and px
    if (ix == iy) {
      iy = (ix < 4) ? (ix + 4) : (ix - 4);
      vyname = GetPartVarName(iy);
    }

    TArrayD *zpos = new TArrayD(fNStep+1);
    TArrayD *sumX = new TArrayD(fNStep);
    TArrayD *sumY = new TArrayD(fNStep);
    TArrayD *sumXX = new TArrayD(fNStep);
    TArrayD *sumYY = new TArrayD(fNStep);
    TArrayD *sumXY = new TArrayD(fNStep);
    TArrayI *nPartsSum = new TArrayI(fNStep);

    TString varNameZ("z");
    for (Int_t step = 0; step < fNStep; ++step) { //initialize the sum value
      (*zpos)[step] = (*fSpos)[step];
      (*sumX)[step] = 0.;
      (*sumY)[step] = 0.;
      (*sumXX)[step] = 0.;
      (*sumYY)[step] = 0.;
      (*sumXY)[step] = 0.;
      (*nPartsSum)[step] = 0;
    }
    (*zpos)[fNStep] = (*fSpos)[fNStep-1]*2 - (*fSpos)[fNStep-2]; //extend to calculate the last slice

    TNtupleD *nt = GetNtuple(0);
    Double_t smin = nt->GetMinimum(varNameZ);
    Double_t smax = nt->GetMaximum(varNameZ);
    Int_t iStart = GetStep(smin);
    if (iStart < 0) iStart = 0;
    Int_t iEnd = GetStep(smax);
    if (iEnd-iStart < 10) { // time step is big for computing screen projected values
      h5Warning(this, "GetScreenProjectedStepVarArray",	fParentFrame,
                "Dude, there are only %d time steps per bunch length, screen projection may be inaccurate!",
		iEnd-iStart);
    } else if (iEnd-iStart < 5) {
      h5Error(this, "GetScreenProjectedStepVarArray",fParentFrame,
	      "Dude, there are not enough time steps (%d per bunch length) to do a screen projection!",iEnd-iStart);
      return(0);
    }
    TH5ProgressBar *progBar = NULL;
    TString progBarMsg = TString("Computing screen-projected means and covariances (");
    progBarMsg += vxname;
    progBarMsg += TString(",");
    progBarMsg += vyname;
    progBarMsg += TString(") for file ");
    progBarMsg += GetFilename();
    progBarMsg += TString("...");

    cout << progBarMsg << endl; 
    for (Int_t step = 0; step < fNStep; ++step) { // loop over each step
      cout << setw(fWidthStep) << step+1 << "/" << setw(fWidthStep) << fNStep << flush;
      
      if (fParentFrame) {
	if (0 == step) {
          progBar = new
	    TH5ProgressBar(gClient->GetRoot(), this, 400, 200,
			   progBarMsg.Data());
          progBar->Reset();
        }
        progBar->IncrementStep(step, fNStep);
        if (progBar->ReceivedCancel()) {
	  cout << endl;
	  return (0);
	}
        gSystem->ProcessEvents();
      }
      
      TNtupleD *nt = GetNtuple(step);
      Double_t smin = nt->GetMinimum(varNameZ);
      Double_t smax = nt->GetMaximum(varNameZ);
      Int_t iStart = GetStep(smin);
      if (iStart < 0) iStart = 0;
      Int_t iEnd = GetStep(smax); 

      for (Int_t iStep = iStart; iStep <= iEnd; ++iStep) { // loop over whole bunch
        TString sel("(");
        sel += varNameZ;;
        sel += TString(">=");
        sel += (*zpos)[iStep];
        sel += TString(") && (");
        sel += varNameZ;;
        sel += TString("<");
        sel += (*zpos)[iStep+1];
        sel += TString(")");

        TCut sliceCut(sel.Data());

        nt->Draw(vxname + TString(":") + vyname, sliceCut, "goff");
        Int_t np = nt->GetSelectedRows();
        (*nPartsSum)[iStep] += np;
        if (np > 0) { 
          Double_t *v1 = nt->GetV1();
          Double_t *v2 = nt->GetV2();
          for (Int_t i = 0; i < np; i++) { //sum all particles in this slice
            (*sumX)[iStep] += (*(v1 + i));
            (*sumY)[iStep] += (*(v2 + i));
            (*sumXX)[iStep] += (*(v1 + i)) * (*(v1 + i));
            (*sumYY)[iStep] += (*(v2 + i)) * (*(v2 + i));
            (*sumXY)[iStep] += (*(v1 + i)) * (*(v2 + i));
          } //end of loop of particles
        } // end of if (np > 0) 
      } // end of loop for whole bunch
      for (Int_t j=0; j<fWidthStep*2+1; ++j) cout << "\b";
      cout << flush;
    } //end of loop for each step

    if (progBar) {
      delete progBar;
      progBar = NULL;
    }

    // collect final results
    for (Int_t step = 0; step < fNStep; ++step) { // loop over each step
      Double_t nPartsFromSum = (*nPartsSum)[step]; 
      // give a warning if the summed particle number diff much (.1 percent) from the total number
      Double_t nPartsFromStep = GetNtuple(step)->GetEntries();       
      if ( TMath::Abs(nPartsFromSum/nPartsFromStep - 1) > 0.001 )
	cout << "Warning: inconsistent particle numbers for step "  
	     << right << setw(fWidthStep) << step
	     << " (spos = " << scientific << setprecision(3) << setw(9) 
	     << (*fSpos)[step]*1.e3
	     << " mm): " 
	     << nPartsFromStep << " vs. " << nPartsFromSum << endl;

      Double_t meanX = 0.;
      Double_t meanY = 0.;
      Double_t covXX = 0.;
      Double_t covYY = 0.;
      Double_t covXY = 0.;

      if (nPartsFromSum > 0) {

	meanX = (*sumX)[step]/nPartsFromSum;
	meanY = (*sumY)[step]/nPartsFromSum;
      
	covXX = (*sumXX)[step] - ((*sumX)[step])*((*sumX)[step])/nPartsFromSum;
	covXX /= nPartsFromSum;
	covYY = (*sumYY)[step] - ((*sumY)[step])*((*sumY)[step])/nPartsFromSum;
        covYY /= nPartsFromSum;
	covXY = (*sumXY)[step] - ((*sumX)[step])*((*sumY)[step])/nPartsFromSum;
	covXY /= nPartsFromSum;

      }

      Double_t eps = covXX * covYY - covXY * covXY;
      eps = sqrt(eps);

      // fill arrays
      fMeanSc->SetBinContent(step, ix, meanX);
      fMeanSc->SetBinContent(step, iy, meanY);

      fSigmaSc->SetBinContent(step, ix, iy, covXY);
      fSigmaSc->SetBinContent(step, ix, ix, covXX);
      fSigmaSc->SetBinContent(step, iy, iy, covYY);

    }
    
    delete zpos; 
    delete sumX; 
    delete sumY;
    delete sumXX;
    delete sumYY;
    delete sumXY;
    delete nPartsSum;

    // arrays are filled, just call the same method again:
    return (GetScreenProjectedStepVarArray(varNameX,varNameY,var,pvtype));
    
  } else {  // relevant arrays are not empty

    for (Int_t step = 0; step < fNStep; ++step) { // loop over each step
      if (pvtype == kMean) 
        var[step] = fMeanSc->GetBinContent(step, ix0);
      else if (pvtype == kCov)
	// return square root of covariance:
        var[step] = sqrt(TMath::Abs(fSigmaSc->GetBinContent(step, ix0, iy0)));
      else if (pvtype == kEmit) {
	Double_t cxx = fSigmaSc->GetBinContent(step, ix0, ix0);
	Double_t cyy = fSigmaSc->GetBinContent(step, iy0, iy0);
	Double_t cxy = fSigmaSc->GetBinContent(step, ix0, iy0);
	var[step] = sqrt(cxx*cyy-cxy*cxy);
      } 
    }
  }
  return (1);
}

Double_t TH5Dataset::ps2mm(const Double_t length_ps, const Int_t step) {

  Double_t length_mm(length_ps);
  TString varGamma("#gamma");
  const Double_t conv = 0.299792458; // mm/ps
  // old gamma definition (gamma - 1) - should be updated
  Double_t gamma0 = GetScalarStepVar(&varGamma, step);
  if (gamma0 > 0.) {
    Double_t beta = sqrt(1. - 1. / (gamma0 + 1.) / (gamma0 + 1.));
    length_mm *= beta * conv;
  } else {
    Warning("ps2mm",
            "Conversion from ps to mm not possible, could not find variable %s", varGamma.Data());
  }
  return(length_mm);
}





/////////////////////////////////////////////////////////////////////////
//***********************************************************************
//                      some auxiliary functions (Bo Liu)
//***********************************************************************
/////////////////////////////////////////////////////////////////////////

Int_t TH5Dataset::GetSliceVar(const TString * varNameX, const TString* varNameY,
                              const Double_t spos,
                              const Double_t width,
                              const EH5SliceUnit sliceUnit,
                              const Double_t spacing,
                              const Bool_t bGaussian,
                              TNtupleD& sliceV)
// get slice information for whole bunch
// spos is the beam centroid position
// width is the slice width, sliceUnit is mm or ps
// spacing is the interval between slices, 
//    if spacing is less than zero, then only one slice is claculated
//    if spacing is equal to zero, then spacing is equal to slice width
// bGaussian indicates if gaussian weighting will be applied
//
// sliceV is filled with z,weight,sumX,sumY,sumXX,sumYY,sumYY, so the caller
// is responsible for defining a TNtupleD with 7 branches.
//
// the return value is the number of slices calculated at s > spos
//
// example usage of this funcition can be seen in obla4.cc
{
  Int_t step = GetStep(spos);
  if(step < 0) step = 0;

  return GetSliceVar(varNameX, varNameY, step, width, sliceUnit, spacing, bGaussian,
                            sliceV);
}

Int_t TH5Dataset::GetSliceVar(const TString* varNameX, const TString* varNameY,
                              const Int_t step,
                              const Double_t width,
                              const EH5SliceUnit sliceUnit,
                              const Double_t spacing,
                              const Bool_t bGaussian,
                              TNtupleD& sliceV)
{
  TNtupleD *nt = GetNtuple(step);
  TString svar("z");
  Double_t min = nt->GetMinimum(svar);
  Double_t max = nt->GetMaximum(svar);

  const Double_t half = 0.5;  //factor for +/- 
  const Double_t mm = 1.e-3;
  Double_t sliceL(width);
  if (sliceUnit == kps) sliceL = ps2mm(sliceL,step);
  if (bGaussian) { 
    sliceL *= 6.;
  }
  Double_t halfLength = sliceL * half * mm;

  Double_t sx(0.),sy(0.),sxx(0.),syy(0.),sxy(0.);
  Double_t spos = GetSpos(step);
  Double_t weight =
      GetSliceVarAt(varNameX, varNameY, step, halfLength, spos, bGaussian,
                    sx, sy, sxx, syy, sxy);
  sliceV.Fill(spos, weight, sx, sy, sxx, syy, sxy);

// if "spacing" is less than zero, then only calculate center slice
// if "spacing" is equal to zero, then spacing is set to slice length
  Int_t idx = 0;
  if (spacing >= 0.) {
    Double_t sp = spacing;
    if (sp == 0.) sp = halfLength*2.;
    for (Double_t z = spos + sp; z <= max + halfLength; z += sp) {
      weight =
      GetSliceVarAt(varNameX, varNameY, step, halfLength, z, bGaussian,
                    sx, sy, sxx, syy, sxy);
      idx++;
      sliceV.Fill(z, weight, sx, sy, sxx, syy, sxy);
    }
    for (Double_t z = spos - sp; z >= min - halfLength; z -= sp) {
      if (z < 0) break;
      weight =
      GetSliceVarAt(varNameX, varNameY, step, halfLength, z, bGaussian,
                    sx, sy, sxx, syy, sxy);
      sliceV.Fill(z, weight, sx, sy, sxx, syy, sxy);
     }
  }

  return idx;
}

Double_t TH5Dataset::GetSliceVarAt(const TString* varNameX, const TString* varNameY,
                                   const Int_t step,
                                   const Double_t halfL,
                                   const Double_t mean,
                                   const Bool_t bGaussian,
                                   Double_t& sx,
                                   Double_t& sy,
                                   Double_t& sxx,
                                   Double_t& syy,
                                   Double_t& sxy)
//get slice var at position "mean" for time "step" with half width "halfL"
{
//construct selection string
  char sel[50];
  sprintf(sel, "z>%f && z<=%f", mean-halfL, mean+halfL);
  TCut sliceCut(sel);

// construct expression to get weight information as well as x and px
// gaussian weighted or uniform weighted
  TString exp;

  if (bGaussian) {
    exp = TString("TMath::Gaus(z,");
    exp += mean;
    exp += ",";
    exp += (halfL*2.);
    exp += "):";
  } else {
    exp = TString("1:");
  }

  exp += *varNameX;
  exp += ":";
  exp += *varNameY;

  TNtupleD *nt = GetNtuple(step);

  nt->Draw(exp, sliceCut, "goff");
  Int_t nPoints = nt->GetSelectedRows();

  Double_t totalWeight = 0.;
  sxx = 0.;
  syy = 0.;
  sxy = 0.;
  if (nPoints > 0) {
    Double_t *v1 = nt->GetV1();
    Double_t *v2 = nt->GetV2();
    Double_t *v3 = nt->GetV3();

     for (Int_t i = 0; i < nPoints; i++) {
      totalWeight += *(v1 + i);

      sx += (*(v1 + i)) * (*(v2 + i)); 
      sy += (*(v1 + i)) * (*(v3 + i));
      sxx += (*(v1 + i)) * (*(v2 + i)) * (*(v2 + i));
      syy += (*(v1 + i)) * (*(v3 + i)) * (*(v3 + i));
      sxy += (*(v1 + i)) * (*(v2 + i)) * (*(v3 + i));
    }
  }

  return totalWeight;
}

Double_t TH5Dataset::GetSliceEmittance(const TString * varName,
                                       const Double_t spos,
                                       const Double_t sigma,
                                       const Double_t spacing,
                                       const Bool_t bGaussian,
                                       Double_t zpos[],
                                       Double_t emittance[], Int_t & count)
// get slice emittance information for whole bunch
// varName should be "x" or "y" or "z"
// spos is the beam centroid position
// sigma is the slice width
// spacing is the interval between slices, 
//    if spacing is less than or equal to zero, then only one slice is calculated
// bGaussian indicates if gaussian weighted will be applied
// zpos and emittance store the result, count is the number of slices calculated
// the return value is the slice emittance for centroid slice
// caller is responsible for allocating the array with enough space
{
  Int_t step = GetStep(spos);
  if(step < 0) step = 0;

  return GetSliceEmittance(varName, step, sigma, spacing, bGaussian,
                            zpos, emittance, count);
}

Double_t TH5Dataset::GetSliceEmittance(const TString* varName,
                                       const Int_t step,
                                       const Double_t sigma,
                                       const Double_t spacing,
                                       const Bool_t bGaussian,
                                       Double_t zpos[],
                                       Double_t emittance[], Int_t & count)
// get slice emittance information for whole bunch
// varName should be "x" or "y" or "z"
// step is current time step
// sigma is the slice width
// spacing is the interval between slices, 
//    if spacing is less than or equal to zero, then only one slice is claculated
// bGaussian indicates if gaussian weighted will be applied
// zpos and emittance store the result, count is the number of slices calculated
// the return value is the slice emittance for centroid slice
// caller is responsible for allocating the array with enough space
{
  TNtupleD *nt = GetNtuple(step);
  TString svar("z");
  Double_t min = nt->GetMinimum(svar);
  Double_t max = nt->GetMaximum(svar);

  Int_t idx = 0;
  const Double_t cutoff = 3.;
  
  Double_t spos = GetSpos(step);
  *(zpos) = spos;
  *(emittance) =
      GetSliceEmittanceAt(varName, step, sigma, spos, bGaussian);
  idx++;

// if "spacing" is less than or equal zero, then only calculate center slice
  if (spacing > 0) {
    for (Double_t z = spos + spacing; z <= max + spacing + sigma * cutoff;
         z += spacing) {
      *(zpos + idx) = z;
      *(emittance + idx) =
          GetSliceEmittanceAt(varName, step, sigma, z, bGaussian);
      idx++;
    }
    for (Double_t z = spos - spacing; z >= min - spacing - sigma * cutoff;
         z -= spacing) {
      *(zpos + idx) = z;
      *(emittance + idx) =
          GetSliceEmittanceAt(varName, step, sigma, z, bGaussian);
      idx++;
    }
  }
  count = idx;

  return *emittance;
}

Double_t TH5Dataset::GetSliceEmittanceAt(const TString* varName,
                                         const Int_t step,
                                         const Double_t sigma,
                                         const Double_t mean,
                                         const Bool_t bGaussian)
//get slice emittance at position "mean" for time "step" with width "sigma"
{
  Double_t sliceEmit(0.);
//construct selection string
//select paticles at "mean" position plus-minus 3 "sigma" for gaussian weight
//or at "mean" position plus-minus 0.5 "sigma" for uniform weight
  TString sliceVar("z");
  TString sel("(");
  Double_t cutoff;
  if (bGaussian) {
    cutoff = 3;
  } else {
    cutoff = 0.5;
  }
  sel += sliceVar;
  sel += TString(">");
  sel += mean - sigma * cutoff;
  sel += TString(") && (");
  sel += sliceVar;
  sel += TString("<");
  sel += mean + sigma * cutoff;
  sel += TString(")");

  TCut sliceCut(sel.Data());

// construct expression to get weight information as well as x and px
// gaussian weighted or uniform weighted
  TString exp;

  if (bGaussian) {
    exp = TString("TMath::Gaus(z,");
    exp += mean;
    exp += ",";
    exp += sigma;
    exp += "):";
  } else {
    exp = TString("1:");
  }

  if (*varName == "x") 
    exp += "x:px";
  else if (*varName == "y")
    exp += "y:py";
  else if (*varName == "z") 
    exp += "z:pz";
  else { 
    cout << "varname (" << *varName << ") not allowed here" << endl;
    return 0.;
  }
//      cout << "string exp:" << exp << endl;

  TNtupleD *nt = GetNtuple(step);
  nt->SetEstimate(nt->GetEntries());

  nt->Draw(exp, sliceCut, "goff");
  Int_t nPoints = nt->GetSelectedRows();

  if (nPoints > 0) {
    Double_t *v1 = nt->GetV1();
    Double_t *v2 = nt->GetV2();
    Double_t *v3 = nt->GetV3();

    Double_t totalWeight = 0.;
    Double_t squareX = 0.;
    Double_t squareXp = 0.;
    Double_t xXp = 0.;

    for (Int_t i = 0; i < nPoints; i++) {
//    cout << *(v1+i) << "\t\t" << *(v2+i) << "\t\t" << *(v3+i) << endl;
      totalWeight += *(v1 + i);
      squareX += (*(v1 + i)) * (*(v2 + i)) * (*(v2 + i));
      squareXp += (*(v1 + i)) * (*(v3 + i)) * (*(v3 + i));
      xXp += (*(v1 + i)) * (*(v2 + i)) * (*(v3 + i));
    }
    sliceEmit = squareX * squareXp - xXp * xXp;
    if (sliceEmit < 0) {
      if (TMath::Abs(sliceEmit) < 1e-20) {
        sliceEmit = 0.;
      } else {
        cout << "error: sliceEmit " << sliceEmit << " less than 0" << endl;
        return 0.;
      }
    }
    if (totalWeight <= 0) {
      cout << "error: weight less than 0" << endl;
      return 0.;
    }
    sliceEmit = sqrt(sliceEmit) / totalWeight;

  }

  cout << "slice emittance " << *varName << " (step:" << step << ") (pos:";
  cout << mean << ") (sigma:" << sigma << "):" << endl;
  cout << "\t" << "particle in slice:" << nPoints << endl;
  cout << "\t" << "slice emittane:" << sliceEmit << endl << endl;
  return sliceEmit;
}

Double_t TH5Dataset::GetEmittanceAtFixedZ(const TString* varName,
                                          Double_t zpos[],
                                          Double_t emittance[])
//get emittance at fixed z position, filling the array
// (for every time step determine zpos and screen emittance, return the two arrays)
{
  Int_t step(0);
  TArrayD *sumXX = new TArrayD(fNStep);
  TArrayD *sumXY = new TArrayD(fNStep);
  TArrayD *sumYY = new TArrayD(fNStep);
  Int_t nParticles[fNStep];

  TString varNameZ("z");
  for (step = 0; step < fNStep; step++) { //initialize the sum value
    zpos[step] = (*fSpos)[step];
    (*sumXX)[step] = 0.;
    (*sumXY)[step] = 0.;
    (*sumYY)[step] = 0.;
    nParticles[step] = 0;
  }
  zpos[fNStep] = (*fSpos)[fNStep-1]*2 - (*fSpos)[fNStep-2]; //extend to calculate the last slice

cout << "get emittance (screen) for " << *varName << " & p" << *varName << ": " << flush;
   for (step = 0; step < fNStep; ++step) { // loop over each step
cout << setw(fWidthStep) << step+1 << "/" << setw(fWidthStep) << fNStep << flush;
    TNtupleD *nt = GetNtuple(step);
    Double_t smin = nt->GetMinimum(varNameZ);
    Double_t smax = nt->GetMaximum(varNameZ);
//cout << "current step: " << step << "( smin: " << smin << 
//                                    "; spos: " << zpos[step] <<
//                                    "; smax: " << smax << ")" << endl;
    Int_t iStart = GetStep(smin);
    if (iStart < 0) iStart = 0;
    Int_t iEnd = GetStep(smax); 
    for (Int_t iStep = iStart; iStep <= iEnd; ++iStep) { // loop over whole bunch
//cout << "\t step " << iStep << ":spos: " << zpos[iStep] << " is included" << endl;
      TString sel("(");
      sel += varNameZ;;
      sel += TString(">=");
      sel += zpos[iStep];
      sel += TString(") && (");
      sel += varNameZ;;
      sel += TString("<");
      sel += zpos[iStep+1];
      sel += TString(")");

      TCut sliceCut(sel.Data());

      nt->Draw(*varName + TString(":p") + *varName, sliceCut, "goff");
      Int_t nPoints = nt->GetSelectedRows();
      nParticles[iStep] += nPoints;
//cout << "particle number in this slice: " << nPoints << endl;
      if (nPoints > 0) { // if np > 0
        Double_t *v1 = nt->GetV1();
        Double_t *v2 = nt->GetV2();
        for (Int_t i = 0; i < nPoints; i++) { //sum all particles in this slice
      //    cout << *(v1+i) << "\t\t" << *(v2+i) << endl;
          (*sumXX)[iStep] += (*(v1 + i)) * (*(v1 + i));
          (*sumXY)[iStep] += (*(v1 + i)) * (*(v2 + i));
          (*sumYY)[iStep] += (*(v2 + i)) * (*(v2 + i));
        } //end of loop of particles
      } // end of if (nPoints > 0) 
    } // end of loop for whole bunch
for (Int_t j=0; j<fWidthStep*2+1; ++j) cout << "\b";
cout << flush;
    } //end of loop for each step

// get final results
  for (step = 0; step < fNStep; ++step) { // loop over each step
    Double_t eps = ((*sumXX)[step])*((*sumYY)[step]) - ((*sumXY)[step])*((*sumXY)[step]);
    emittance[step] = sqrt(eps)/nParticles[step];
//cout << "step:" << step << " particles:" << nParticles[step] << " emit:" << emittance[step] << endl;
  }

  return 0.;
}

Double_t TH5Dataset::GetEmittanceAtFixedSingleZ(const TString* varName,
						Double_t zpos)
//get projected emittance at fixed z position 
{
  Double_t sumXX(0.),sumXY(0.),sumYY(0.);
  TString varNameZ("z");

  Int_t centerStep = GetStep(zpos);
  if (centerStep < 0)
    centerStep = 0; 
  Double_t zStart = GetSpos(centerStep);
  Double_t zEnd = (centerStep == fNStep - 1) ? zStart*2-GetSpos(fNStep-2) : GetSpos(centerStep+1);

  TNtupleD *nt = GetNtuple(centerStep);
  Double_t smin = nt->GetMinimum(varNameZ);
  Double_t smax = nt->GetMaximum(varNameZ);
//cout << "smin:" << smin << " zstart:" << zStart << " zend:" << zEnd << " smax:" << smax << endl;

  Int_t iStartStep = GetStep(zStart-smax+smin) ; // get first step that includes particles for this slice
  if (iStartStep < 0) iStartStep = 0;
  Int_t iEndStep = GetStep(zStart+smax-smin) ; // get last step that includes particles for this slice
  if (iEndStep >= fNStep) iEndStep = fNStep - 1;

cout << iStartStep << "\t" << iEndStep << endl;

  Int_t i1 = (iStartStep == 0)? 0: iStartStep-1;
  nt = GetNtuple(i1);
  smax = nt->GetMaximum(varNameZ);
  Int_t i2 = (iEndStep == fNStep - 1)? fNStep-1: iEndStep+1;
  nt = GetNtuple(i2);
  smin = nt->GetMinimum(varNameZ);
//cout << "smax at iStart-1: " << smax << " smin at iEnd+1: " << smin << endl;
  
//if (iEndStep < fNStep - 1) {
//    nt = GetNtuple(iEndStep+1);
//    smax = nt->GetMaximum(varNameZ);
//    iEndStep = GetStep(smax);
//  } 
//cout << iStartStep << "\t" << iEndStep << endl;

  Int_t nPartsTot(0);
  for (Int_t step = iStartStep; step <= iEndStep; ++step) { // loop over related steps
//cout << "\t step " << step << ":spos: " << (*fSpos)[step] << " is included" << endl;
    TNtupleD *nt = GetNtuple(step);
    TString sel("(");
    sel += varNameZ;;
    sel += TString(">=");
    sel += zStart;
    sel += TString(") && (");
    sel += varNameZ;;
    sel += TString("<");
    sel += zEnd;
    sel += TString(")");

    TCut sliceCut(sel.Data());

    nt->Draw(*varName + TString(":p") + *varName, sliceCut, "goff");
    Int_t nPoints = nt->GetSelectedRows();
cout << "particle number in this slice: " << nPoints << endl;
    nPartsTot += nPoints;
    if (nPoints > 0) { // if np > 0
      Double_t *v1 = nt->GetV1();
      Double_t *v2 = nt->GetV2();
      for (Int_t i = 0; i < nPoints; i++) { //sum all particles in this slice
//cout << *(v1+i) << "\t\t" << *(v2+i) << endl;
        sumXX += (*(v1 + i)) * (*(v1 + i));
        sumXY += (*(v1 + i)) * (*(v2 + i));
        sumYY += (*(v2 + i)) * (*(v2 + i));
      } //end of loop of particles
    } // end of if (nPoints > 0) 
  } //end of loop for each step

  Double_t eps = sumXX * sumYY - sumXY * sumXY;

  eps = sqrt(eps)/nPartsTot;

  cout << "total particles: " << nPartsTot << endl;
  return eps;
}

