#include "TH5LoadDialog.h"
#include<iostream>

using namespace std;

ClassImp(TH5LoadDialog)
  
TH5LoadDialog::TH5LoadDialog(const TGWindow *p, TH5MainFrame *mainFrame, UInt_t w,
			     UInt_t h, UInt_t options):fParent(mainFrame)
{
  fMain = new TGTransientFrame(p, mainFrame, w, h, options);
  fMain->Connect("CloseWindow()", "TH5LoadDialog", this, "CloseWindow()");
  fMain->DontCallClose(); // to avoid double deletions.
  
  // use hierarchical cleaning
  fMain->SetCleanup(kDeepCleanup);
  
  fFileLabel = new TGLabel(fMain,"Enter filename: ");
  
  fFileEntry = new TGTextEntry(fMain, fFilenameBuffer = new TGTextBuffer(100));
  fFileEntry->Resize(400, fFileEntry->GetDefaultHeight());
  fFileEntry->SetToolTipText("Enter the filename, dude!");
  fFileEntry->Connect("ReturnPressed()", "TH5LoadDialog", this, "DoLoad()");

  fMain->AddFrame(fFileLabel, new TGLayoutHints(kLHintsLeft,5,5,5,5)); 
  fMain->AddFrame(fFileEntry, new TGLayoutHints(kLHintsLeft,5,5,5,5)); 

  fMemLabel = new TGLabel(fMain,"Select memory management: ");

  fMain->AddFrame(fMemLabel, new TGLayoutHints(kLHintsLeft,5,5,20,5)); 

  fMain->AddFrame(fRadioMem[0] = new TGRadioButton(fMain, 
                        "load steps as needed and dump them after use", kLoadAndDump), 
			new TGLayoutHints(kLHintsLeft,20,2,2,2)); 
  fMain->AddFrame(fRadioMem[1] = new TGRadioButton(fMain, 
                        "load steps as needed and keep them in memory", kLoadAndKeep), 
			new TGLayoutHints(kLHintsLeft,20,2,2,2)); 
  fMain->AddFrame(fRadioMem[2] = new TGRadioButton(fMain, 
                        "preload all steps when loading file and keep them in memory", kPreloadAll), 
			new TGLayoutHints(kLHintsLeft,20,2,2,2)); 
  fRadioMem[0]->SetToolTipText("use this option for very large files");
  fRadioMem[1]->SetToolTipText("use this option if you just need to look at a few steps!");
  fRadioMem[2]->SetToolTipText("use this option for smaller files");
  fRadioMem[0]->Connect("Pressed()", "TH5LoadDialog", this, "DoMemOptRadioButtons()");
  fRadioMem[1]->Connect("Pressed()", "TH5LoadDialog", this, "DoMemOptRadioButtons()");
  fRadioMem[2]->Connect("Pressed()", "TH5LoadDialog", this, "DoMemOptRadioButtons()");
  fRadioMem[0]->SetOn();

  fSubsampFrame = new TGHorizontalFrame(fMain, 400, 20, kFixedWidth);

  fSubsampLabel = new TGLabel(fSubsampFrame,"Select subsampling rate: ");
  fSubsampFrame->AddFrame(fSubsampLabel, new TGLayoutHints(kLHintsLeft,5,5,5,5)); 

  fSubsampEntry = new TGTextEntry(fSubsampFrame, fSubsampBuffer = new TGTextBuffer(100));
  fSubsampEntry->Resize(50, fSubsampEntry->GetDefaultHeight());
  fSubsampEntry->SetToolTipText("Enter the desired subsampling rate");
  //  fSubsampEntry->Connect("ReturnPressed()", "TH5LoadDialog", this, "DoLoad()");
  fSubsampFrame->AddFrame(fSubsampEntry, new TGLayoutHints(kLHintsLeft,5,5,3,7)); 

  // write default (global) subsampling rate into buffer:
  char bufSubsamp[32];
  sprintf(bufSubsamp, "%d", fParent->GetGlobalSubsamplingRate());
  fSubsampBuffer->AddText(0, bufSubsamp);

  fMain->AddFrame(fSubsampFrame, new TGLayoutHints(kLHintsLeft,0,0,20,0)); 

  fBottomFrame = new TGHorizontalFrame(fMain, 60, 20, kFixedWidth);
  
  fLoadButton = new TGTextButton(fBottomFrame, "&Load", 1);
  fLoadButton->Connect("Clicked()", "TH5LoadDialog", this, "DoLoad()");
  fCancelButton = new TGTextButton(fBottomFrame, "&Cancel", 2);
  fCancelButton->Connect("Clicked()", "TH5LoadDialog", this, "DoCancel()");
  
  fBottomFrame->AddFrame(fLoadButton,   new TGLayoutHints(kLHintsTop | kLHintsRight ,10, 10, 5, 5));
  fBottomFrame->AddFrame(fCancelButton, new TGLayoutHints(kLHintsTop | kLHintsRight ,10, 10, 5, 5));
 
  fMain->AddFrame(fBottomFrame, new TGLayoutHints(kLHintsBottom | kLHintsExpandX, 0, 0, 10, 0));
  
  fMain->MapSubwindows();

  fMain->Resize();
  
  // position relative to the parent's window
  fMain->CenterOnParent();
  
  fMain->SetWindowName("H5PartROOT Load");
  
  fMain->MapWindow();
  //gClient->WaitFor(fMain);    // otherwise canvas contextmenu does not work
}

TH5LoadDialog::~TH5LoadDialog()
{
  // Delete load dialog widgets.
  fMain->DeleteWindow();  // deletes fMain
}

void TH5LoadDialog::CloseWindow()
{
  // Called when window is closed via the window manager.
  delete this;
}

void TH5LoadDialog::DoMemOptRadioButtons()
{
  
  TGButton *btn = (TGButton *) gTQSender;
  Int_t id = btn->WidgetId();
  
  switch (id) {
  case kLoadAndDump:  
    fRadioMem[1]->SetState(kButtonUp);
    fRadioMem[2]->SetState(kButtonUp);
    break;
  case kLoadAndKeep:  
    fRadioMem[0]->SetState(kButtonUp);
    fRadioMem[2]->SetState(kButtonUp);
    break;
  case kPreloadAll:  
    fRadioMem[0]->SetState(kButtonUp);
    fRadioMem[1]->SetState(kButtonUp);
    break;
  default:
    break;
   }
}

void TH5LoadDialog::DoLoad()
{
  if (fFilenameBuffer->GetTextLength()!=0) { 
    EH5MemOpt memOpt;
    if (kButtonDown == fRadioMem[2]->GetState()) memOpt = kPreloadAll; 
    if (kButtonDown == fRadioMem[1]->GetState()) memOpt = kLoadAndKeep; 
    if (kButtonDown == fRadioMem[0]->GetState()) memOpt = kLoadAndDump; 
    fParent->LoadFile(fFilenameBuffer->GetString(),memOpt,atoi(fSubsampBuffer->GetString()));   
  } 
  fMain->SendCloseMessage();
}

void TH5LoadDialog::DoCancel()
{
  fMain->SendCloseMessage();
}
