#include <TApplication.h>
#include <TSystem.h>
#include <TGClient.h>
#include <TGString.h>
#include <TString.h>
#include <TCanvas.h>
#include <TF1.h>
#include <TH2.h>
#include <TH3.h>
#include <TRandom.h>
#include <TRootEmbeddedCanvas.h>
#include <TGraph.h>
#include <TTime.h>
#include <TROOT.h>
#include <TMath.h>

#include <string>
#include <iostream>

#include "TH5MainFrame.h"
#include "TH5LoadDialog.h"
#include "TH5PrintDialog.h"
#include "TH5SliceDialog.h"
#include "TH5ProgressBar.h"
#include "TH5Legend.h"

using namespace std;
using namespace TH5Util;

ClassImp(TH5MainFrame)
  
TH5MainFrame::TH5MainFrame(const TGWindow *p, UInt_t w, UInt_t h, Int_t ssr)
  : TGMainFrame(p, w, h), fgSubsampRate(ssr) {
  
  // Initializations:

  // printer formats (stored as 15x3 array)
  // (0: string needed by ROOT, 
  //  1: corresponding file extension, 
  //  2: corresponding entry in TGComboBox) 
  H5PrintFormat = new TString*[15];
  for(int i=0;i<15;i++) H5PrintFormat[i] = new TString[3];
  H5PrintFormat[0][0] = TString("pdf");	       
  H5PrintFormat[1][0] = TString("jpg");	       
  H5PrintFormat[2][0] = TString("png");	       
  H5PrintFormat[3][0] = TString("gif");	       
  H5PrintFormat[4][0] = TString("tiff");	       
  H5PrintFormat[5][0] = TString("ps");	       
  H5PrintFormat[6][0] = TString("Portrait");   
  H5PrintFormat[7][0] = TString("Landscape");  
  H5PrintFormat[8][0] = TString("eps");	       
  H5PrintFormat[9][0] = TString("Preview");    
  H5PrintFormat[10][0] = TString("svg");	       
  H5PrintFormat[11][0] = TString("xpm");	       
  H5PrintFormat[12][0] = TString("xml");	       
  H5PrintFormat[13][0] = TString("cxx");	       
  H5PrintFormat[14][0] = TString("root");         

  // other names are the same... 
  for (Int_t i=0; i<15; ++i) {
    H5PrintFormat[i][1]=TString(".")+H5PrintFormat[i][0];
    H5PrintFormat[i][2]=H5PrintFormat[i][0];
  }
  // ...except:
  H5PrintFormat[6][1] = H5PrintFormat[5][1]; 
  H5PrintFormat[7][1] = H5PrintFormat[5][1]; 
  H5PrintFormat[9][1] = H5PrintFormat[8][1]; 
  H5PrintFormat[6][2] = TString("ps-port");
  H5PrintFormat[7][2] = TString("ps-land");
  H5PrintFormat[9][2] = TString("eps-pre");
  
  // options to position slice within bunch (stored as array of TStrings)
  H5SlicePosition = new TString[6];
  H5SlicePosition[0] = TString("centered around mean in z");	         
  H5SlicePosition[1] = TString("centered around median in z");	         
  H5SlicePosition[2] = TString("centered around (zmax-zmin)/2");	         
  H5SlicePosition[3] = TString("at the start of the bunch");	         
  H5SlicePosition[4] = TString("at the end of the bunch");	         
  H5SlicePosition[5] = TString("centered around peak in z");	         

  //initialize pointers to datasets:
  for (Int_t n=0; n<5; ++n) {
    fDataset[n] = NULL;
  }

  fNStepMax = 0;

  DrawLogX = kFALSE;
  DrawLogY = kFALSE;
  DrawLogZ = kFALSE;

  //--------------------------------------------------------------------------
  // pull-down menus
  fMenuDock = new TGDockableFrame(this);
  AddFrame(fMenuDock, new TGLayoutHints(kLHintsExpandX, 0, 0, 1, 0));
  fMenuDock->SetWindowName("H5PartROOT");

  fMenuBarLayout = new TGLayoutHints(kLHintsTop | kLHintsExpandX);
  fMenuBarItemLayout = new TGLayoutHints(kLHintsTop | kLHintsLeft, 0, 4, 0, 0);
  fMenuBarHelpLayout = new TGLayoutHints(kLHintsTop | kLHintsRight);

  fMenuFile = new TGPopupMenu(gClient->GetRoot());
  fMenuFile->AddEntry("&Load...", M_FILE_LOAD);
  fMenuFile->AddEntry("&Unload...", M_FILE_UNLOAD);
  fMenuFile->AddSeparator();
  fMenuFile->AddEntry("&Print...", M_FILE_PRINT);
  fMenuFile->AddSeparator();
  fMenuFile->AddEntry("E&xit", M_FILE_EXIT);

  fMenuFile->DisableEntry(M_FILE_UNLOAD); // to be implemented later
 
  fMenuFile->Connect("Activated(Int_t)", "TH5MainFrame", this,
		     "DoMenu(Int_t)");

  fMenuEdit = new TGPopupMenu(gClient->GetRoot());
  fMenuEdit->AddEntry("&Canvas...", M_EDIT_CANV);

  fMenuEdit->Connect("Activated(Int_t)", "TH5MainFrame", this,
		     "DoMenu(Int_t)");

  fMenuOptLog = new TGPopupMenu(gClient->GetRoot());
  fMenuOptLog->AddEntry("Log&X", M_OPT_LOGX);
  fMenuOptLog->UnCheckEntry(M_OPT_LOGX);
  fMenuOptLog->AddEntry("Log&Y", M_OPT_LOGY);
  fMenuOptLog->UnCheckEntry(M_OPT_LOGY);
  fMenuOptLog->AddEntry("Log&Z", M_OPT_LOGZ);
  fMenuOptLog->UnCheckEntry(M_OPT_LOGZ);
  
  fMenuOpt = new TGPopupMenu(gClient->GetRoot());
  
  fMenuOpt->AddEntry("&Slice...", M_OPT_SLICE);
  fMenuOpt->AddPopup("&Log", fMenuOptLog);

  fMenuOpt->Connect("Activated(Int_t)", "TH5MainFrame", this,
		    "DoMenu(Int_t)");

  fMenuHelp = new TGPopupMenu(gClient->GetRoot());
  fMenuHelp->AddEntry("&About", M_HELP_ABOUT);

  fMenuHelp->Connect("Activated(Int_t)", "TH5MainFrame", this,
		     "DoMenu(Int_t)");

  fMenuHelp->DisableEntry(M_HELP_ABOUT); // to be implemented later

  fMenuBar = new TGMenuBar(fMenuDock, 1, 1, kHorizontalFrame);
  fMenuBar->AddPopup("&File", fMenuFile, fMenuBarItemLayout);
  fMenuBar->AddPopup("E&dit", fMenuEdit, fMenuBarItemLayout);
  fMenuBar->AddPopup("&Options", fMenuOpt, fMenuBarItemLayout);
  fMenuBar->AddPopup("&Help", fMenuHelp, fMenuBarHelpLayout);

  fMenuDock->AddFrame(fMenuBar, fMenuBarLayout);

  //--------------------------------------------------------------------------
  // main canvas for plotting:
  fEcanvas = new TRootEmbeddedCanvas ("Ecanvas",this,w,h);
  AddFrame(fEcanvas, new TGLayoutHints(kLHintsExpandX | kLHintsExpandY,10,10,10,1));

  //--------------------------------------------------------------------------
  // work space: (everything below the canvas)
  fWorkspace = new TGHorizontalFrame(this, 600,40);

  //--------------------------------------------------------------------------
  // selection space: for file and step selection
  fSelspace = new TGVerticalFrame(fWorkspace, 300,100);

  // group frame for file selection:
  fFileSelection = new TGGroupFrame(fSelspace, "File selection",kVerticalFrame);
  fFileSelection->SetTitlePos(TGGroupFrame::kLeft);

  for (Int_t n=0; n<5; ++n) {
    fFileButton[n] = new TGCheckButton(fFileSelection, "      no file loaded yet         ",200);
    fFileButton[n]->SetState(kButtonDisabled);
    fFileSelection->AddFrame(fFileButton[n], new TGLayoutHints(kLHintsLeft|kLHintsExpandX,5,5,3,3));
    TString slotCommand = TString("ActivateFile(Int_t=");
    slotCommand+=n;
    slotCommand+=TString(")");
    fFileButton[n]->Connect("Clicked()","TH5MainFrame",this,slotCommand.Data());  
  } 
  
  // group frame for step selection:
  fStepSelection = new TGGroupFrame(fSelspace, "Step selection",kVerticalFrame);
  fStepSelection->SetTitlePos(TGGroupFrame::kLeft);

  fStepSlider = new TGHSlider(fStepSelection, 220, kSlider1 | kScaleDownRight, stepSliderID);
  fStepSlider->SetRange(0,100);
  fStepSlider->Connect("PositionChanged(Int_t)", "TH5MainFrame", this, "DoStepSlider(Int_t)");
  fStepSlider->Connect("Released()", "TH5MainFrame", this, "DoLastPlot()");

  // various subframes needed for layout below step slider:
  fStepSposTEFrame = new TGHorizontalFrame(fStepSelection);
  fStepTEFrame = new TGVerticalFrame(fStepSposTEFrame);
  fSposTEFrame = new TGVerticalFrame(fStepSposTEFrame);
  fPrevStepFrame = new TGVerticalFrame(fStepSposTEFrame);
  fNextStepFrame = new TGVerticalFrame(fStepSposTEFrame);

  fStepLabel = new TGLabel(fStepTEFrame,"Step");
  fStepSliderTE = new TGTextEntry(fStepTEFrame, fStepSliderTB = new TGTextBuffer(5), stepSliderTEID);
  fStepSliderTB->AddText(0, "0");
  fStepSliderTE->Connect("TextChanged(char*)", "TH5MainFrame", this, "DoStepText(char*)");
  fStepTEFrame->AddFrame(fStepLabel, new TGLayoutHints(kLHintsLeft,2,1,1,0));
  fStepTEFrame->AddFrame(fStepSliderTE, new TGLayoutHints(kLHintsCenterX,2,1,0,5));

  fSposLabel = new TGLabel(fSposTEFrame,"Spos");
  fSposSliderTE = new TGTextEntry(fSposTEFrame, fSposSliderTB = new TGTextBuffer(5), sposSliderTEID);
  fSposSliderTB->AddText(0, "0");
  fSposSliderTE->Connect("TextChanged(char*)", "TH5MainFrame", this, "DoStepText(char*)");
  fSposSliderTE->Connect("ReturnPressed()","TH5MainFrame",this,"DoStepTextReturn(char*)");
  fSposTEFrame->AddFrame(fSposLabel, new TGLayoutHints(kLHintsLeft,2,1,1,0));
  fSposTEFrame->AddFrame(fSposSliderTE, new TGLayoutHints(kLHintsCenterX,2,1,0,5));

  fPrevStepLabel = new TGLabel(fPrevStepFrame,"   ");
  fPrevButton = new TGTextButton(fPrevStepFrame," &Prev ");
  fPrevButton->SetToolTipText("Go to previous step");
  fPrevButton->Connect("Clicked()","TH5MainFrame",this,"PlotNextStep(Int_t=-1)");
  fPrevStepFrame->AddFrame(fPrevStepLabel, new TGLayoutHints(kLHintsLeft,2,1,1,0));
  fPrevStepFrame->AddFrame(fPrevButton, new TGLayoutHints(kLHintsRight,2,1,0,5));

  fNextStepLabel = new TGLabel(fNextStepFrame,"   ");
  fNextButton = new TGTextButton(fNextStepFrame," &Next ");
  fNextButton->SetToolTipText("Go to next step");
  fNextButton->Connect("Clicked()","TH5MainFrame",this,"PlotNextStep(Int_t=1)");
  fNextStepFrame->AddFrame(fNextStepLabel, new TGLayoutHints(kLHintsLeft,2,1,1,0));
  fNextStepFrame->AddFrame(fNextButton, new TGLayoutHints(kLHintsRight,2,1,0,5));

  fStepSposTEFrame->AddFrame(fStepTEFrame, new TGLayoutHints(kLHintsCenterX,2,1,0,0));
  fStepSposTEFrame->AddFrame(fSposTEFrame, new TGLayoutHints(kLHintsCenterX,2,1,0,0));
  fStepSposTEFrame->AddFrame(fNextStepFrame, new TGLayoutHints(kLHintsRight,5,1,0,0));
  fStepSposTEFrame->AddFrame(fPrevStepFrame, new TGLayoutHints(kLHintsRight,10,1,0,0));

  fStepSelection->AddFrame(fStepSlider, new TGLayoutHints(kLHintsLeft | kLHintsExpandX,5,1,0,5));
  fStepSelection->AddFrame(fStepSposTEFrame, new TGLayoutHints(kLHintsLeft | kLHintsExpandX,5,1,2,0));

  fSelspace->AddFrame(fFileSelection, new TGLayoutHints(kLHintsLeft|kLHintsExpandX,5,5,3,5));
  fSelspace->AddFrame(fStepSelection, new TGLayoutHints(kLHintsLeft|kLHintsExpandX,5,5,3,3)); 

  fSelspace->MapSubwindows();

  //--------------------------------------------------------------------------
  // variable space: for variable selection (also contains histogram buttons)
  fHistospace = new TGVerticalFrame(fWorkspace, 300,100);

  // group frame for variable selection
  fVarSelection = new TGGroupFrame(fHistospace, "Variable selection",kHorizontalFrame);
  fVarSelection->SetTitlePos(TGGroupFrame::kLeft);

  fVarSelectionL = new TGVerticalFrame(fVarSelection);
  fVarSelectionR = new TGVerticalFrame(fVarSelection);

  // default names for particle variables 
  // (will be overwritten by loaded file)
  fVarNameDefault[0] = TString(" x ");
  fVarNameDefault[1] = TString(" y ");
  fVarNameDefault[2] = TString(" z ");
  fVarNameDefault[3] = TString(" phiz");
  fVarNameDefault[4] = TString(" px ");
  fVarNameDefault[5] = TString(" py ");
  fVarNameDefault[6] = TString(" pz ");
  fVarNameDefault[7] = TString(" enez");

  for (Int_t n=0; n<4; ++n) {
    fVarName[n] = fVarNameDefault[n];
    fVarButton[n] = new TGCheckButton(fVarSelectionL, fVarName[n],200);
    fVarButton[n]->SetState(kButtonDisabled);
    Int_t t = 2;
    if (n==3) t = 10;
    fVarSelectionL->AddFrame(fVarButton[n], new TGLayoutHints(kLHintsLeft,5,5,t,2));
  } 
  
  for (Int_t n=4; n<8; ++n) {
    fVarName[n] = fVarNameDefault[n];
    fVarButton[n] = new TGCheckButton(fVarSelectionR, fVarName[n],200);
    fVarButton[n]->SetState(kButtonDisabled);
    Int_t t = 2;
    if (n==7) t = 10;
    fVarSelectionR->AddFrame(fVarButton[n], new TGLayoutHints(kLHintsLeft,5,5,t,2));
  } 
  
  fVarSelection->AddFrame(fVarSelectionL, new TGLayoutHints(kLHintsLeft,0,0,2,0)); 
  fVarSelection->AddFrame(fVarSelectionR, new TGLayoutHints(kLHintsLeft,0,8,2,0)); 
  fVarSelection->MapSubwindows();

  // group frame for histogramming, with three subframes
  fHistoFrame = new TGGroupFrame(fHistospace, "Histograms",kVerticalFrame);
  f1dFrame = new TGHorizontalFrame(fHistoFrame);
  f2dFrame = new TGHorizontalFrame(fHistoFrame);
  f3dFrame = new TGHorizontalFrame(fHistoFrame);
  fSEFrame = new TGHorizontalFrame(fHistoFrame);

  // 1d histograms
  fHisto1d = new TGTextButton(f1dFrame,"  &1D  ");
  fHisto1d->SetToolTipText("Draw 1d histogram of selected variable(s)");
  fHisto1d->Connect("Clicked()","TH5MainFrame",this,"Histo1d()");
  fHisto1d->Resize(20,fHisto1d->GetDefaultHeight());

//  fHisto1dBox = new TGComboBox(f1dFrame,"style",100);
  fHisto1dBox = new TGComboBox(f1dFrame,100);
  fHisto1dBox->AddEntry("HIST",0);   H5DrawOpt1d[0] = TString("");
  fHisto1dBox->AddEntry("ERRBAR",1); H5DrawOpt1d[1] = TString("e1p");
  fHisto1dBox->AddEntry("ERRBOX",2); H5DrawOpt1d[2] = TString("e2p");
  fHisto1dBox->AddEntry("CURVE",3);  H5DrawOpt1d[3] = TString("c");
  fHisto1dBox->AddEntry("BAR",4);    H5DrawOpt1d[4] = TString("bar2"); // does the same as "b"...??
  fHisto1dBox->AddEntry("LINE",5);   H5DrawOpt1d[5] = TString("l");
  fHisto1dBox->AddEntry("LEGO",6);   H5DrawOpt1d[6] = TString("lego");
  fHisto1dBox->Select(0);
  fHisto1dBox->Resize(70,20);

  // 2d histograms
  fHisto2d = new TGTextButton(f2dFrame,"  &2D  ");
  fHisto2d->SetToolTipText("Draw 2d histogram of selected variables");
  fHisto2d->Connect("Clicked()","TH5MainFrame",this,"Histo2d()");
  fHisto2d->Resize(70,fHisto2d->GetDefaultHeight());

//  fHisto2dBox = new TGComboBox(f2dFrame,"style",100);
  fHisto2dBox = new TGComboBox(f2dFrame,100);
  fHisto2dBox->AddEntry("NOBIN",0);  H5DrawOpt2d[0] = TString("unbinned");
  fHisto2dBox->AddEntry("SCAT",1);   H5DrawOpt2d[1] = TString("");
  fHisto2dBox->AddEntry("BOX",2);    H5DrawOpt2d[2] = TString("box");
  fHisto2dBox->AddEntry("ARR",3);    H5DrawOpt2d[3] = TString("arr");
  fHisto2dBox->AddEntry("COLZ",4);   H5DrawOpt2d[4] = TString("colz");
  fHisto2dBox->AddEntry("TEXT",5);   H5DrawOpt2d[5] = TString("text");
  fHisto2dBox->AddEntry("CONT",6);   H5DrawOpt2d[6] = TString("cont");
  fHisto2dBox->AddEntry("CONTZ",7);  H5DrawOpt2d[7] = TString("contz");
  fHisto2dBox->AddEntry("CONT1",8);  H5DrawOpt2d[8] = TString("cont1");
  fHisto2dBox->AddEntry("CONT2",9);  H5DrawOpt2d[9] = TString("cont2");
  fHisto2dBox->AddEntry("CONT3",10); H5DrawOpt2d[10] = TString("cont3");
  fHisto2dBox->AddEntry("LEGO",11);  H5DrawOpt2d[11] = TString("lego");
  fHisto2dBox->AddEntry("LEGO1",12); H5DrawOpt2d[12] = TString("lego1");
  fHisto2dBox->AddEntry("LEGO2",13); H5DrawOpt2d[13] = TString("lego2");
  fHisto2dBox->AddEntry("SURF1",14); H5DrawOpt2d[14] = TString("surf1");
  fHisto2dBox->AddEntry("SURF2z",15);H5DrawOpt2d[15] = TString("surf2z");
  fHisto2dBox->AddEntry("SURF3",16); H5DrawOpt2d[16] = TString("surf3");
  fHisto2dBox->AddEntry("SURF4",17); H5DrawOpt2d[17] = TString("surf4");
  fHisto2dBox->AddEntry("SURF1POL",18); H5DrawOpt2d[18] = TString("surf1pol");
  fHisto2dBox->AddEntry("SURF1CYL",19); H5DrawOpt2d[19] = TString("surf1cyl");
  fHisto2dBox->Select(0);
  fHisto2dBox->Resize(70,20);

  // 3d histograms
  fHisto3d = new TGTextButton(f3dFrame,"  &3D  ");
  fHisto3d->SetToolTipText("Draw 3d histogram of selected variables");
  fHisto3d->Connect("Clicked()","TH5MainFrame",this,"Histo3d()");
  fHisto3d->Resize(100,fHisto3d->GetDefaultHeight());

//  fHisto3dBox = new TGComboBox(f3dFrame,"style",100);
  fHisto3dBox = new TGComboBox(f3dFrame,100);
  fHisto3dBox->AddEntry("SCAT",0); H5DrawOpt3d[0] = TString("");
  fHisto3dBox->AddEntry("BOX",1);  H5DrawOpt3d[1] = TString("box");
  fHisto3dBox->Select(0);
  fHisto3dBox->Resize(70,20);

  fHistoSE = new TGTextButton(fSEFrame,"S&l Em");
  fHistoSE->Connect("Clicked()","TH5MainFrame",this,"HistoSliceEmit()");
  fHistoSE->Resize(20,fHistoSE->GetDefaultHeight());

//  fHistoSEBox = new TGComboBox(fSEFrame,"style",100);
  fHistoSEBox = new TGComboBox(fSEFrame,100);
  fHistoSEBox->AddEntry("HIST",0); 
  fHistoSEBox->AddEntry("CURVE",3);
  fHistoSEBox->AddEntry("BAR",4);  
  fHistoSEBox->AddEntry("LINE",5); 
  fHistoSEBox->AddEntry("LEGO",6); 
  fHistoSEBox->Select(0);
  fHistoSEBox->Resize(70,20);

  f1dFrame->AddFrame(fHisto1d, new TGLayoutHints(kLHintsLeft,2,10,2,2));
  f1dFrame->AddFrame(fHisto1dBox, new TGLayoutHints(kLHintsLeft,2,2,2,2)); 

  f2dFrame->AddFrame(fHisto2d, new TGLayoutHints(kLHintsLeft,2,10,2,2)); 
  f2dFrame->AddFrame(fHisto2dBox, new TGLayoutHints(kLHintsLeft,2,2,2,2));

  f3dFrame->AddFrame(fHisto3d, new TGLayoutHints(kLHintsLeft,2,10,2,2)); 
  f3dFrame->AddFrame(fHisto3dBox, new TGLayoutHints(kLHintsLeft,2,2,2,2));  

  fSEFrame->AddFrame(fHistoSE, new TGLayoutHints(kLHintsLeft,2,10,2,2)); 
  fSEFrame->AddFrame(fHistoSEBox, new TGLayoutHints(kLHintsLeft,2,2,2,2)); 

  fHistoFrame->AddFrame(f1dFrame, new TGLayoutHints(kLHintsLeft,2,2,10,2)); 
  fHistoFrame->AddFrame(f2dFrame, new TGLayoutHints(kLHintsLeft,2,2,2,2)); 
  fHistoFrame->AddFrame(f3dFrame, new TGLayoutHints(kLHintsLeft,2,2,2,2)); 
  fHistoFrame->AddFrame(fSEFrame, new TGLayoutHints(kLHintsLeft,2,2,2,2)); 
  fHistoFrame->MapSubwindows();

  fHistospace->AddFrame(fVarSelection, new TGLayoutHints(kLHintsLeft||kLHintsExpandX,5,5,3,3)); 
  fHistospace->AddFrame(fHistoFrame, new TGLayoutHints(kLHintsLeft||kLHintsExpandX,5,5,3,3));

  fHistospace->MapSubwindows();

  //--------------------------------------------------------------------------
  // line plot space: 
  fLineplotspace = new TGVerticalFrame(fWorkspace, 300,100);

  // group frame for line plots
  fLinePlotFrame = new TGGroupFrame(fLineplotspace, "Line plots",kVerticalFrame);

  fSelLPFrame = new TGHorizontalFrame(fLinePlotFrame);

  fSelLPFrameLeft = new TGVerticalFrame(fSelLPFrame);
  fSelLPLabel1 = new TGLabel(fSelLPFrame,"Independent");
  fSelLPLabel2 = new TGLabel(fSelLPFrame,"variable:");
  fSelLPFrameLeft->AddFrame(fSelLPLabel1, new TGLayoutHints(kLHintsLeft,0,0,0,0)); 
  fSelLPFrameLeft->AddFrame(fSelLPLabel2, new TGLayoutHints(kLHintsRight,0,0,0,0)); 

  fSelLPFrameRight = new TGVerticalFrame(fSelLPFrame);
//  fSelLPBox = new TGComboBox(fSelLPFrameRight,"select",150);
  fSelLPBox = new TGComboBox(fSelLPFrameRight,150);
  fSelLPBox->Resize(70,20);
  fSelLPFrameRight->AddFrame(fSelLPBox, new TGLayoutHints(kLHintsLeft,5,0,5,0)); 

  fSelLPSlider = new TGDoubleHSlider(fLinePlotFrame, 150, kDoubleScaleDownRight, selLPSliderID);
  fSelLPSlider->SetRange(0.,1.);
  fSelLPSlider->SetPosition(0.,1.);
  fSelLPSlider->Connect("Released()", "TH5MainFrame", this, "DoLastPlot()");

  fScalarLPFrame = new TGHorizontalFrame(fLinePlotFrame);

  fScalar = new TGTextButton(fScalarLPFrame,"S&calar");
  fScalar->SetToolTipText("Draw line plot of scalar variable from file");
  fScalar->Connect("Clicked()","TH5MainFrame",this,"ScalarLinePlot()");

//  fScalarLPBox = new TGComboBox(fScalarLPFrame,"scalar",150);
  fScalarLPBox = new TGComboBox(fScalarLPFrame,150);
  fScalarLPBox->Resize(70,20);

  fVectorLPFrame = new TGHorizontalFrame(fLinePlotFrame);

  fVector = new TGTextButton(fVectorLPFrame,"&Vector");
  fVector->SetToolTipText("Draw line plot of vector variable from file");
  fVector->Connect("Clicked()","TH5MainFrame",this,"VectorLinePlot()");

//  fVectorLPBox = new TGComboBox(fVectorLPFrame,"vector",150);
  fVectorLPBox = new TGComboBox(fVectorLPFrame,150);
  fVectorLPBox->Resize(70,20);

  fSelLPFrame->AddFrame(fSelLPFrameLeft, new TGLayoutHints(kLHintsLeft,0,0,0,0)); 
  fSelLPFrame->AddFrame(fSelLPFrameRight, new TGLayoutHints(kLHintsRight,5,0,2,0)); 

  fScalarLPFrame->AddFrame(fScalar, new TGLayoutHints(kLHintsLeft,2,10,2,2)); 
  fScalarLPFrame->AddFrame(fScalarLPBox, new TGLayoutHints(kLHintsLeft,2,2,2,2));

  fVectorLPFrame->AddFrame(fVector, new TGLayoutHints(kLHintsLeft,2,10,2,2)); 
  fVectorLPFrame->AddFrame(fVectorLPBox, new TGLayoutHints(kLHintsLeft,2,2,2,2)); 

  fRadioLPFrame = new TGHorizontalFrame(fLinePlotFrame);
  fRadioLPFrame->AddFrame(fRadioLP[0] = new TGRadioButton(fRadioLPFrame, "x", linePlotXID), 
			new TGLayoutHints(kLHintsLeft,20,2,2,2)); 
  fRadioLPFrame->AddFrame(fRadioLP[1] = new TGRadioButton(fRadioLPFrame, "y", linePlotYID), 
			new TGLayoutHints(kLHintsLeft,2,2,2,2)); 
  fRadioLPFrame->AddFrame(fRadioLP[2] = new TGRadioButton(fRadioLPFrame, "z", linePlotZID), 
			new TGLayoutHints(kLHintsLeft,2,2,2,2)); 
  fRadioLP[0]->Connect("Pressed()", "TH5MainFrame", this, "DoLinePlotRadioButtons()");
  fRadioLP[1]->Connect("Pressed()", "TH5MainFrame", this, "DoLinePlotRadioButtons()");
  fRadioLP[2]->Connect("Pressed()", "TH5MainFrame", this, "DoLinePlotRadioButtons()");

  fRadioLP[0]->SetOn();

  fPartLabel = new TGLabel(fLinePlotFrame,"from particles:");

  fPartLPFrame = new TGHorizontalFrame(fLinePlotFrame);

  fMean = new TGTextButton(fPartLPFrame,"&Mean",linePlotMeanID);
  fMean->SetToolTipText("Draw line plot of mean of selected variable, obtained from particle distributions");
  fMean->Connect("Clicked()","TH5MainFrame",this,"PartLinePlot()");
  fCov = new TGTextButton(fPartLPFrame,"&Sigma",linePlotCovID);
  fCov->SetToolTipText("Draw line plot of square root of (co-)variance of two (one) selected variable(s), obtained from particle distributions");
  fCov->Connect("Clicked()","TH5MainFrame",this,"PartLinePlot()");
  fEmit = new TGTextButton(fPartLPFrame,"&Emitt.",linePlotEmitID);
  fEmit->SetToolTipText("Draw line plot of emittance of two selected variables, obtained from particle distributions");
  fEmit->Connect("Clicked()","TH5MainFrame",this,"PartLinePlot()");

  fPartLPFrame->AddFrame(fMean, new TGLayoutHints(kLHintsLeft,2,5,2,2)); 
  fPartLPFrame->AddFrame(fCov, new TGLayoutHints(kLHintsLeft,2,5,2,2)); 
  fPartLPFrame->AddFrame(fEmit, new TGLayoutHints(kLHintsLeft,2,2,2,2)); 

  fSliceLPFrame = new TGHorizontalFrame(fLinePlotFrame);
  fProjButton = new TGCheckButton(fSliceLPFrame, "proj.");
  fSliceButton = new TGCheckButton(fSliceLPFrame, "slice");
  fScreenButton = new TGCheckButton(fSliceLPFrame, "screen");
  fProjButton->SetState(kButtonDown);

  fSliceLPFrame->AddFrame(fProjButton, new TGLayoutHints(kLHintsLeft,2,10,2,2)); 
  fSliceLPFrame->AddFrame(fSliceButton, new TGLayoutHints(kLHintsLeft,2,10,2,2)); 
  fSliceLPFrame->AddFrame(fScreenButton, new TGLayoutHints(kLHintsLeft,2,10,2,2)); 

  fLinePlotFrame->AddFrame(fSelLPFrame, new TGLayoutHints(kLHintsLeft,2,2,10,2));
  fLinePlotFrame->AddFrame(fSelLPSlider, new TGLayoutHints(kLHintsLeft,2,2,10,2));
  fLinePlotFrame->AddFrame(fScalarLPFrame, new TGLayoutHints(kLHintsLeft,2,2,10,2));
  fLinePlotFrame->AddFrame(fVectorLPFrame, new TGLayoutHints(kLHintsLeft,2,2,2,2));
  fLinePlotFrame->AddFrame(fRadioLPFrame, new TGLayoutHints(kLHintsLeft,2,2,2,2));
  fLinePlotFrame->AddFrame(fPartLabel, new TGLayoutHints(kLHintsLeft,2,2,2,2));
  fLinePlotFrame->AddFrame(fPartLPFrame, new TGLayoutHints(kLHintsLeft,2,2,2,2));
  fLinePlotFrame->AddFrame(fSliceLPFrame, new TGLayoutHints(kLHintsLeft,2,2,2,2));
  fLinePlotFrame->MapSubwindows();

  fLineplotspace->AddFrame(fLinePlotFrame, new TGLayoutHints(kLHintsLeft|kLHintsExpandX,5,5,3,3));
  
  fLineplotspace->MapSubwindows();

  // add the top frames to the workspace:

  fWorkspace->AddFrame(fSelspace,      new TGLayoutHints(kLHintsLeft,5,5,3,3)); 
  fWorkspace->AddFrame(fHistospace,    new TGLayoutHints(kLHintsLeft,5,5,3,3)); 
  fWorkspace->AddFrame(fLineplotspace, new TGLayoutHints(kLHintsLeft,5,5,3,3)); 
  
  fWorkspace->MapSubwindows();
  AddFrame(fWorkspace, new TGLayoutHints(kLHintsLeft|kLHintsExpandX,5,5,5,5));
  
  // Sets window name and shows the main frame
  SetWindowName("H5PartROOT");
  MapSubwindows();
  Resize(GetDefaultSize());
  MapWindow();	
  
  fLastPlot = kNoPlot;
  
}

TH5MainFrame::~TH5MainFrame() {

  delete [] fDataset;

  Cleanup();  // deletes all buttons and frames etc.

}

void TH5MainFrame::LoadFile(const char * filename, const EH5MemOpt memOpt,
			    const Int_t subsampRate) {

  Int_t ssr = subsampRate;
  if (0==ssr) ssr = fgSubsampRate;

  FileStat_t fs;
  Int_t notExist = gSystem->GetPathInfo(filename, fs);
  if (notExist) {
    h5Error(this,"LoadFile", 1,"Dude! There is no file named \"%s\"!",filename);
    return;
  } 

  // find the first available slot:
  Int_t slot(0);
  for (Int_t n=0; n<5; ++n) {
    if (!fDataset[n]) {
      slot = n;
      break;
    }
    if (n==4) {
      h5Error(this,"LoadFile",1, "You have already loaded 5 files, dude!");    
      return;
    }  
  }

  fDataset[slot] = new TH5Dataset(filename, ssr, memOpt, this);

  // update name of check button (strip path):
  TString fname = fDataset[slot]->GetFilename();
  Int_t nSteps = fDataset[slot]->GetNSteps();

  TString bname(fname+TString(" ("));
  bname+=nSteps;
  bname+=TString(" steps)");
  fFileButton[slot]->SetText(bname);

  // ... and check the box!
  fFileButton[slot]->SetState(kButtonDown);  
  // set file to active:
  fDataset[slot]->SetState(kActive);

  // warn if number of steps is larger than for first file:
  if (nSteps != fDataset[0]->GetNSteps())
    h5Warning(this,"LoadFile",1,"Watch out dude!\n The new file has %d steps, whereas the first has %d!",
	      nSteps,fDataset[0]->GetNSteps());

  UpdateSelections();

}

void TH5MainFrame::ActivateFile(Int_t slot) {
  if (fDataset[slot]->GetState() == kNotExist) {
    fFileButton[slot]->SetState(kButtonDisabled);
    return;
  }
  if (fFileButton[slot]->GetState()==kButtonUp)   fDataset[slot]->SetState(kNotActive);
  if (fFileButton[slot]->GetState()==kButtonDown) fDataset[slot]->SetState(kActive);

  UpdateSelections();

}

void TH5MainFrame::UpdateSelections() {

  // update step and variable selection (according to first selected file)
  TH5Dataset* activeData[5];
  Int_t nFiles(0);
  GetSelFiles(activeData,nFiles);

  if (activeData[0]) {

    fStepSlider->SetRange(0,(activeData[0]->GetNSteps())-1);
    fStepSlider->SetPosition(0);

    for(Int_t i=0;i<8;i++){ 
      EButtonState prevState = fVarButton[i]->GetState();
      fVarName[i] = activeData[0]->GetPartVarName(i);
      if (fVarName[i] != TString("")) {
	if (prevState==kButtonDown) {
	  fVarButton[i]->SetState(prevState);
	} else {
	  fVarButton[i]->SetState(kButtonUp);
	}	  
	fVarButton[i]->SetText(activeData[0]->GetPartVarName(i));
      } else {
	fVarName[i] = fVarNameDefault[i];
	fVarButton[i]->SetState(kButtonDisabled);
      }
    }

    Int_t prevSelEntry = fSelLPBox->GetSelected();
    Int_t prevScalarEntry = fScalarLPBox->GetSelected();
    Int_t prevVectorEntry = fVectorLPBox->GetSelected();

    //    fSelLPBox->RemoveAll();  // only in most recent version??
    //    fScalarLPBox->RemoveAll();  // only in most recent version??
    fSelLPBox->RemoveEntries(0,fSelLPBox->GetNumberOfEntries()-1);
    fScalarLPBox->RemoveEntries(0,fScalarLPBox->GetNumberOfEntries()-1);
    Int_t sposID(0);
    for (Int_t is = 0; is < activeData[0]->GetNScalarAttr(); is++) {
      TString vname = activeData[0]->GetScalarStepVarName(is);
      fSelLPBox->AddEntry(vname,is);      
      fScalarLPBox->AddEntry(vname,is);      
      if (vname=="SPOS"||vname=="sPOS"||vname=="Spos"||vname=="spos"||vname=="s") sposID = is;
    }

    //    fVectorLPBox->RemoveAll();  // only in most recent version??
    fVectorLPBox->RemoveEntries(0,fVectorLPBox->GetNumberOfEntries()-1);
    for (Int_t iv = 0; iv < activeData[0]->GetNVectorAttr(); iv++) {
      fVectorLPBox->AddEntry(activeData[0]->GetVectorStepVarName(iv),iv);      
    }

    // restore previous setting if possible
    if (activeData[0]->GetNScalarAttr() > prevSelEntry && prevSelEntry > -1) {
      fSelLPBox->Select(prevSelEntry);
    } else {
      fSelLPBox->Select(sposID);
    }
    if (activeData[0]->GetNScalarAttr() > prevScalarEntry && prevScalarEntry > -1) {
      fScalarLPBox->Select(prevScalarEntry);
    } else {
      fScalarLPBox->Select(0);
    }
    if (activeData[0]->GetNVectorAttr() > prevVectorEntry && prevVectorEntry > -1) {
      fVectorLPBox->Select(prevVectorEntry);
    } else {
      fVectorLPBox->Select(0);      
    }
  }

}


void TH5MainFrame::Histo1d() {

  TH5Dataset* dataToPlot[5];
  Int_t nFiles(0);
  GetSelFiles(dataToPlot,nFiles);

  switch (nFiles) {
  case 0:
    h5Error(this,"Histo1d",1,"You have not selected any files, dude!");
    break;
  case 1:
    Histo1d(dataToPlot[0]);
    break;
  case 2:
    Histo1d(dataToPlot[0],dataToPlot[1]);
    break;
  default:
    h5Error(this,"Histo1d",1, "Sorry dude, histogramming for %d files is not yet implemented!",nFiles);
  } // switch
  
  fLastPlot = kHisto1d;

}

void TH5MainFrame::Histo1d(TH5Dataset* data) {

  Int_t step = fStepSlider->GetPosition();
  Int_t opt = fHisto1dBox->GetSelected();
  TString drawOpt = H5DrawOpt1d[opt];

  TString legEntry = TString("l");
  if (drawOpt=="e1p") legEntry = TString("lp");
  if (drawOpt=="e2p") legEntry = TString("fp");

  TString* varToPlot[8];
  Int_t nVars;
  GetSelVar(varToPlot,nVars);

  TCanvas *canv = fEcanvas->GetCanvas();

  Double_t plotMin(0.);
  Double_t plotMax(0.);

  switch (nVars) {
  case 0:
    h5Error(this,"Histo1d",1, "You have not selected any variables, dude!");
    break;

  case 1:  // nVars == 1
      
    {
      TH1F* hist; 
      if (data) {
	data->PlotRange(varToPlot[0], step, plotMin, plotMax);
	if (plotMin <= 0. && DrawLogX) LogX(0);
	hist = data->Histo1d(varToPlot[0],step,plotMin,plotMax); // deletion taken care of in TH5Dataset
        if (data->GetUnit(*varToPlot[0]) != "") 
	  hist->SetXTitle(*varToPlot[0] + AddBrackets(data->GetUnit(*varToPlot[0])));
      }
      canv->Clear();
      canv->SetRightMargin(0.25);

      hist->SetFillColor(kBlue);
      if (drawOpt=="e2p") hist->SetFillColor(38);
      if (drawOpt=="bar2") {
	hist->SetBarOffset(0.2);
	hist->SetBarWidth(0.75);
	hist->SetFillColor(38);
      }
      hist->Draw(drawOpt);
      
      TH5Legend* leg = new TH5Legend(0.76,0.8,1.,0.95);
      TString header(*varToPlot[0]+TString( "distribution, step "));
      header+=step;
      leg->AddEntry(hist, data->GetFilename(),legEntry);
      leg->SetHeader(header);
      leg->Draw();
      
      canv->Update();
    }
    break;
  case 2:  // nVars == 2
    {
      // need to rename the two histograms...
      TH1F* hist0 = (TH1F*) gROOT->FindObject("hist0");
      if (hist0) delete hist0;
      TH1F* hist1 = (TH1F*) gROOT->FindObject("hist1");
      if (hist1) delete hist1;
      
      if (data) {
	data->PlotRange(varToPlot[0], step, plotMin, plotMax);
	if (plotMin <= 0. && DrawLogX) LogX(0);
	hist0 = data->Histo1d(varToPlot[0],step,plotMin,plotMax);
	hist0->SetName("hist0");
	if (data->GetUnit(*varToPlot[0]) != "") 
	  hist0->SetXTitle(*varToPlot[0] + AddBrackets(data->GetUnit(*varToPlot[0])));
	data->PlotRange(varToPlot[1], step, plotMin, plotMax);
	if (plotMin <= 0. && DrawLogX) LogX(0);
	hist1 = data->Histo1d(varToPlot[1],step,plotMin,plotMax);
	hist1->SetName("hist1");
	if (data->GetUnit(*varToPlot[1]) != "") 
	  hist1->SetXTitle(*varToPlot[1] + AddBrackets(data->GetUnit(*varToPlot[1])));
      }
      
      canv->Clear();
      canv->Divide(2,1);
      
      hist0->SetFillColor(kBlue);hist1->SetFillColor(kBlue);
      if (drawOpt=="e2p") {
	hist0->SetFillColor(38);hist1->SetFillColor(38);
      }
      if (drawOpt=="bar2") {
	hist0->SetBarOffset(0.2);hist1->SetBarOffset(0.2);
	hist0->SetBarWidth(0.75);hist1->SetBarWidth(0.75);
	hist0->SetFillColor(38);hist1->SetFillColor(38);
      }
      
      canv->cd(1);
      hist0->Draw(drawOpt);
      TH5Legend* leg0 = new TH5Legend(0.7,0.75,.9,0.9);
      leg0->AddEntry(hist0, data->GetFilename(),legEntry);
      leg0->SetHeader(*varToPlot[0]+TString(" distribution"));
      leg0->Draw();
      
      canv->cd(2);
      hist1->Draw(drawOpt);
      TH5Legend* leg1 = new TH5Legend(0.7,0.75,.9,0.9);
      leg1->AddEntry(hist1, data->GetFilename(),legEntry);
      leg1->SetHeader(*varToPlot[1]+TString(" distribution"));
      leg1->Draw();
    }
    break;
  default:  // nVars == 3
    h5Error(this,"Histo1d",1, "Sorry dude, histogramming for %d variables is not yet implemented!",nVars);
  }//switch
  
  canv->Update();

}

void TH5MainFrame::Histo1d(TH5Dataset* data0, TH5Dataset* data1) {

  Int_t step = fStepSlider->GetPosition();
  Int_t opt = fHisto1dBox->GetSelected();

  TString drawOpt = H5DrawOpt1d[opt];
  TString drawOptSame = H5DrawOpt1d[opt]+TString("same");

  TString legEntry = TString("l");
  if (drawOpt=="e1p") legEntry = TString("lp");
  if (drawOpt=="e2p") legEntry = TString("fp");

  if (opt!=0) h5Warning(this,"Histo1d",1, 
			"Drawing option %s may give weird results for two files, dude!", 
			drawOpt.Data());

  TString* varToPlot[8];
  Int_t nVars;
  GetSelVar(varToPlot,nVars);

  TCanvas *canv = fEcanvas->GetCanvas();
  canv->Clear();
  canv->SetRightMargin(0.25);
  
  Double_t plotMin(0.);
  Double_t plotMax(0.);
  Double_t min0(0.);
  Double_t max0(0.);
  Double_t min1(0.);
  Double_t max1(0.);

  switch (nVars) {
  case 0:
    h5Error(this,"Histo1d",1, "No variables have been selected, dude!");
    break;

  case 1:  { // nVars == 1
    TH1F* hist0 = (TH1F*) gROOT->FindObject("hist0");
    if (hist0) delete hist0;
    TH1F* hist1 = (TH1F*) gROOT->FindObject("hist1");
    if (hist1) delete hist1;

    if (data0) data0->Range(varToPlot[0], step, min0, max0);
    if (data1) data1->Range(varToPlot[0], step, min1, max1);
    plotMin =  std::min(min0,min1);  
    plotMax =  std::max(max0,max1); 
    AddMargin(plotMin,plotMax);
    if (plotMin <= 0. && DrawLogX) LogX(0);
      
    if (data0) hist0 = data0->Histo1d(varToPlot[0], step, plotMin, plotMax);
    hist0->SetName("hist0");
    if (data1) hist1 = data1->Histo1d(varToPlot[0], step, plotMin, plotMax);
    hist1->SetName("hist1");    

    if (data0->GetUnit(*varToPlot[0]) == data1->GetUnit(*varToPlot[0])) {
      if (data0->GetUnit(*varToPlot[0]) != "") {
	hist0->SetXTitle(*varToPlot[0] + AddBrackets(data0->GetUnit(*varToPlot[0])));
      }
    } else {
      h5Warning(this,"Histo1d",1,"Inconsistent units between files %s and %s, dude!",
		data0->GetFilename().Data(),data1->GetFilename().Data());
      if (data0->GetUnit(*varToPlot[0]) != "") {
	hist0->SetXTitle(*varToPlot[0] + AddBrackets(data0->GetUnit(*varToPlot[0])));
      } else if (data0->GetUnit(*varToPlot[0]) != "") {
	hist0->SetXTitle(*varToPlot[0] + AddBrackets(data0->GetUnit(*varToPlot[0])));
      }
    }

    if (drawOpt=="e2p") {
      hist0->SetFillColor(35);hist1->SetFillColor(38);
    }

    hist1->SetLineStyle(kDashed);
    hist1->SetMarkerStyle(25);

    if (hist0->GetMaximum() > hist1->GetMaximum()) {
      hist0->Draw(drawOpt); hist1->Draw(drawOptSame);
    } else {
      hist1->Draw(drawOpt); hist0->Draw(drawOptSame);
    }

    TH5Legend* leg = new TH5Legend(0.76,0.8,1.,0.95);
    leg->AddEntry(hist0, data0->GetFilename(),legEntry);
    leg->AddEntry(hist1, data1->GetFilename(),legEntry);
    leg->SetHeader(*varToPlot[0]+TString(" distribution"));
    leg->Draw();
    
    break; }

  case 2:  { // nVars == 2

    TH1F* hist00 = (TH1F*) gROOT->FindObject("hist00");
    if (hist00) delete hist00;
    TH1F* hist01 = (TH1F*) gROOT->FindObject("hist01");
    if (hist01) delete hist01;
    TH1F* hist10 = (TH1F*) gROOT->FindObject("hist10");
    if (hist10) delete hist10;
    TH1F* hist11 = (TH1F*) gROOT->FindObject("hist11");
    if (hist11) delete hist11;

    if (data0) data0->Range(varToPlot[0], step, min0, max0);
    if (data1) data1->Range(varToPlot[0], step, min1, max1);
    plotMin =  std::min(min0,min1);  
    plotMax =  std::max(max0,max1);  
    AddMargin(plotMin,plotMax);
    if (plotMin <= 0. && DrawLogX) LogX(0);

    if (data0) hist00 = data0->Histo1d(varToPlot[0], step, plotMin, plotMax);
    hist00->SetName("hist00");
    if (data1) hist10 = data1->Histo1d(varToPlot[0], step, plotMin, plotMax);
    hist10->SetName("hist10");

    if (data0->GetUnit(*varToPlot[0]) == data1->GetUnit(*varToPlot[0])) {
      if (data0->GetUnit(*varToPlot[0]) != "") {
	hist00->SetXTitle(*varToPlot[0] + AddBrackets(data0->GetUnit(*varToPlot[0])));
      }
    } else {
      h5Warning(this,"Histo1d",1,"Inconsistent units between files %s and %s, dude!",
	      data0->GetFilename().Data(),data1->GetFilename().Data());
      if (data0->GetUnit(*varToPlot[0]) != "") {
	hist00->SetXTitle(*varToPlot[0] + AddBrackets(data0->GetUnit(*varToPlot[0])));
      } else if (data0->GetUnit(*varToPlot[0]) != "") {
	hist00->SetXTitle(*varToPlot[0] + AddBrackets(data0->GetUnit(*varToPlot[0])));
      }
    }

    if (data0) data0->Range(varToPlot[1], step, min0, max0);
    if (data1) data1->Range(varToPlot[1], step, min1, max1);
    plotMin =  std::min(min0,min1);  
    plotMax =  std::max(max0,max1);  
    AddMargin(plotMin,plotMax);
    if (plotMin <= 0. && DrawLogX) LogX(0);

    if (data0) hist01 = data0->Histo1d(varToPlot[1], step, plotMin, plotMax);
    hist01->SetName("hist01");
    if (data1) hist11 = data1->Histo1d(varToPlot[1], step, plotMin, plotMax);
    hist11->SetName("hist11");

    if (data0->GetUnit(*varToPlot[1]) == data1->GetUnit(*varToPlot[1])) {
      if (data0->GetUnit(*varToPlot[1]) != "") {
	hist01->SetXTitle(*varToPlot[1] + AddBrackets(data0->GetUnit(*varToPlot[1])));
      }
    } else {
      h5Warning(this,"Histo1d",1,"Inconsistent units between files %s and %s, dude!",
	      data0->GetFilename().Data(),data1->GetFilename().Data());
      if (data0->GetUnit(*varToPlot[1]) != "") {
	hist01->SetXTitle(*varToPlot[1] + AddBrackets(data0->GetUnit(*varToPlot[1])));
      } else if (data0->GetUnit(*varToPlot[1]) != "") {
	hist01->SetXTitle(*varToPlot[1] + AddBrackets(data0->GetUnit(*varToPlot[1])));
      }
    }

    if (drawOpt=="e2p") {
      hist00->SetFillColor(35);hist10->SetFillColor(38);
      hist01->SetFillColor(35);hist11->SetFillColor(38);
    }

    canv->Clear();
    canv->Divide(2,1);

    canv->cd(1);

    hist10->SetLineStyle(kDashed);
    hist10->SetMarkerStyle(25);

    if (hist00->GetMaximum() > hist10->GetMaximum()) {
      hist00->Draw(drawOpt); hist10->Draw(drawOptSame);
    } else {
      hist10->Draw(drawOpt); hist00->Draw(drawOptSame);
    }

    TH5Legend* leg0 = new TH5Legend(0.7,0.75,.9,0.9);
    leg0->AddEntry(hist00, data0->GetFilename(),legEntry);
    leg0->AddEntry(hist10, data1->GetFilename(),legEntry);
    leg0->SetHeader(*varToPlot[0]+TString(" distribution"));
    leg0->Draw();

    canv->cd(2);

    hist11->SetLineStyle(kDashed);
    hist11->SetMarkerStyle(25);

    if (hist01->GetMaximum() > hist11->GetMaximum()) {
      hist01->Draw(drawOpt); hist11->Draw(drawOptSame);
    } else {
      hist11->Draw(drawOpt); hist01->Draw(drawOptSame);
    }

    TH5Legend* leg1 = new TH5Legend(0.7,0.75,.9,0.9);
    leg1->AddEntry(hist01, data0->GetFilename(),legEntry);
    leg1->AddEntry(hist11, data1->GetFilename(),legEntry);
    leg1->SetHeader(*varToPlot[1] + TString(" distribution"));
    leg1->Draw();

    break; }
  default:  // nVars == 3
    h5Error(this,"Histo1d",1, "Sorry dude, histogramming for %d variables is not yet implemented",nVars);
  } //switch
  
  canv->Update();


}

void TH5MainFrame::Histo2d() {

  TH5Dataset* dataToPlot[5];
  Int_t nFiles(0);
  GetSelFiles(dataToPlot,nFiles);

  switch (nFiles) {
  case 0:
    h5Error(this,"Histo2d",1, "You have not selected any files, dude!");
    break;
  case 1:
    Histo2d(dataToPlot[0]);
    break;
  default:
    h5Error(this,"Histo2d",1, "Sorry dude, 2d histogramming for %d files is not yet implemented!",nFiles);
  } // switch
  
  fLastPlot = kHisto2d;

}

void TH5MainFrame::Histo2d(TH5Dataset* data) {

  Int_t step = fStepSlider->GetPosition();
  Int_t opt = fHisto2dBox->GetSelected();

  TString* varToPlot[8];
  Int_t nVars;
  GetSelVar(varToPlot,nVars);

  TCanvas *canv = fEcanvas->GetCanvas();

  canv->Clear();
  if (H5DrawOpt2d[opt]=="colz"||H5DrawOpt2d[opt]=="contz"||H5DrawOpt2d[opt]=="surf2z") {
    canv->SetRightMargin(0.35);
  } else {
    canv->SetRightMargin(0.25);
  }

  Double_t plotMinX(0.);
  Double_t plotMaxX(0.);
  Double_t plotMinY(0.);
  Double_t plotMaxY(0.);

  switch (nVars) {
  case 0:
    h5Error(this,"Histo2d",1, "You have not selected any variables, dude!");
    break;

  case 1:  // nVars == 1
      
    {
      TH2F* hist;
      if (data) {
	data->PlotRange(varToPlot[0], step, plotMinX, plotMaxX);
	if (plotMinX <= 0. && DrawLogX) LogX(0);
	hist = data->Histo2d(varToPlot[0],varToPlot[0],step,plotMinX,plotMaxX,plotMinX,plotMaxX);
        if (data->GetUnit(*varToPlot[0]) != "") {
	  hist->SetXTitle(*varToPlot[0] + AddBrackets(data->GetUnit(*varToPlot[0])));
	  hist->SetYTitle(*varToPlot[0] + AddBrackets(data->GetUnit(*varToPlot[0])));
	}
      }
      hist->SetMarkerStyle(1);

      if (H5DrawOpt2d[opt]=="box") {
	hist->SetFillColor(kBlue);
	hist->SetFillStyle(1);  
      }	

      if (H5DrawOpt2d[opt]=="unbinned") { // true scatter plot
	canv->Clear();
	TH1F* shist = data->Scatter(varToPlot[0],varToPlot[0],canv,step,plotMinX,plotMaxX,plotMinX,plotMaxX);
        if (data->GetUnit(*varToPlot[0]) != "") {
	  shist->SetXTitle(*varToPlot[0] + AddBrackets(data->GetUnit(*varToPlot[0])));
	  shist->SetYTitle(*varToPlot[0] + AddBrackets(data->GetUnit(*varToPlot[0])));
	}
      } else {
	hist->Draw(H5DrawOpt2d[opt]);
      }
      
      TH5Legend* leg = new TH5Legend(0.76,0.8,1.,0.95);
      TString header(*varToPlot[0]+TString("-") + *varToPlot[0] + 
		     TString(" distr., step "));
      header+=step;
      leg->AddEntry(hist, data->GetFilename(),"p");
      leg->SetHeader(header);
      leg->Draw();
      
      canv->Update();
    }
    break;
  case 2:  // nVars == 2

    {
      TH2F* hist;
      if (data) {
	data->PlotRange(varToPlot[0], step, plotMinX, plotMaxX);
	if (plotMinX <= 0. && DrawLogX) LogX(0);
      	data->PlotRange(varToPlot[1], step, plotMinY, plotMaxY);
	if (plotMinY <= 0. && DrawLogY) LogY(0);
	hist = data->Histo2d(varToPlot[0],varToPlot[1],step,plotMinX,plotMaxX,plotMinY,plotMaxY);
        if (data->GetUnit(*varToPlot[0]) != "") 
	  hist->SetXTitle(*varToPlot[0] + AddBrackets(data->GetUnit(*varToPlot[0])));
        if (data->GetUnit(*varToPlot[1]) != "") 
	  hist->SetYTitle(*varToPlot[1] + AddBrackets(data->GetUnit(*varToPlot[1])));
      }
      hist->SetMarkerStyle(1);
      
      if (H5DrawOpt2d[opt]=="box") {
	hist->SetFillColor(kBlue);
	hist->SetFillStyle(1);  
      }	
	
      if (H5DrawOpt2d[opt]=="unbinned") { // true scatter plot
	canv->Clear();
	TH1F* shist = data->Scatter(varToPlot[0],varToPlot[1],canv,step,plotMinX,plotMaxX,plotMinY,plotMaxY);
        if (data->GetUnit(*varToPlot[0]) != "") 
	  shist->SetXTitle(*varToPlot[0] + AddBrackets(data->GetUnit(*varToPlot[0])));
        if (data->GetUnit(*varToPlot[1]) != "") 
	  shist->SetYTitle(*varToPlot[1] + AddBrackets(data->GetUnit(*varToPlot[1])));
      } else {
	hist->Draw(H5DrawOpt2d[opt]);
      }

      TH5Legend* leg = new TH5Legend(0.76,0.8,1.,0.95);
      TString header(*varToPlot[0] + TString("-") + *varToPlot[1] + 
		     TString(" distr., step "));
      header+=step;
      leg->AddEntry(hist, data->GetFilename(),"p");
      leg->SetHeader(header);
      leg->Draw();
            
      canv->Update();
    }
    break;
  case 3:  // nVars == 3
    h5Error(this,"Histo2d",1, "Sorry dude, cannot produce 2d histogram with %d variables!",nVars);
    break;
  case 4:  // nVars == 4
    h5Error(this,"Histo2d",1, "2d histogram for %d variables not yet implemented",nVars);
    break;
  case 5:  // nVars == 5
    h5Error(this,"Histo2d",1, "Sorry dude, cannot produce 2d histogram with %d variables!",nVars);
    break;
  case 6:  // nVars == 6
    h5Error(this,"Histo2d",1, "2d histogram for %d variables not yet implemented",nVars);
    break;
  case 7:  // nVars == 7
    h5Error(this,"Histo2d",1, "Sorry dude, cannot produce 2d histogram with %d variables!",nVars);
    break;
  case 8:  // nVars == 8
    h5Error(this,"Histo2d",1, "2d histogram for %d variables not yet implemented",nVars);
    break;
  default:
    cout << "Huh?" << endl;
  }//switch

}

void TH5MainFrame::Histo3d() {

  TH5Dataset* dataToPlot[5];
  Int_t nFiles(0);
  GetSelFiles(dataToPlot,nFiles);

  switch (nFiles) {
  case 0:
    h5Error(this,"Histo3d",1, "You have not selected any files, dude!");
    break;
  case 1:
    Histo3d(dataToPlot[0]);
    break;
  default:
    h5Error(this,"Histo3d",1, 
	    "Sorry dude, 3d histogramming for %d files is not yet implemented\n(why would you want that anyway?)",
	    nFiles);
  } // switch
  
  fLastPlot = kHisto3d;

}

void TH5MainFrame::Histo3d(TH5Dataset* data) {

  Int_t step = fStepSlider->GetPosition();
  Int_t opt = fHisto3dBox->GetSelected();

  TString* varToPlot[8];
  Int_t nVars;
  GetSelVar(varToPlot,nVars);

  TCanvas *canv = fEcanvas->GetCanvas();

  canv->Clear();
  canv->SetRightMargin(0.50);  // is ignored by TH3F...

  Double_t plotMinX(0.);
  Double_t plotMaxX(0.);
  Double_t plotMinY(0.);
  Double_t plotMaxY(0.);
  Double_t plotMinZ(0.);
  Double_t plotMaxZ(0.);

  switch (nVars) {
  case 0:
    h5Error(this,"Histo3d",1, "You have not selected any variables, dude!");
    break;

  case 1:  // nVars == 1
    h5Error(this,"Histo3d",1, "Cannot produce 3d histogram with %d variables, dude!",nVars);
    break;
  case 2:  // nVars == 2
    h5Error(this,"Histo3d",1, "Cannot produce 3d histogram with %d variables, dude!",nVars);
    break;
  case 3:  // nVars == 3
    {

      TH3F* hist;
      if (data) {
	data->PlotRange(varToPlot[0], step, plotMinX, plotMaxX);
	if (plotMinX <= 0. && DrawLogX) LogX(0);
	data->PlotRange(varToPlot[1], step, plotMinY, plotMaxY);
	if (plotMinY <= 0. && DrawLogY) LogY(0);
	data->PlotRange(varToPlot[2], step, plotMinZ, plotMaxZ);
	if (plotMinZ <= 0. && DrawLogZ) LogZ(0);
	hist = data->Histo3d(varToPlot[0],varToPlot[1],varToPlot[2],step,
			     plotMinX,plotMaxX,plotMinY,plotMaxY,plotMinZ,plotMaxZ);
        if (data->GetUnit(*varToPlot[0]) != "") 
	  hist->SetXTitle(*varToPlot[0] + AddBrackets(data->GetUnit(*varToPlot[0])));
        if (data->GetUnit(*varToPlot[1]) != "") 
	  hist->SetYTitle(*varToPlot[1] + AddBrackets(data->GetUnit(*varToPlot[1])));
        if (data->GetUnit(*varToPlot[2]) != "") 
	  hist->SetZTitle(*varToPlot[2] + AddBrackets(data->GetUnit(*varToPlot[2])));
      }
      hist->SetMarkerStyle(6);
      hist->SetMarkerColor(kBlue);

      hist->Draw(H5DrawOpt3d[opt]);

      TH5Legend* leg = new TH5Legend(0.76,0.8,1.,0.95);
      TString header(*varToPlot[0] + TString("-") + *varToPlot[1] +
		     TString("-") + *varToPlot[2] + TString(" distr., step ")); 
      header += step;
      leg->AddEntry(hist, data->GetFilename(),"p");
      leg->SetHeader(header);
      leg->Draw();
      
      canv->Update();
    }
    break;
  default:  // nVars > 3
    h5Error(this,"Histo3d",1, "Cannot produce 3d histogram for %d variables, dude!",nVars);
  }//switch

}

void TH5MainFrame::HistoSliceEmit() {

  TH5Dataset* dataToPlot[5];
  Int_t nFiles(0);
  GetSelFiles(dataToPlot,nFiles);

  switch (nFiles) {
  case 0:
    h5Error(this,"HistoSliceEmit",1,"You have not selected any files, dude!");
    break;
  case 1:
    HistoSliceEmit(dataToPlot[0]);
    break;
  case 2:
  default:
    h5Error(this,"HistoSliceEmit",1, "Sorry dude, histogramming for %d files is not yet implemented!",nFiles);
  } // switch

  fLastPlot = kHistoSE;

}

void TH5MainFrame::HistoSliceEmit(TH5Dataset * data) {

  Int_t step = fStepSlider->GetPosition();
  Int_t opt = fHistoSEBox->GetSelected();

  TString drawOpt = H5DrawOpt1d[opt];

  TString legEntry = TString("l");

  TString* varToPlot[8];
  Int_t nVars;
  GetSelVar(varToPlot,nVars);

  TCanvas *canv = fEcanvas->GetCanvas();

  TH1F* hist;
  switch (nVars) {
  case 0:
    h5Error(this,"HistoSliceEmit",1, "You have not selected any variables, dude!");
    return;

  case 1:  // nVars == 1
    {
      if (data) {
	if (data->GetSliceExtentOpt()==kPercent) {
	  h5Error(this,"HistoSliceEmit",1, "Sorry dude, slice emittance histogram not possible for this slice option!");
	  return;
	} else {
	  hist = data->HistoSliceEmit(varToPlot[0],varToPlot[0],step);
	}
      }
    }
    break;
  case 2:  // nVars == 2
    {
      if (data) {
	if (data->GetSliceExtentOpt()==kPercent) {
	  h5Error(this,"HistoSliceEmit",1, "Sorry dude, slice emittance histogram not possible for this slice option!");
	  return;
	} else {
	  hist = data->HistoSliceEmit(varToPlot[0],varToPlot[1],step);
	}
      }
    }
    break;
  default:  // nVars == 3
    h5Error(this,"HistoSliceEmit",1, "Sorry dude, slice emittance histogram for %d variables is not possible!",nVars);
    return;
  }//switch
  
  canv->Clear();
  canv->SetRightMargin(0.25);
  
  if (data->GetUnit("z") != "")
    hist->SetXTitle("z" + AddBrackets(data->GetUnit("z")));

  hist->SetFillColor(kRed);
  if (drawOpt=="bar2") {
    hist->SetBarOffset(0.2); 
    hist->SetBarWidth(0.75);
    hist->SetFillColor(46);
  }

  hist->Draw(drawOpt);
  
  TH5Legend* leg = new TH5Legend(0.76,0.8,1.,0.95);
  TString header("slice emitt., step "); header+=step; 
  leg->AddEntry(hist, data->GetFilename(),legEntry);
  leg->SetHeader(header);
  leg->Draw();
      
  canv->Update();

}

void TH5MainFrame::ScalarLinePlot() {

  // determine active datafiles
  TH5Dataset* dataToPlot[5];
  Int_t nFiles(0);
  GetSelFiles(dataToPlot,nFiles);
    
  // get selected variables
  TString stepVarNameX;
  TString stepVarNameY;
  if (nFiles) {
    stepVarNameX = TString(fSelLPBox->GetSelectedEntry()->GetTitle());
    stepVarNameY = TString(fScalarLPBox->GetSelectedEntry()->GetTitle());
  }

  switch (nFiles) {
  case 0:
    h5Error(this,"ScalarLinePlot",1, "You have not selected any files, dude!");
    break;
  case 1:
  case 2:
  case 3:
  case 4:
  case 5:
    LinePlot(dataToPlot, nFiles, stepVarNameX, stepVarNameY, kScalar);
    break;
  default:
    h5Error(this,"ScalarLinePlot",1, "Sorry dude, LinePlot for %d files is not yet implemented!",nFiles);
  } // switch

  fLastPlot = kScalarLinePlot;
}

void TH5MainFrame::VectorLinePlot() {

  // determine active datafiles
  TH5Dataset* dataToPlot[5];
  Int_t nFiles(0);
  GetSelFiles(dataToPlot,nFiles);
    
  // get selected variables
  TString stepVarNameX;
  TString stepVarNameY;
  if (nFiles) {
    stepVarNameX = TString(fSelLPBox->GetSelectedEntry()->GetTitle());
    stepVarNameY = TString(fVectorLPBox->GetSelectedEntry()->GetTitle());
  }

  // get selected variable type (coordinate):
  EH5VarType vtype;

  if (kButtonDown == fRadioLP[2]->GetState()) vtype = kVectorZ; 
  if (kButtonDown == fRadioLP[1]->GetState()) vtype = kVectorY; 
  if (kButtonDown == fRadioLP[0]->GetState()) vtype = kVectorX; 

  switch (nFiles) {
  case 0:
    h5Error(this,"VectorLinePlot",1, "You have not selected any files, dude!");
    break;
  case 1:
  case 2:
  case 3:
  case 4:
  case 5:
    LinePlot(dataToPlot, nFiles, stepVarNameX, stepVarNameY, vtype);
    break;
  default:
    h5Error(this,"VectorLinePlot",1, "Sorry dude, LinePlot for %d files is not yet implemented",nFiles);
  } // switch

  fLastPlot = kVectorLinePlot;
}

void TH5MainFrame::PartLinePlot(Int_t id) {

  // if pushed from a button, which one was it?
  if (id < 0) {
    TGButton *btn = (TGButton *) gTQSender;
    id = btn->WidgetId();
  }  

  // how many and which datafiles have been selected?
  TH5Dataset* dataToPlot[5];
  Int_t nFiles(0);
  GetSelFiles(dataToPlot,nFiles);

  // how many and which variables have been selected?
  TString* varToPlot[8];
  Int_t nVars;
  GetSelVar(varToPlot,nVars);

  // what is the independent variable?
  TString stepVarNameX;
  if (nFiles) {
    stepVarNameX = TString(fSelLPBox->GetSelectedEntry()->GetTitle());
  }
  
  // is the "projected" option selected?
  Bool_t proj(kFALSE); 
  if (fProjButton->GetState()==kButtonDown) proj = kTRUE;

  // is the "slice" option selected?
  Bool_t slice(kFALSE); 
  if (fSliceButton->GetState()==kButtonDown) slice = kTRUE;

  // is the "screen" option selected?
  Bool_t screen(kFALSE); 
  if (fScreenButton->GetState()==kButtonDown) screen = kTRUE;

  EH5VarType vtype = kPart;

  if (!proj &&  slice && !screen) vtype = kPartSlice;
  if (!proj && !slice &&  screen) vtype = kPartScreen;
  if ( proj &&  slice && !screen) vtype = kPartProjAndSlice;
  if ( proj && !slice &&  screen) vtype = kPartProjAndScreen;
  if (!proj &&  slice &&  screen) vtype = kPartSliceAndScreen;
  if ( proj &&  slice &&  screen) vtype = kPartProjAndSliceAndScreen;
  if (!proj && !slice && !screen) {
    h5Error(this,"PartLinePlot",1, "Dude!! You must select at least one of proj., slice or screen!");
    return;
  }

  TString stepVarNameY("");

  switch (id) {
  case linePlotMeanID:  
    stepVarNameY+=TString("#mu");
    fLastPlot = kMeanLinePlot;
    break;
  case linePlotCovID:  
    stepVarNameY+=TString("#sigma");
    fLastPlot = kCovLinePlot;
    break;
  case linePlotEmitID:  
    stepVarNameY+=TString("#varepsilon");
    fLastPlot = kEmitLinePlot;
    break;
  default:
    cout << "Huh?" << endl;
  } // switch

  stepVarNameY+=TString("(");
  switch (nVars) {
  case 0:
    h5Error(this,"PartLinePlot",1, "You have not selected any variables, dude!");
    return;
    break;
  case 1:  // nVars == 1
    stepVarNameY += *varToPlot[0];  
    break;
  case 2:  // nVars == 2
    if (id==linePlotMeanID) {
      h5Error(this,"PartLinePlot",1, "One variable only, dude!");
      return;
    } else {
      stepVarNameY += *varToPlot[0];
      stepVarNameY+=TString(",");  
      stepVarNameY += *varToPlot[1];  
    }
    break;
  default:
    if (id==linePlotMeanID) {
      h5Error(this,"PartLinePlot",1, "One variable only, dude!");
      return;
    } else { 
      h5Error(this,"PartLinePlot",1, "One or two variables only, dude!");
      return;
    }
  }    

  stepVarNameY+=TString(")");

  switch (nFiles) {
  case 0:
    h5Error(this,"PartLinePlot",1, "You have not selected any files, dude!");
    return;
    break;
  case 1:
  case 2:
  case 3:
  case 4:
  case 5:
    LinePlot(dataToPlot, nFiles, stepVarNameX,stepVarNameY,vtype);
    break;
  default:
    h5Error(this,"PartLinePlot",1, "Sorry dude, LinePlot for %d files is not implemented!",nFiles);
    return;
  } // switch

}

void TH5MainFrame::PartLinePlotTwo(TH5Dataset* data, 
				   const TString stepVarNameX, 
				   const TString stepVarNameY,
				   const EH5VarType y0type,
				   const EH5VarType y1type)
{
  // version of PartLinePlot for plotting two variables (from the same dataset) at once
  Int_t nStep = data->GetNSteps();

  Double_t varX[nStep];
  Double_t varY0[nStep];
  Double_t varY1[nStep];

  Double_t varXMin = 1.e20; 
  Double_t varXMax = -1.e20;
  Double_t varY0Min = 1.e20;
  Double_t varY0Max = -1.e20;
  Double_t varY1Min = 1.e20;
  Double_t varY1Max = -1.e20;

  TCanvas *canv = fEcanvas->GetCanvas();
  canv->SetRightMargin(0.25);

  if (!data->GetStepVarArray(stepVarNameX, varX, kScalar)) {
    canv->Clear();
    return;
  }
  if (!data->GetStepVarArray(stepVarNameY, varY0, y0type)) {
    canv->Clear();
    return;
  }
  if (!data->GetStepVarArray(stepVarNameY, varY1, y1type)) {
    canv->Clear();
    return;
  }

  // find min. and max. for x:
  for(Int_t step = 0; step < nStep; step++){ 
    varXMin = std::min(varXMin,varX[step]);
    varXMax = std::max(varXMax,varX[step]);
  }

  // adapt min. and max. for x to slider positions:
  Double_t upper = fSelLPSlider->GetMaxPosition();
  Double_t lower = fSelLPSlider->GetMinPosition();

  varXMin = varXMin + lower*(varXMax-varXMin);
  varXMax = varXMin + upper*(varXMax-varXMin);

  // find corresponding min. and max. for y1 and y2:
  for(Int_t step = 0; step < nStep; step++){ 
    if (varX[step] <= varXMax && varX[step] >= varXMin) {
      varY0Min = std::min(varY0[step],varY0Min);
      varY0Max = std::max(varY0[step],varY0Max);
      varY1Min = std::min(varY1[step],varY1Min);
      varY1Max = std::max(varY1[step],varY1Max);
    }
  }

  Double_t varYMin = std::min(varY0Min,varY1Min);
  Double_t varYMax = std::max(varY0Max,varY1Max);

  if (varYMin >  1.e19) varYMin = -0.9;
  if (varYMax < -1.e19) varYMax = +0.9;

  AddMargin(varXMin,varXMax);
  AddMargin(varYMin,varYMax);

  TH2F *linePlot = static_cast<TH2F*>(gROOT->FindObject("linePlot"));
  if (linePlot) delete linePlot;

  linePlot = new TH2F("linePlot","line plot",
		      200,varXMin,varXMax,200,varYMin,varYMax);
  canv->Clear();
  linePlot->SetStats(kFALSE);
  linePlot->Draw();

  TGraph* varGraph0 = static_cast<TGraph*>(gROOT->FindObject("varGraph0"));
  if (varGraph0) delete varGraph0;

  varGraph0 = new TGraph(nStep,varX,varY0);
  varGraph0->SetName("varGraph0");
  varGraph0->SetMarkerStyle(20);
  varGraph0->SetLineStyle(kSolid);
  varGraph0->SetMarkerColor(2);
  varGraph0->SetLineColor(2);

  TGraph* varGraph1 = static_cast<TGraph*>(gROOT->FindObject("varGraph1"));
  if (varGraph1) delete varGraph1;

  varGraph1 = new TGraph(nStep,varX,varY1);
  varGraph1->SetName("varGraph1");
  varGraph1->SetMarkerStyle(25);
  varGraph1->SetLineStyle(kDashed);
  varGraph1->SetMarkerColor(3);
  varGraph1->SetLineColor(3);

  varGraph0->Draw("LP");
  varGraph1->Draw("LPsame");

  TString varXTitle = stepVarNameX;
  TString varYTitle = stepVarNameY;

  if (data->GetUnit(stepVarNameX) != "") 
    varXTitle += AddBrackets(data->GetUnit(stepVarNameX));
  if (data->GetUnit(stepVarNameY) != "") 
    varYTitle += AddBrackets(data->GetUnit(stepVarNameY));

  TH5Legend* leg = new TH5Legend(0.76,0.8,1.,0.95);
  TString legName0, legName1;
  if (y0type == kPart)
    legName0 = "proj.";
  else if (y0type == kPartSlice)
    legName0 = "slice";
  else if (y0type == kPartScreen)
    legName0 = "screen";
  else {
    cout<< "Huh?" << endl;
    legName0 = "*****";
  }
  if (y1type == kPart)
    legName1 = "proj.";
  else if (y1type == kPartSlice)
    legName1 = "slice";
  else if (y1type == kPartScreen)
    legName1 = "screen";
  else {
    cout<< "Huh?" << endl;
    legName1 = "*****";
  }
  leg->AddEntry(varGraph0, legName0,"lp");
  leg->AddEntry(varGraph1, legName1,"lp");
  leg->SetHeader(data->GetFilename());
  leg->Draw();

  linePlot->SetYTitle(varYTitle);
  linePlot->SetXTitle(varXTitle);
  canv->Update();
}

void TH5MainFrame::PartLinePlotThree(TH5Dataset* data, 
				     const TString stepVarNameX, 
				     const TString stepVarNameY,
				     const EH5VarType y0type,
				     const EH5VarType y1type,
				     const EH5VarType y2type)
{
  // version of LinePlot for plotting three variables at once
  Int_t nStep = data->GetNSteps();

  Double_t varX[nStep];
  Double_t varY0[nStep];
  Double_t varY1[nStep];
  Double_t varY2[nStep];

  Double_t varXMin = 1.e20; 
  Double_t varXMax = -1.e20;
  Double_t varY0Min = 1.e20;
  Double_t varY0Max = -1.e20;
  Double_t varY1Min = 1.e20;
  Double_t varY1Max = -1.e20;
  Double_t varY2Min = 1.e20;
  Double_t varY2Max = -1.e20;

  TCanvas *canv = fEcanvas->GetCanvas();
  canv->SetRightMargin(0.25);

  if (!data->GetStepVarArray(stepVarNameX, varX, kScalar)) {
    canv->Clear();
    return;
  }
  if (!data->GetStepVarArray(stepVarNameY, varY0, y0type)) {
    canv->Clear();
    return;
  }
  if (!data->GetStepVarArray(stepVarNameY, varY1, y1type)) {
    canv->Clear();
    return;
  }
  if (!data->GetStepVarArray(stepVarNameY, varY2, y2type)) {
    canv->Clear();
    return;
  }

  // find min. and max. for x:
  for(Int_t step = 0; step < nStep; step++){ 
    if (varX[step] < varXMin) varXMin = varX[step];
    if (varX[step] > varXMax) varXMax = varX[step];
  }

  // adapt min. and max. for x to slider positions:
  Double_t upper = fSelLPSlider->GetMaxPosition();
  Double_t lower= fSelLPSlider->GetMinPosition();

  varXMin = varXMin + lower*(varXMax-varXMin);
  varXMax = varXMin + upper*(varXMax-varXMin);

  // find corresponding min. and max. for y0, y1 and y2:
  for(Int_t step = 0; step < nStep; step++){ 
    if (varX[step] <= varXMax && varX[step] >= varXMin) {
      varY0Min = std::min(varY0[step],varY0Min);
      varY0Max = std::max(varY0[step],varY0Max);
      varY1Min = std::min(varY1[step],varY1Min);
      varY1Max = std::max(varY1[step],varY1Max);
      varY2Min = std::min(varY2[step],varY2Min);
      varY2Max = std::max(varY2[step],varY2Max);
     }
  }

  Double_t varYMin = std::min(varY0Min,varY1Min);
  varYMin = std::min(varYMin,varY2Min);
  Double_t varYMax = std::max(varY0Max,varY1Max);
  varYMax = std::max(varYMax,varY2Max);

  if (varYMin >  1.e19) varYMin = -0.9;
  if (varYMax < -1.e19) varYMax = +0.9;

  AddMargin(varXMin,varXMax);
  AddMargin(varYMin,varYMax);

  TH2F *linePlot = static_cast<TH2F*>(gROOT->FindObject("linePlot"));
  if (linePlot) delete linePlot;

  linePlot = new TH2F("linePlot","line plot",
		      200,varXMin,varXMax,200,varYMin,varYMax);
  canv->Clear();
  linePlot->SetStats(kFALSE);
  linePlot->Draw();

  TGraph* varGraph0 = static_cast<TGraph*>(gROOT->FindObject("varGraph0"));
  if (varGraph0) delete varGraph0;

  varGraph0 = new TGraph(nStep,varX,varY0);
  varGraph0->SetName("varGraph0");
  varGraph0->SetMarkerStyle(20);
  varGraph0->SetLineStyle(kSolid);
  varGraph0->SetMarkerColor(2);
  varGraph0->SetLineColor(2);

  TGraph* varGraph1 = static_cast<TGraph*>(gROOT->FindObject("varGraph1"));
  if (varGraph1) delete varGraph1;

  varGraph1 = new TGraph(nStep,varX,varY1);
  varGraph1->SetName("varGraph1");
  varGraph1->SetMarkerStyle(25);
  varGraph1->SetLineStyle(kDashed);
  varGraph1->SetMarkerColor(3);
  varGraph1->SetLineColor(3);

  TGraph* varGraph2 = static_cast<TGraph*>(gROOT->FindObject("varGraph2"));
  if (varGraph2) delete varGraph2;

  varGraph2 = new TGraph(nStep,varX,varY2);
  varGraph2->SetName("varGraph2");
  varGraph2->SetMarkerStyle(21);
  varGraph2->SetLineStyle(kDotted);
  varGraph2->SetMarkerColor(4);
  varGraph2->SetLineColor(4);

  varGraph0->Draw("LP");
  varGraph1->Draw("LPsame");
  varGraph2->Draw("LPsame");

  TString varXTitle = stepVarNameX;
  TString varYTitle = stepVarNameY;

  if (data->GetUnit(stepVarNameX) != "") 
    varXTitle += AddBrackets(data->GetUnit(stepVarNameX));
  if (data->GetUnit(stepVarNameY) != "") 
    varYTitle += AddBrackets(data->GetUnit(stepVarNameY));

  TH5Legend* leg = new TH5Legend(0.76,0.8,1.,0.95);
  TString legName0, legName1, legName2;
  if (y0type == kPart)
    legName0 = "proj.";
  else if (y0type == kPartSlice)
    legName0 = "slice";
  else if (y0type == kPartScreen)
    legName0 = "screen";
  else {
    cout<< "error, should not be here" << endl;
    legName0 = "*****";
  }
  if (y1type == kPart)
    legName1 = "proj.";
  else if (y1type == kPartSlice)
    legName1 = "slice";
  else if (y1type == kPartScreen)
    legName1 = "screen";
  else {
    cout<< "error, should not be here" << endl;
    legName1 = "*****";
  }
  if (y2type == kPart)
    legName2 = "proj.";
  else if (y2type == kPartSlice)
    legName2 = "slice";
  else if (y2type == kPartScreen)
    legName2 = "screen";
  else {
    cout<< "error, should not be here" << endl;
    legName2 = "*****";
  }
  leg->AddEntry(varGraph0, legName0,"lp");
  leg->AddEntry(varGraph1, legName1,"lp");
  leg->AddEntry(varGraph2, legName2,"lp");
  leg->SetHeader(data->GetFilename());
  leg->Draw();

  linePlot->SetYTitle(varYTitle);
  linePlot->SetXTitle(varXTitle);
  canv->Update();
}

void TH5MainFrame::PartLinePlotProjAndSlice(TH5Dataset* data, 
					     const TString stepVarNameX, 
					     const TString stepVarNameY) {
  // version of LinePlot for plotting two variables at once
  Int_t nStep = data->GetNSteps();

  Double_t varX[nStep];
  Double_t varY0[nStep];
  Double_t varY1[nStep];

  Double_t varXMin = 1.e20; 
  Double_t varXMax = -1.e20;
  Double_t varY0Min = 1.e20;
  Double_t varY0Max = -1.e20;
  Double_t varY1Min = 1.e20;
  Double_t varY1Max = -1.e20;

  TCanvas *canv = fEcanvas->GetCanvas();
  canv->SetRightMargin(0.25);

  if (!data->GetStepVarArray(stepVarNameX, varX, kScalar)) {
    canv->Clear();
    return;
  }
  if (!data->GetStepVarArray(stepVarNameY, varY0, kPart)) {
    canv->Clear();
    return;
  }
  if (!data->GetStepVarArray(stepVarNameY, varY1, kPartSlice)) {
    canv->Clear();
    return;
  }

  // find min. and max. for x:
  for(Int_t step = 0; step < nStep; step++){ 
    if (varX[step] < varXMin) varXMin = varX[step];
    if (varX[step] > varXMax) varXMax = varX[step];
  }
  // adapt min. and max. for x to slider positions:

  Double_t upper = fSelLPSlider->GetMaxPosition();
  Double_t lower= fSelLPSlider->GetMinPosition();

  varXMin = varXMin + lower*(varXMax-varXMin);
  varXMax = varXMin + upper*(varXMax-varXMin);

  // find corresponding min. and max. for y1 and y2:
  for(Int_t step = 0; step < nStep; step++){ 
    if (varX[step] <= varXMax && varX[step] >= varXMin) {
      if (varY0[step] < varY0Min) varY0Min = varY0[step];
      if (varY0[step] > varY0Max) varY0Max = varY0[step];
      if (varY1[step] < varY1Min) varY1Min = varY1[step];
      if (varY1[step] > varY1Max) varY1Max = varY1[step];
    }
  }

  Double_t varYMin = std::min(varY0Min,varY1Min);
  Double_t varYMax = std::max(varY0Max,varY1Max);

  if (varYMin >  1.e19) varYMin = -0.9;
  if (varYMax < -1.e19) varYMax = +0.9;

  AddMargin(varXMin,varXMax);
  AddMargin(varYMin,varYMax);

  TH2F *linePlot = static_cast<TH2F*>(gROOT->FindObject("linePlot"));
  if (linePlot) delete linePlot;

  linePlot = new TH2F("linePlot","line plot",
		      200,varXMin,varXMax,200,varYMin,varYMax);
  canv->Clear();
  linePlot->SetStats(kFALSE);
  linePlot->Draw();

  TGraph* varGraph0 = static_cast<TGraph*>(gROOT->FindObject("varGraph0"));
  if (varGraph0) delete varGraph0;

  varGraph0 = new TGraph(nStep,varX,varY0);
  varGraph0->SetName("varGraph0");
  varGraph0->SetMarkerStyle(20);
  varGraph0->SetLineStyle(kSolid);

  TGraph* varGraph1 = static_cast<TGraph*>(gROOT->FindObject("varGraph1"));
  if (varGraph1) delete varGraph1;

  varGraph1 = new TGraph(nStep,varX,varY1);
  varGraph1->SetName("varGraph1");
  varGraph1->SetMarkerStyle(25);
  varGraph1->SetLineStyle(kDashed);

  varGraph0->Draw("LP");
  varGraph1->Draw("LPsame");

  TString varXTitle = stepVarNameX;
  TString varYTitle = stepVarNameY;

  if (data->GetUnit(stepVarNameX) != "") 
    varXTitle += AddBrackets(data->GetUnit(stepVarNameX));
  if (data->GetUnit(stepVarNameY) != "") 
    varYTitle += AddBrackets(data->GetUnit(stepVarNameY));

  TH5Legend* leg = new TH5Legend(0.76,0.8,1.,0.95);
  leg->AddEntry(varGraph0, "projected","lp");
  leg->AddEntry(varGraph1, "slice","lp");
  leg->SetHeader(data->GetFilename());
  leg->Draw();

  linePlot->SetYTitle(varYTitle);
  linePlot->SetXTitle(varXTitle);
  canv->Update();

}

void TH5MainFrame::LinePlot(TH5Dataset* data[], const Int_t nFiles,  
			    const TString stepVarNameX, const TString stepVarNameY, 
			    const EH5VarType vtype)
{
  Bool_t multiPlot = kFALSE;
  multiPlot = (vtype==kPartProjAndSlice   || 
	       vtype==kPartProjAndScreen  || 
	       vtype==kPartSliceAndScreen || 
	       vtype==kPartProjAndSliceAndScreen);

  // multi plots: delegate to special functions for one file, refuse for more than one file, 
  if (multiPlot) {
    if (nFiles==1) {
      if (vtype==kPartProjAndSlice) {
	PartLinePlotTwo(data[0], stepVarNameX, stepVarNameY, kPart, kPartSlice);
	return;
      }
      if (vtype==kPartSliceAndScreen) {
	PartLinePlotTwo(data[0], stepVarNameX, stepVarNameY, kPartSlice, kPartScreen);
	return;
      }
      if (vtype==kPartProjAndScreen) {
	PartLinePlotTwo(data[0], stepVarNameX, stepVarNameY, kPart, kPartScreen);
	return;
      }
      if (vtype==kPartProjAndSliceAndScreen) {
	PartLinePlotThree(data[0], stepVarNameX, stepVarNameY, kPart, kPartSlice, kPartScreen);
	return;
      }
    }
    if ( nFiles > 1) {
      h5Error(this,"LinePlot",1, "Sorry dude! Multiple plots are not implemented for two or more files!");
      return;
    }
  }
  
  TString coordName;
  if (vtype==kScalar) coordName = TString("");
  if (vtype==kVectorX) coordName = TString(" (x) ");
  if (vtype==kVectorY) coordName = TString(" (y) ");
  if (vtype==kVectorZ) coordName = TString(" (z) ");

  Int_t nStep[nFiles];
  Double_t* pVarX[nFiles];
  Double_t* pVarY[nFiles];
  Double_t XMin[nFiles];
  Double_t YMin[nFiles];
  Double_t XMax[nFiles];
  Double_t YMax[nFiles];
  for (Int_t i=0; i<nFiles; ++i) {
    nStep[i] = data[i]->GetNSteps();
    pVarX[i] = new Double_t[nStep[i]];
    pVarY[i] = new Double_t[nStep[i]];
    XMin[i] = 1.e20;
    XMax[i] = -1.e20;
    YMin[i] = 1.e20;
    YMax[i] = -1.e20;
  }
  TCanvas *canv = fEcanvas->GetCanvas();
  canv->SetRightMargin(0.25);

  for (Int_t i=0; i<nFiles; ++i) {
    if (!data[i]->GetStepVarArray(stepVarNameX, pVarX[i], kScalar)) {
      canv->Clear();
      return;
    }
    if (!data[i]->GetStepVarArray(stepVarNameY, pVarY[i], vtype)) {
      canv->Clear();
      return;
    }
  }

  for (Int_t i=0; i<nFiles; ++i) {
    for(Int_t step = 0; step < nStep[i]; step++){ 
      if (pVarX[i][step] < XMin[i]) XMin[i] = pVarX[i][step];
      if (pVarX[i][step] > XMax[i]) XMax[i] = pVarX[i][step];
    }
  }
  Double_t varXMin = XMin[0];
  Double_t varXMax = XMax[0];
  for (Int_t i=0; i<nFiles; ++i) {
    varXMin = std::min(varXMin,XMin[i]);
    varXMax = std::max(varXMax,XMax[i]);
  }
  // adapt min. and max. for x to slider positions:
  Double_t upper = fSelLPSlider->GetMaxPosition();
  Double_t lower= fSelLPSlider->GetMinPosition();

  varXMin = varXMin + lower*(varXMax-varXMin);
  varXMax = varXMin + upper*(varXMax-varXMin);

  // find corresponding min. and max. for y:
  for (Int_t i=0; i<nFiles; ++i) {
    for(Int_t step = 0; step < nStep[i]; step++){
      if (pVarX[i][step] <= varXMax && pVarX[i][step] >= varXMin) {
        if (pVarY[i][step] < YMin[i]) YMin[i] = pVarY[i][step];
        if (pVarY[i][step] > YMax[i]) YMax[i] = pVarY[i][step];
      }
    }
  }
  Double_t varYMin = YMin[0];
  Double_t varYMax = YMax[0];
  for (Int_t i=0; i<nFiles; ++i) {
    varYMin = std::min(varYMin,YMin[i]);
    varYMax = std::max(varYMax,YMax[i]);
  }

  if (varYMin >  1.e19) varYMin = -0.9;
  if (varYMax < -1.e19) varYMax = +0.9;

  AddMargin(varXMin,varXMax);
  AddMargin(varYMin,varYMax);
  
  TH2F *linePlot = static_cast<TH2F*>(gROOT->FindObject("linePlot"));
  if (linePlot) delete linePlot;

  linePlot = new TH2F("linePlot","line plot",
			    200,varXMin,varXMax,200,varYMin,varYMax);
  canv->Clear();
  linePlot->SetStats(kFALSE);
  linePlot->Draw();

  TGraph* varGraph[nFiles];
  TString nameVarGraph("varGraph");
  const Color_t graphColors[5] = {2,3,4,6,7};
  const Style_t markerStyles[5] = {20,25,21,28,30};
  const ELineStyle lineStyles[5] = {kSolid, kDashed, kDotted, kDashDotted, kDotted};
  for (Int_t i=0; i<nFiles; ++i) {
    TString nameGraph(nameVarGraph);
    nameGraph += nFiles;
    varGraph[i] = static_cast<TGraph*>(gROOT->FindObject(nameGraph));
    if (varGraph[i]) delete varGraph[i];

    varGraph[i] = new TGraph(nStep[i],pVarX[i],pVarY[i]);
    varGraph[i]->SetName(nameGraph);
    varGraph[i]->SetMarkerStyle(markerStyles[i]);
    varGraph[i]->SetLineStyle(lineStyles[i]);
    varGraph[i]->SetMarkerColor(graphColors[i]);
    varGraph[i]->SetLineColor(graphColors[i]);
   }  
  
  varGraph[0]->Draw("LP");
  for (Int_t i=1; i<nFiles; ++i) {
    varGraph[i]->Draw("LPsame");
  }

  TString varXTitle = stepVarNameX;
  TString varYTitle = stepVarNameY+coordName;

  TString sUnit[nFiles];
  // add unit for x axis
  for (Int_t i = 0; i < nFiles; ++i) {
    sUnit[i] = data[i]->GetUnit(stepVarNameX); 
  }
  for (Int_t i = 0; i < nFiles-1; ++i) {
    for (Int_t j = i+1; j < nFiles; ++j) {
      if (sUnit[i] != sUnit[j])
          h5Warning(this,"LinePlot",1,"Inconsistent units between files %s and %s, dude!",
	          data[i]->GetFilename().Data(),data[j]->GetFilename().Data());
    }
  }
  for (Int_t i = 0; i < nFiles; ++i) {
    if (sUnit[i] != "")
      varXTitle += AddBrackets(sUnit[i]);
  }
  // add unit for y axis
  for (Int_t i = 0; i < nFiles; ++i) {
    sUnit[i] = data[i]->GetUnit(stepVarNameY); 
  }
  for (Int_t i = 0; i < nFiles-1; ++i) {
    for (Int_t j = i+1; j < nFiles; ++j) {
      if (sUnit[i] != sUnit[j])
          h5Warning(this,"LinePlot",1,"Inconsistent units between files %s and %s, dude!",
	          data[i]->GetFilename().Data(),data[j]->GetFilename().Data());
    }
  }
  for (Int_t i = 0; i < nFiles; ++i) {
    if (sUnit[i] != "") {
      varYTitle += AddBrackets(sUnit[i]);
      break;
    }
  }

  TH5Legend* leg = new TH5Legend(0.76,0.8,1.,0.95);
  for (Int_t i=0; i<nFiles; ++i) 
    leg->AddEntry(varGraph[i], data[i]->GetFilename(),"lp");
  leg->SetHeader("line plot");
  leg->Draw();
  
  linePlot->SetYTitle(varYTitle);
  linePlot->SetXTitle(varXTitle);
  canv->Update();

  for (Int_t i=0; i<nFiles; ++i) { 
    delete[] pVarX[i];
    delete[] pVarY[i];
  }
}

void TH5MainFrame::GetSelVar(TString* selVarName[8], Int_t& nv) {

  nv=0;
  Int_t iter(0);
  for (Int_t n=0; n<8; ++n) {
    selVarName[n] = NULL;
    if (fVarName[n]) {
      if (fVarButton[n]->GetState()==kButtonDown) {
	nv++;
	selVarName[iter] = &fVarName[n];
	iter++;
      }
    }
  }
}

void TH5MainFrame::GetSelFiles(TH5Dataset* selFile[5], Int_t& nf) {

  nf=0;
  Int_t iter(0);

  for (Int_t n=0; n<5; ++n) {
    selFile[n] = NULL;
    if (fDataset[n]) {
      if (fDataset[n]->GetState()==kActive)  {
	nf++;
	selFile[iter] = fDataset[n];
	iter++;
      }
    }
  }
}

void TH5MainFrame::GetAllFiles(TH5Dataset* selFile[5], Int_t& nf) {

  nf=0;
  Int_t iter(0);

  for (Int_t n=0; n<5; ++n) {
    selFile[n] = NULL;
    if (fDataset[n]) {
      nf++;
      selFile[iter] = fDataset[n];
      iter++;
    }
  }
}

TString TH5MainFrame::GetPrintFilename() {

  // construct filename from active files and plotted variables:
  TString printFilename;

  Int_t nFiles(0);
  for (Int_t n=0; n<5; ++n) {
    if (fDataset[n]) {
      if (fDataset[n]->GetState()==kActive)  {
	if (nFiles > 0) {
	  printFilename+= TString("-");
	  printFilename+= fDataset[n]->GetFilenameNoExt();
	} else {
	  printFilename = fDataset[n]->GetFilenameNoExt();
	}
	nFiles++;
      }
    }
  }
    
  Int_t nVars(0);

  if (fLastPlot==kHisto1d || fLastPlot==kHisto2d || fLastPlot==kHisto3d) {
    for (Int_t n=0; n<8; ++n) {
      if (fVarName[n]) {
	if (fVarButton[n]->GetState()==kButtonDown) {
	  printFilename += TString("-");
	  printFilename += fVarName[n];
	  nVars++;
	}
      }
    }
    printFilename += TString("-step-");
    printFilename += fStepSlider->GetPosition();
  }

  if (fLastPlot==kHistoSE) {
    printFilename += TString("-SliceEmit");
    for (Int_t n=0; n<8; ++n) {
      if (fVarName[n]) {
	if (fVarButton[n]->GetState()==kButtonDown) {
	  printFilename += TString("-");
	  printFilename += fVarName[n];
	  nVars++;
	}
      }
    }
    printFilename += TString("-step-");
    printFilename += fStepSlider->GetPosition();
  }

  if (fLastPlot==kScalarLinePlot) {
    printFilename += TString("-");
    printFilename += fScalarLPBox->GetSelectedEntry()->GetTitle();
    printFilename += TString("-");
    printFilename += fSelLPBox->GetSelectedEntry()->GetTitle();
  }

  if (fLastPlot==kVectorLinePlot) {
    TString coordName;
    if (kButtonDown == fRadioLP[2]->GetState()) coordName = TString("z"); 
    if (kButtonDown == fRadioLP[1]->GetState()) coordName = TString("y");
    if (kButtonDown == fRadioLP[0]->GetState()) coordName = TString("x"); 
  
    printFilename += TString("-");
    printFilename += fVectorLPBox->GetSelectedEntry()->GetTitle();
    printFilename += TString("-");
    printFilename += coordName;
    printFilename += TString("-");
    printFilename += fSelLPBox->GetSelectedEntry()->GetTitle();
  }

  if (fLastPlot==kMeanLinePlot||fLastPlot==kCovLinePlot||fLastPlot==kEmitLinePlot) {
    printFilename += TString("-");
    if (fLastPlot==kMeanLinePlot) printFilename += TString("Mean");
    if (fLastPlot==kCovLinePlot) printFilename += TString("Sigma");
    if (fLastPlot==kEmitLinePlot) printFilename += TString("Emit");
    for (Int_t n=0; n<8; ++n) {
      if (fVarName[n]) {
	if (fVarButton[n]->GetState()==kButtonDown) {
	  printFilename += TString("-");
	  printFilename += fVarName[n];
	  nVars++;
	}
      }
    }
  }

  // Remove any special characters from the filename:
  while (printFilename.Contains("#")) {
    printFilename.Remove(printFilename.First("#"),1);
  }

  while (printFilename.Contains("$")) {
    printFilename.Remove(printFilename.First("$"),1);
  }

  while (printFilename.Contains("~")) {
    printFilename.Remove(printFilename.First("~"),1);
  }
  /// etc...

  return(printFilename);

}

void TH5MainFrame::PrintCanv(TString printFilename, Int_t format) {

  TCanvas *canv = fEcanvas->GetCanvas();
  canv->Print(printFilename,H5PrintFormat[format][0]);
}

void TH5MainFrame::CloseWindow()
{
   // Got close message for this TH5MainFrame. Terminate the application.
   gApplication->Terminate();
}

void TH5MainFrame::DoMenu(Int_t id)
{
  // Handle menu items.
  
  switch (id) {
    
  case M_FILE_LOAD:
    {
      new TH5LoadDialog(gClient->GetRoot(), this, 400, 200);
    }
    break;
    
  case M_FILE_PRINT:
    {
      new TH5PrintDialog(gClient->GetRoot(), this, 400, 200,H5PrintFormat);
    }
    break;
    
  case M_FILE_EXIT:
    CloseWindow();   
    break;
    
  case M_EDIT_CANV:
    fEcanvas->GetCanvas()->EditorBar();
    break;
    
  case M_OPT_SLICE:
    {
      new TH5SliceDialog(gClient->GetRoot(), this, 400, 200, H5SlicePosition);
    }
    break;
    
  case M_OPT_LOGX:
    {
      if(DrawLogX) {LogX(0);} else {LogX(1);}
    } 
    break;

  case M_OPT_LOGY:
    {
      if(DrawLogY) {LogY(0);} else {LogY(1);}
    } 
    break;

  case M_OPT_LOGZ:
    {
      if(DrawLogZ) {LogZ(0);} else {LogZ(1);}
    } 
    break;


  default:
    printf("Menu item %d selected\n", id);
    break;
  }
}

void TH5MainFrame::LogX(Int_t y) {

  if (y) {
    DrawLogX = kTRUE;
    fEcanvas->GetCanvas()->SetLogx();
    fMenuOptLog->CheckEntry(M_OPT_LOGX);
  } else {
    DrawLogX = kFALSE;
    fEcanvas->GetCanvas()->SetLogx(0);
    fMenuOptLog->UnCheckEntry(M_OPT_LOGX);
  }

}

void TH5MainFrame::LogY(Int_t y) {

  if (y) {
    DrawLogY = kTRUE;
    fEcanvas->GetCanvas()->SetLogy();
    fMenuOptLog->CheckEntry(M_OPT_LOGY);
  } else {
    DrawLogY = kFALSE;
    fEcanvas->GetCanvas()->SetLogy(0);
    fMenuOptLog->UnCheckEntry(M_OPT_LOGY);
  }

}

void TH5MainFrame::LogZ(Int_t y) {

  if (y) {
    DrawLogZ = kTRUE;
    fEcanvas->GetCanvas()->SetLogz();
    fMenuOptLog->CheckEntry(M_OPT_LOGZ);
  } else {
    DrawLogZ = kFALSE;
    fEcanvas->GetCanvas()->SetLogz(0);
    fMenuOptLog->UnCheckEntry(M_OPT_LOGZ);
  }

}

void TH5MainFrame::DoStepSlider(Int_t step) {
  
  Int_t id;
  TGFrame *frm = (TGFrame *) gTQSender;
  if (frm->IsA()->InheritsFrom(TGSlider::Class())) {
    TGSlider *sl = (TGSlider*) frm;
    id = sl->WidgetId();
  } else {
    TGDoubleSlider *sd = (TGDoubleSlider *) frm;
    id = sd->WidgetId();
  }
  
  char bufStep[32];
  sprintf(bufStep, "%d", step);
  
  Double_t spos = Spos(step);
  char bufSpos[32];
  sprintf(bufSpos, "%g", spos);
  
  switch (id) {
  case stepSliderID:
    fStepSliderTB->Clear();
    fStepSliderTB->AddText(0, bufStep);
    // Re-align the cursor with the characters.
    fStepSliderTE->SetCursorPosition(fStepSliderTE->GetCursorPosition());
    fStepSliderTE->Deselect();
    gClient->NeedRedraw(fStepSliderTE);
    
    fSposSliderTB->Clear();
    fSposSliderTB->AddText(0, bufSpos);
    // Re-align the cursor with the characters.
    fSposSliderTE->SetCursorPosition(fSposSliderTE->GetCursorPosition());
    fSposSliderTE->Deselect();
    gClient->NeedRedraw(fSposSliderTE);

    break;
  default:
    break;
  }
}


void TH5MainFrame::DoStepText(const char * ) {

  TGTextEntry *te = (TGTextEntry *) gTQSender;
  Int_t id = te->WidgetId();
  
  Double_t spos;
  Int_t step;
  char bufSpos[32];
  char bufStep[32];

  switch (id) {
  case stepSliderTEID:
    step = atoi(fStepSliderTB->GetString());
    fStepSlider->SetPosition(step);
    // update spos field:
    spos = Spos(step);
    sprintf(bufSpos, "%g", spos);
    fSposSliderTB->Clear();
    fSposSliderTB->AddText(0, bufSpos);
    fSposSliderTE->SetCursorPosition(fSposSliderTE->GetCursorPosition());
    fSposSliderTE->Deselect();
    gClient->NeedRedraw(fSposSliderTE);
    break;
  case sposSliderTEID:
    spos = atof(fSposSliderTB->GetString());
    step = Step(spos);
    fStepSlider->SetPosition(step);
    // update step field:
    sprintf(bufStep, "%d", step);
    fStepSliderTB->Clear();
    fStepSliderTB->AddText(0, bufStep);
    fStepSliderTE->SetCursorPosition(fStepSliderTE->GetCursorPosition());
    fStepSliderTE->Deselect();
    gClient->NeedRedraw(fStepSliderTE);
    break;
  default:
    break;
  }
}

void TH5MainFrame::DoStepTextReturn(const char * ) {

  TGTextEntry *te = (TGTextEntry *) gTQSender;
  Int_t id = te->WidgetId();
  
  Double_t spos;
  Int_t step;
  char bufSpos[32];
  char bufStep[32];

  switch (id) {
  case sposSliderTEID:
    spos = atof(fSposSliderTB->GetString());
    step = Step(spos);
    fStepSlider->SetPosition(step);
    // update step field:
    sprintf(bufStep, "%d", step);
    fStepSliderTB->Clear();
    fStepSliderTB->AddText(0, bufStep);
    fStepSliderTE->SetCursorPosition(fStepSliderTE->GetCursorPosition());
    fStepSliderTE->Deselect();
    gClient->NeedRedraw(fStepSliderTE);
    // update spos field: (jump to next smaller spos)
    spos = Spos(step);
    sprintf(bufSpos, "%g", spos);
    fSposSliderTB->Clear();
    fSposSliderTB->AddText(0, bufSpos);
    fSposSliderTE->SetCursorPosition(fSposSliderTE->GetCursorPosition());
    fSposSliderTE->Deselect();
    gClient->NeedRedraw(fSposSliderTE);
    break;
  default:
    break;
  }
}

void TH5MainFrame::PlotNextStep(Int_t jump) {

  // update step slider:
  Int_t step = fStepSlider->GetPosition();
  Int_t stepMax = fStepSlider->GetMaxPosition();
  step += jump;
  if (step > stepMax) step = stepMax;
  if (step < 0) step = 0;
  fStepSlider->SetPosition(step);
  fStepSlider->PositionChanged(step);  // triggers DoStepSlider!

  DoLastPlot();

}
 
void TH5MainFrame::DoLastPlot() {

  // redo last plot:
  if (fLastPlot==kHisto1d) Histo1d();
  if (fLastPlot==kHisto2d) Histo2d();
  if (fLastPlot==kHisto3d) Histo3d();
  if (fLastPlot==kHistoSE) HistoSliceEmit();
  if (fLastPlot==kScalarLinePlot) ScalarLinePlot();
  if (fLastPlot==kVectorLinePlot) VectorLinePlot();
  if (fLastPlot==kMeanLinePlot)   PartLinePlot(linePlotMeanID);
  if (fLastPlot==kCovLinePlot)   PartLinePlot(linePlotCovID);
  if (fLastPlot==kEmitLinePlot)   PartLinePlot(linePlotEmitID);

}

void TH5MainFrame::DoSelLPSlider(Int_t pos) {
  
  Int_t id;
  TGFrame *frm = (TGFrame *) gTQSender;
  if (frm->IsA()->InheritsFrom(TGSlider::Class())) {
    TGSlider *sl = (TGSlider*) frm;
    id = sl->WidgetId();
  } else {
    TGDoubleSlider *sd = (TGDoubleSlider *) frm;
    id = sd->WidgetId();
  }

  switch (id) {
  case selLPSliderID:


    break;
  default:
    break;
  }
}


void TH5MainFrame::DoLinePlotRadioButtons()
{
  
  TGButton *btn = (TGButton *) gTQSender;
  Int_t id = btn->WidgetId();
  
  switch (id) {
  case linePlotXID:  
    fRadioLP[1]->SetState(kButtonUp);
    fRadioLP[2]->SetState(kButtonUp);
    break;
  case linePlotYID:  
    fRadioLP[0]->SetState(kButtonUp);
    fRadioLP[2]->SetState(kButtonUp);
    break;
  case linePlotZID:  
    fRadioLP[0]->SetState(kButtonUp);
    fRadioLP[1]->SetState(kButtonUp);
    break;
  default:
    break;
   }
}

Int_t TH5MainFrame::Step(const Double_t spos ) {
  
  TH5Dataset* activeData[5];
  Int_t nFiles(0);
  GetSelFiles(activeData,nFiles);

  Int_t step;
  if (activeData[0]) {
    step = activeData[0]->GetStep(spos);
  } else {
    step = TMath::Nint(0.5*spos); // dummy
  }
  return(step);
}

Double_t TH5MainFrame::Spos(const Int_t step ) {

  TH5Dataset* activeData[5];
  Int_t nFiles(0);
  GetSelFiles(activeData,nFiles);

  Double_t spos;
  if (activeData[0]) {
    spos = activeData[0]->GetSpos(step);
  } else {
    spos = 2.*step;   // dummy
  }
  return (spos);
}

Int_t TH5MainFrame::UpdateSliceOptions(const EH5SlicePosOpt posOpt, 
				       const Int_t nSlicePartPerBin,
				       const EH5SliceExtentOpt extOpt,
				       const Double_t slicePercent,
				       const Double_t lengthValue,
				       const EH5SliceUnit lengthUnit,
				       const Double_t sigmaValue,
				       const EH5SliceUnit sigmaUnit,
				       const EH5SliceUpdateOpt updOpt)

{
  TH5Dataset* activeData[5];
  Int_t nFiles(0);

  if (kSel == updOpt) {
    GetSelFiles(activeData,nFiles);
  } else {
    GetAllFiles(activeData,nFiles);    
  }

  for (Int_t iFile=0; iFile < nFiles; iFile++) {
    activeData[iFile]->SetSliceOptions(posOpt,nSlicePartPerBin,
          extOpt,slicePercent,lengthValue,lengthUnit,
          sigmaValue,sigmaUnit);
  }

  printf("\n\tSlice options have been updated for files:\n");
  for (Int_t iFile=0; iFile < nFiles; iFile++) {
    TString s = TString("\t\t")+activeData[iFile]->GetFilename()+TString("\n");
    printf(s.Data());
  }
  printf("\tSlice position:\n\t\t");
  printf(H5SlicePosition[posOpt].Data());  
  if (posOpt==kPeakZ) printf("\t(Particles per bin: %d )",nSlicePartPerBin);
  printf("\n");

  printf("\tSlice extention:\n");
  if (extOpt == kPercent)
    printf("\t\tSlice fraction: %5.2f%% of bunch\n",slicePercent);
  if (extOpt == kLength) {
    printf("\t\tSlice length: %5.3f ", lengthValue);
    (sigmaUnit == kmm)? printf("mm") : printf("ps");
    printf("\n");
  }

  printf("\tSlice weighting:\n");
  if (sigmaValue <= 0.) {
    printf("\t\tUniform \n");
  } else {
    printf("\t\tGaussian: sigma=%5.3f ", sigmaValue);
    (sigmaUnit == kmm)? printf("mm") : printf("ps");
    printf("\n");
  } 
  printf("\n");

  return(0);
}

EH5SlicePosOpt TH5MainFrame::GetSlicePosOpt() 
{
  EH5SlicePosOpt posOpt;
  TH5Dataset* activeData[5];
  Int_t nFiles(0);
  GetSelFiles(activeData,nFiles);
  if (activeData[0]) {
    posOpt = activeData[0]->GetSlicePosOpt();
  } else {  
    posOpt = kMeanZ;
  }
   return(posOpt);
}  

Double_t TH5MainFrame::GetSlicePercent() 
{
  Double_t percent;
  TH5Dataset* activeData[5];
  Int_t nFiles(0);
  GetSelFiles(activeData,nFiles);
  if (activeData[0]) {
    percent = activeData[0]->GetSlicePercent();
  } else {  
    percent = 20.;
  }
  return(percent);
}

Int_t TH5MainFrame::GetSlicePartPerBin() 
{
  Int_t partPerBin;
  TH5Dataset* activeData[5];
  Int_t nFiles(0);
  GetSelFiles(activeData,nFiles);
  if (activeData[0]) {
    partPerBin = activeData[0]->GetSlicePartPerBin();
  } else {  
    partPerBin = 100;
  }
  return(partPerBin);

}

Double_t TH5MainFrame::GetSliceLength() 
{
  Double_t lengthZ;
  TH5Dataset* activeData[5];
  Int_t nFiles(0);
  GetSelFiles(activeData,nFiles);
  if (activeData[0]) {
    lengthZ = activeData[0]->GetSliceLength();
  } else {  
    lengthZ = 0.3;
  }
  return(lengthZ);
}

EH5SliceUnit TH5MainFrame::GetSliceLengthUnit() 
{
  EH5SliceUnit lengthU;
  TH5Dataset* activeData[5];
  Int_t nFiles(0);
  GetSelFiles(activeData,nFiles);
  if (activeData[0]) {
    lengthU = activeData[0]->GetSliceLengthUnit();
  } else {  
    lengthU = kmm;
  }
  return(lengthU);
}

EH5SliceExtentOpt TH5MainFrame::GetSliceExtentOpt() 
{
  EH5SliceExtentOpt extOpt;
  TH5Dataset* activeData[5];
  Int_t nFiles(0);
  GetSelFiles(activeData,nFiles);
  if (activeData[0]) {
    extOpt = activeData[0]->GetSliceExtentOpt();
  } else {  
    extOpt = kPercent;
  }
   return(extOpt);
}

Double_t TH5MainFrame::GetSliceSigma() 
{
  Double_t sigmaValue;
  TH5Dataset* activeData[5];
  Int_t nFiles(0);
  GetSelFiles(activeData,nFiles);
  if (activeData[0]) {
    sigmaValue = activeData[0]->GetSliceSigma();
  } else {  
    sigmaValue= 0.3;
  }
  return(sigmaValue);
}


EH5SliceUnit TH5MainFrame::GetSliceSigmaUnit() 
{
  EH5SliceUnit sigmaUnit;
  TH5Dataset* activeData[5];
  Int_t nFiles(0);
  GetSelFiles(activeData,nFiles);
  if (activeData[0]) {
    sigmaUnit = activeData[0]->GetSliceSigmaUnit();
  } else {  
    sigmaUnit = kmm;
  }
   return(sigmaUnit);
}

void TH5MainFrame::Test() {
  cout << "Huh?" << endl;
}

