#ifndef __TH5MainFrame__
#define __TH5MainFrame__

#include <TGFrame.h>
#include <TRootEmbeddedCanvas.h>

#include <TGDockableFrame.h>
#include <TGMenu.h>
#include <TGButton.h>
#include <TGTextBuffer.h>
#include <TGTextEntry.h>
#include <TGLabel.h>
#include <TGComboBox.h>
#include <TGSlider.h>
#include <TGDoubleSlider.h>
#include <TClonesArray.h>

#include "TH5Dataset.h"
#include "TH5Util.h"

enum EWidgetIDs { stepSliderID, stepSliderTEID, sposSliderTEID,
   selLPSliderID,
   linePlotXID, linePlotYID, linePlotZID,
   linePlotMeanID, linePlotCovID, linePlotEmitID
};

enum EMenuIDs {
   M_FILE_LOAD,
   M_FILE_UNLOAD,
   M_FILE_PRINT,
   M_FILE_EXIT,
   M_EDIT_CANV,
   M_OPT_SLICE,
   M_OPT_LOGX,
   M_OPT_LOGY,
   M_OPT_LOGZ,
   M_HELP_ABOUT
};

using namespace std;
using namespace TH5Util;

class TH5MainFrame:public TGMainFrame {

 public:

   TH5MainFrame(const TGWindow * p = 0, UInt_t w = 700, UInt_t h =
                500, Int_t ssr = 1);
    virtual ~ TH5MainFrame();

   void LoadFile(const char *filename, const EH5MemOpt memOpt =
                 kLoadAndDump, const Int_t subsampRate = 0);

   void ActivateFile(Int_t slot);
   void UpdateSelections();

   void Histo1d();
   void Histo1d(TH5Dataset * data);
   void Histo1d(TH5Dataset * data0, TH5Dataset * data1);

   void Histo2d();
   void Histo2d(TH5Dataset * data);

   void Histo3d();
   void Histo3d(TH5Dataset * data);

   void HistoSliceEmit();
   void HistoSliceEmit(TH5Dataset * data);

   void ScalarLinePlot();
   void VectorLinePlot();
   void PartLinePlot(Int_t id = -1);
/*    void LinePlot(TH5Dataset * data, const TString stepVarNameX, */
/*                  const TString stepVarNameY, const EH5VarType vtype); */
/*    void LinePlot(TH5Dataset * data0, TH5Dataset * data1, */
/*                  const TString stepVarNameX, const TString stepVarNameY, */
/*                  const EH5VarType vtype); */
   void LinePlot(TH5Dataset* data[], const Int_t nFiles,  
			           const TString stepVarNameX, const TString stepVarNameY, 
			           const EH5VarType vtype);
   void PartLinePlotProjAndSlice(TH5Dataset * data,
                 const TString stepVarNameX,
                 const TString stepVarNameY);
   void PartLinePlotTwo(TH5Dataset* data, 
					     const TString stepVarNameX, 
					     const TString stepVarNameY,
               const EH5VarType y0type,
               const EH5VarType y1type);
   void PartLinePlotThree(TH5Dataset* data, 
					     const TString stepVarNameX, 
					     const TString stepVarNameY,
               const EH5VarType y0type,
               const EH5VarType y1type,
               const EH5VarType y2type);

   void GetSelVar(TString * selVar[6], Int_t & nv);
   void GetSelFiles(TH5Dataset * selFile[5], Int_t & nf);
   void GetAllFiles(TH5Dataset * selFile[5], Int_t & nf);

   Int_t GetGlobalSubsamplingRate() {
      return (fgSubsampRate);
   };

   TString GetPrintFilename();
   void PrintCanv(TString printFilename, Int_t format);

   void CloseWindow();

   void Test();

   void DoMenu(Int_t id);
   void DoStepSlider(Int_t pos);
   void DoStepText(const char *);
   void DoStepTextReturn(const char *);
   void DoSelLPSlider(Int_t pos);
   void DoLinePlotRadioButtons();

   void LogX(Int_t y);
   void LogY(Int_t y);
   void LogZ(Int_t y);

   void PlotNextStep(Int_t jump);

   void DoLastPlot();

   Int_t Step(const Double_t spos);
   Double_t Spos(const Int_t step);

   Int_t UpdateSliceOptions(const EH5SlicePosOpt posOpt = kMeanZ,
                            const Int_t nSlicePartPerBin = -1,
                            const EH5SliceExtentOpt extOpt = kLength,
                            const Double_t slicePercent = 20,
                            const Double_t lengthValue = 0.3,
                            const EH5SliceUnit lengthUnit = kmm,
                            const Double_t sigmaValue = 0.3,
                            const EH5SliceUnit = kmm,
                            const EH5SliceUpdateOpt updOpt = kAll);

   EH5SlicePosOpt GetSlicePosOpt();
   Double_t GetSlicePercent();
   Int_t GetSlicePartPerBin();
   Double_t GetSliceLength();
   EH5SliceUnit GetSliceLengthUnit();
   EH5SliceExtentOpt GetSliceExtentOpt();
   Double_t GetSliceSigma();
   EH5SliceUnit GetSliceSigmaUnit();

 private:
   TH5Dataset * fDataset[5];    // array of H5files 
   TString fVarName[8];         // array of particle variables 
   TString fVarNameDefault[8];  // 
   Int_t fNStepMax;             // maximum number of steps in loaded files
   EH5PlotType fLastPlot;       // type of last plot
   Int_t fgSubsampRate;         // global subsampling rate

   Bool_t DrawLogX, DrawLogY, DrawLogZ; // logarithmic axes

   // frames, canvas
   TRootEmbeddedCanvas *fEcanvas;
   TGHorizontalFrame *fWorkspace;
   TGVerticalFrame *fSelspace, *fHistospace, *fLineplotspace;
   TGGroupFrame *fFileSelection, *fStepSelection, *fVarSelection;
   TGVerticalFrame *fVarSelectionL, *fVarSelectionR;

   TGGroupFrame *fHistoFrame;
   TGHorizontalFrame *f1dFrame, *f2dFrame, *f3dFrame, *fSEFrame;
   TGGroupFrame *fLinePlotFrame;

   // pull-down menus
   TGDockableFrame *fMenuDock;
   TGMenuBar *fMenuBar;
   TGPopupMenu *fMenuFile, *fMenuOpt, *fMenuEdit, *fMenuHelp;
   TGPopupMenu *fMenuOptLog;

   TGLayoutHints *fMenuBarLayout, *fMenuBarItemLayout, *fMenuBarHelpLayout;

   //buttons, labels, etc.
   TGTextButton *fHisto1d, *fHisto2d, *fHisto3d, *fHistoSE;
   TGCheckButton *fFileButton[5], *fVarButton[8];
   TGComboBox *fStepBox;
   TGComboBox *fHisto1dBox, *fHisto2dBox, *fHisto3dBox, *fHistoSEBox;

   TGHSlider *fStepSlider;
   TGHorizontalFrame *fStepSposTEFrame;
   TGVerticalFrame *fStepTEFrame, *fSposTEFrame;
   TGVerticalFrame *fPrevStepFrame, *fNextStepFrame;
   TGLabel *fStepLabel, *fSposLabel, *fPrevStepLabel, *fNextStepLabel;
   TGTextEntry *fStepSliderTE, *fSposSliderTE;
   TGTextBuffer *fStepSliderTB, *fSposSliderTB;
   TGTextButton *fNextButton, *fPrevButton;

   TGHorizontalFrame *fSelLPFrame;
   TGDoubleHSlider *fSelLPSlider;
   TGVerticalFrame *fSelLPFrameLeft, *fSelLPFrameRight;
   TGHorizontalFrame *fScalarLPFrame, *fVectorLPFrame, *fRadioLPFrame,
       *fPartLPFrame, *fSliceLPFrame;
   TGLabel *fSelLPLabel1, *fSelLPLabel2;
   TGTextButton *fScalar, *fVector;
   TGComboBox *fSelLPBox, *fScalarLPBox, *fVectorLPBox;
   TGRadioButton *fRadioLP[3];
   TGLabel *fPartLabel;
   TGTextButton *fMean, *fCov, *fEmit, *fSliceOpt;
   TGCheckButton *fProjButton, *fSliceButton, *fScreenButton;

   TString H5DrawOpt1d[10], H5DrawOpt2d[20], H5DrawOpt3d[2];

   // printer formats
   TString **H5PrintFormat;

   // slice positioning
   TString *H5SlicePosition;

   ClassDef(TH5MainFrame, 0)
};

#endif
