#include "TH5PrintDialog.h"
#include<iostream>

using namespace std;

ClassImp(TH5PrintDialog)
  
TH5PrintDialog::TH5PrintDialog(const TGWindow *p, TH5MainFrame *main, 
			       UInt_t w,UInt_t h, 
			       TString** H5PrintFormat,
			       UInt_t options) : 
  fParent(main),fPrintFormat(H5PrintFormat)
{
  fMain = new TGTransientFrame(p, main, w, h, options);
  fMain->Connect("CloseWindow()", "TH5PrintDialog", this, "CloseWindow()");
  fMain->DontCallClose(); // to avoid double deletions.
  
  // use hierarchical cleaning
  fMain->SetCleanup(kDeepCleanup);
  
  // first frame: selection of file format

  fFrame1 = new TGHorizontalFrame(fMain, 200, 50);

  fFormatLabel = new TGLabel(fFrame1,"File format: ");

  fPrintBox = new TGComboBox(fFrame1,"format",100);
  for (Int_t i=0; i<15; ++i) {
    fPrintBox->AddEntry(fPrintFormat[i][2],i);
  }   
  fPrintBox->Select(0);
  fPrintBox->Resize(70,20);

  fPrintBox->Connect("Selected(Int_t,Int_t)", "TH5PrintDialog", this, "DoFormat(Int_t,Int_t)");

  fFrame1->AddFrame(fFormatLabel, new TGLayoutHints(kLHintsLeft,5,5,5,5)); 
  fFrame1->AddFrame(fPrintBox, new TGLayoutHints(kLHintsRight,5,5,5,5)); 
  
  fMain->AddFrame(fFrame1, new TGLayoutHints(kLHintsTop | kLHintsLeft,2, 2, 2, 2));

  // second frame: selection of filename

  fFrame2 = new TGHorizontalFrame(fMain, 400, 50);

  fFilenameLabel = new TGLabel(fFrame2,"Filename:");

  fFilenameEntry = new TGTextEntry(fFrame2, fFilenameBuffer = new TGTextBuffer(100));

  fFilenameEntry->Resize(300, fFilenameEntry->GetDefaultHeight());
  fFilenameEntry->SetText(fParent->GetPrintFilename());
  fFilenameEntry->SetToolTipText("Enter the output filename, dude!");
  fFilenameEntry->Connect("ReturnPressed()", "TH5PrintDialog", this, "DoPrint()");

  fExtensionLabel = new TGLabel(fFrame2,".pdf");

  fFrame2->AddFrame(fFilenameLabel, new TGLayoutHints(kLHintsLeft,5,5,5,5)); 
  fFrame2->AddFrame(fFilenameEntry, new TGLayoutHints(kLHintsCenterX,5,5,5,5)); 
  fFrame2->AddFrame(fExtensionLabel, new TGLayoutHints(kLHintsRight,0,5,5,5)); 

  fMain->AddFrame(fFrame2, new TGLayoutHints(kLHintsLeft,2, 2, 2, 2));

  // bottom frame with print and cancel buttons: 

  fBottomFrame = new TGHorizontalFrame(fMain, 60, 20, kFixedWidth);
  
  fPrintButton = new TGTextButton(fBottomFrame, "&Print", 1);
  fPrintButton->Connect("Clicked()", "TH5PrintDialog", this, "DoPrint()");
  fCancelButton = new TGTextButton(fBottomFrame, "&Cancel", 2);
  fCancelButton->Connect("Clicked()", "TH5PrintDialog", this, "DoCancel()");
  
  fPrintButton->Resize(400,fPrintButton->GetDefaultHeight());

  fL1 = new TGLayoutHints(kLHintsTop | kLHintsRight ,10, 10, 2, 2);
  fL2 = new TGLayoutHints(kLHintsBottom | kLHintsExpandX, 2, 2, 5, 1);
  
  fBottomFrame->AddFrame(fPrintButton, fL1);
  fBottomFrame->AddFrame(fCancelButton, fL1);
  
  fBottomFrame->Resize(150, fPrintButton->GetDefaultHeight());
  fMain->AddFrame(fBottomFrame, fL2);
  
  fMain->MapSubwindows();
  fMain->Resize();
  
  // position relative to the parent's window
  fMain->CenterOnParent();
  
  fMain->SetWindowName("H5PartROOT Print");
  
  fMain->MapWindow();
  //gClient->WaitFor(fMain);    // otherwise canvas contextmenu does not work
}

TH5PrintDialog::~TH5PrintDialog()
{
  // Delete load dialog widgets.
  fMain->DeleteWindow();  // deletes fMain
}

void TH5PrintDialog::CloseWindow()
{
  // Called when window is closed via the window manager.
  delete this;
}

void TH5PrintDialog::DoFormat(Int_t wid, Int_t id)
{
  fExtensionLabel->SetText(fPrintFormat[id][1]);
  fMain->Layout();
  fMain->MapSubwindows();
  fMain->Resize();
}

void TH5PrintDialog::DoPrint()
{
  TString filename;

  if (fFilenameBuffer->GetTextLength()!=0) { 
    filename = fFilenameBuffer->GetString();
  } 
  filename += *(fExtensionLabel->GetText());
  fParent->PrintCanv(filename,fPrintBox->GetSelected());

  fMain->SendCloseMessage();
}

void TH5PrintDialog::DoCancel()
{
  fMain->SendCloseMessage();
}
