#include <TROOT.h>
#include <TRandom.h>

#include "TH5ProgressBar.h"
#include <iostream>
#include <math.h>

using namespace std;

ClassImp(TH5ProgressBar)
  
TH5ProgressBar::TH5ProgressBar(const TGWindow *p, TH5Dataset *dataset, 
			       UInt_t w, UInt_t h, 
			       const char * message,
			       UInt_t options) :
  fParentDataset(dataset), fCancel(kFALSE)
{   
  fParentFrame = fParentDataset->GetParentMainFrame(); 
  fMain = new TGTransientFrame(p, fParentFrame, w, h, options);
  fMain->Connect("CloseWindow()", "TH5ProgressBar", this, "CloseWindow()");
  fMain->DontCallClose(); // to avoid double deletions.
  
  // use hierarchical cleaning
  fMain->SetCleanup(kDeepCleanup);
  
  fMessage = new TGLabel(fMain,message);
  fMain->AddFrame(fMessage, new TGLayoutHints(kLHintsLeft,5,5,2,2)); 
  
  fProgBar = new TGHProgressBar(fMain, TGProgressBar::kFancy, 500);
  fProgBar->SetBarColor("lightblue");
  fProgBar->ShowPosition(kTRUE, kFALSE, "%.0f steps");

  fMain->AddFrame(fProgBar,      new TGLayoutHints(kLHintsLeft,5,5,2,2));

  const char * patienceMsg = "Time for a coffee, dude!";
  gRandom->SetSeed(0); // set to the current  machine clock
  Double_t rr = 1000.*gRandom->Rndm(); rr = rr-floor(rr); // use last digits only

  Int_t r = static_cast<Int_t>(9.*rr);

  switch (r) {
  case 0:
    patienceMsg = "Time for a coffee, dude!";
    break;
  case 1:
    patienceMsg = "\"Patience is the best of qualities; he who possesses it possesses all things.\" (anonymous)";
    break;
  case 2:
    patienceMsg = "\"Our patience will achieve more than our force.\" (Edmund Burke)";
    break;
  case 3:
    patienceMsg = "\"Patience is a bitter plant but it has sweet fruit.\" (German proverb)";
    break;
  case 4:
    patienceMsg = "\"Though patience be a tired mare, yet she will plod.\" (William Shakespeare)";
    break;
  case 5:
    patienceMsg = "\"Patience and diligence, like faith, remove mountains.\" (William Penn)";
    break;
  case 6:
    patienceMsg = "\"Beware the fury of a patient man.\" (John Dryden)";
    break;
  case 7:
    patienceMsg = "\"At the bottom of patience, there is heaven\" (anonymous)";
    break;
  case 8:
    patienceMsg = "Patience, dude!";
    break;
  default: ;
  } // switch

  fMessage2 = new TGLabel(fMain,patienceMsg);
  fMain->AddFrame(fMessage2, new TGLayoutHints(kLHintsLeft,5,5,2,2)); 

  fCancelButton = new TGTextButton(fMain, "&Cancel", 2);
  fCancelButton->Connect("Clicked()", "TH5ProgressBar", this, "DoCancel()");
  fMain->AddFrame(fCancelButton, new TGLayoutHints(kLHintsTop | kLHintsRight ,10, 10, 5, 5));
 
  fMain->MapSubwindows();

  fMain->Resize();
  
  // position relative to the parent's window
  fMain->CenterOnParent();
  
  fMain->SetWindowName("H5PartROOT Progress");
  
  fMain->MapWindow();
  //  gClient->WaitFor(fMain);    // otherwise canvas contextmenu does not work

}

TH5ProgressBar::~TH5ProgressBar()
{
  fMain->DeleteWindow();  // deletes fMain
}

void TH5ProgressBar::Increment(Double_t inc) {
  fProgBar->Increment(inc);
}

void TH5ProgressBar::IncrementStep(Int_t step, Int_t nStep) {
  fProgBar->Increment(100./nStep);
  TString s = TString("");
  s += (step+1); s += " of "; s += nStep; s += TString(" steps");
  fProgBar->ShowPosition(kTRUE, kFALSE, s.Data());
}

void TH5ProgressBar::WriteInBar(TString s) {
  fProgBar->ShowPosition(kTRUE, kFALSE, s.Data());
}

void TH5ProgressBar::CloseWindow()
{
  // Called when window is closed via the window manager.
  delete this;
}

void TH5ProgressBar::DoCancel()
{
  fCancel = kTRUE;
  fMain->SendCloseMessage();
}
