#include "TH5SliceDialog.h"
#include<iostream>

using namespace std;

ClassImp(TH5SliceDialog)
  
TH5SliceDialog::TH5SliceDialog(const TGWindow *p, TH5MainFrame *mainFrame, 
			       UInt_t w, UInt_t h, 
			       TString* H5SlicePosition,
			       UInt_t options) :
  fParent(mainFrame), fSlicePosition(H5SlicePosition)
{
  fMain = new TGTransientFrame(p, mainFrame, w, h, options);
  fMain->Connect("CloseWindow()", "TH5SliceDialog", this, "CloseWindow()");
  fMain->DontCallClose(); // to avoid double deletions.
  
  // use hierarchical cleaning
  fMain->SetCleanup(kDeepCleanup);

////////////////////////////////////////////////////
//slice position option
  fPosLabel = new TGLabel(fMain,"Slice position: ");

  fMain->AddFrame(fPosLabel, new TGLayoutHints(kLHintsLeft,5,5,5,5)); 

  fMain->AddFrame(fRadioSlicePos[0] = new TGRadioButton(fMain, 
                  fSlicePosition[kMeanZ], kMeanZ), 
		  new TGLayoutHints(kLHintsLeft,20,2,0,0)); 
  fMain->AddFrame(fRadioSlicePos[1] = new TGRadioButton(fMain, 
                  fSlicePosition[kMedianZ], kMedianZ), 
 		  new TGLayoutHints(kLHintsLeft,20,2,0,0)); 
  fMain->AddFrame(fRadioSlicePos[2] = new TGRadioButton(fMain, 
                  fSlicePosition[kHalfZ], kHalfZ), 
		  new TGLayoutHints(kLHintsLeft,20,2,0,0)); 
  fMain->AddFrame(fRadioSlicePos[3] = new TGRadioButton(fMain, 
                  fSlicePosition[kStartZ], kStartZ), 
		  new TGLayoutHints(kLHintsLeft,20,2,0,0)); 
  fMain->AddFrame(fRadioSlicePos[4] = new TGRadioButton(fMain, 
                  fSlicePosition[kEndZ], kEndZ), 
		  new TGLayoutHints(kLHintsLeft,20,2,0,0)); 

  fPeakAndBins = new TGHorizontalFrame(fMain, 600,40);

  fBinsLabel1 = new TGLabel(fPeakAndBins,"(part./bin: ");
  fBinsLabel2 = new TGLabel(fPeakAndBins,") ");

  fBinsEntry = new TGTextEntry(fPeakAndBins, fBinsBuffer = new TGTextBuffer(20));
  fBinsEntry->Resize(40, fBinsEntry->GetDefaultHeight());
  fBinsEntry->SetToolTipText("Enter the number of particles per bin to be used in peak-finding");
  //  fBinsEntry->Connect("ReturnPressed()", "TH5SliceDialog", this, "DoSet()");

  // write previous binning into buffer:
  char bufBins[20];
  sprintf(bufBins, "%d", fParent->GetSlicePartPerBin());
  fBinsBuffer->AddText(0, bufBins);

  fPeakAndBins->AddFrame(fRadioSlicePos[5] = new TGRadioButton(fPeakAndBins, 
       		      	 fSlicePosition[kPeakZ], kPeakZ), 
			 new TGLayoutHints(kLHintsLeft,20,2,0,0)); 
  fPeakAndBins->AddFrame(fBinsLabel1, new TGLayoutHints(kLHintsLeft,2,2,2,0)); 
  fPeakAndBins->AddFrame(fBinsEntry, new TGLayoutHints(kLHintsLeft,2,2,0,0)); 
  fPeakAndBins->AddFrame(fBinsLabel2, new TGLayoutHints(kLHintsLeft,2,2,0,0)); 
  
  fPeakAndBins->MapSubwindows();
  fMain->AddFrame(fPeakAndBins, new TGLayoutHints(kLHintsLeft|kLHintsExpandX,0,2,0,0));

  for (Int_t j = 0; j < 6; j++) {
    fRadioSlicePos[j]->Connect("Pressed()", "TH5SliceDialog", this, "DoSlicePosRadioButtons()");
  }
  EH5SlicePosOpt slicePos = fParent->GetSlicePosOpt();
  fRadioSlicePos[slicePos]->SetOn();
  fBinsEntry->SetEnabled(slicePos == kPeakZ);

//////////////////////////////////////////////////////////
// slice extent option
  fSliceExtentLabel = new TGLabel(fMain,"Slice extent: ");
  fMain->AddFrame(fSliceExtentLabel, new TGLayoutHints(kLHintsLeft,5,5,15,5)); 

  fSliceExtentL = new TGHorizontalFrame(fMain, 600,40);
  fSliceExtentLLabel = new TGLabel(fSliceExtentL,"slice length = ");
  fLengthEntry = new TGTextEntry(fSliceExtentL, fLengthBuffer = new TGTextBuffer(20));
  fLengthEntry->Resize(40, fLengthEntry->GetDefaultHeight());
	fLengthEntry->SetToolTipText("Enter the slice length, dude!");
//  fLengthEntry->Connect("ReturnPressed()", "TH5SliceDialog", this, "DoSet()");
  // write previous length into buffer:
  char bufLength[20];
  sprintf(bufLength, "%5.3f", fParent->GetSliceLength());
  fLengthBuffer->AddText(0, bufLength);

  fSliceLengthUnit = new TGComboBox(fSliceExtentL,200);
  fSliceLengthUnit->AddEntry("mm",0);
  fSliceLengthUnit->AddEntry("ps",1);
  fSliceLengthUnit->Select(fParent->GetSliceLengthUnit());
  fSliceLengthUnit->Resize(45,20);
  fSliceExtentL->AddFrame(fRadioSliceExtent[1] = new TGRadioButton(fSliceExtentL, 
       		      	  "by slice length:      ", kLength), 
			              new TGLayoutHints(kLHintsLeft,20,2,0,0)); 
  fSliceExtentL->AddFrame(fSliceExtentLLabel, new TGLayoutHints(kLHintsLeft,2,2,2,0)); 
  fSliceExtentL->AddFrame(fLengthEntry, new TGLayoutHints(kLHintsLeft,2,2,0,0)); 
  fSliceExtentL->AddFrame(fSliceLengthUnit, new TGLayoutHints(kLHintsLeft,2,2,0,0)); 
  fSliceExtentL->MapSubwindows();

  fMain->AddFrame(fSliceExtentL, new TGLayoutHints(kLHintsLeft|kLHintsExpandX,0,2,0,0));

  fSliceExtentP = new TGHorizontalFrame(fMain, 600,40);
  fSliceExtentPLabel1 = new TGLabel(fSliceExtentP,"slice includes");
  fSliceExtentPLabel2 = new TGLabel(fSliceExtentP,"% particles");
  fPercentEntry = new TGTextEntry(fSliceExtentP, fPercentBuffer = new TGTextBuffer(20));
  fPercentEntry->Resize(40, fPercentEntry->GetDefaultHeight());
	fPercentEntry->SetToolTipText("Enter percentage of bunch that makes up a slice");
//  fPercentEntry->Connect("ReturnPressed()", "TH5SliceDialog", this, "DoSet()");
  // write previous percentage into buffer:
  char bufPercent[20];
  sprintf(bufPercent, "%5.2f", fParent->GetSlicePercent());
  fPercentBuffer->AddText(0, bufPercent);
  fSliceExtentP->AddFrame(fRadioSliceExtent[0] = new TGRadioButton(fSliceExtentP, 
       		      	 "by particle number: ", kPercent), 
			 new TGLayoutHints(kLHintsLeft,20,2,0,0)); 
  fSliceExtentP->AddFrame(fSliceExtentPLabel1, new TGLayoutHints(kLHintsLeft,2,2,2,0)); 
  fSliceExtentP->AddFrame(fPercentEntry, new TGLayoutHints(kLHintsLeft,2,2,0,0)); 
  fSliceExtentP->AddFrame(fSliceExtentPLabel2, new TGLayoutHints(kLHintsLeft,2,2,0,0)); 
  fSliceExtentP->MapSubwindows();
  fMain->AddFrame(fSliceExtentP, new TGLayoutHints(kLHintsLeft|kLHintsExpandX,0,2,0,0));

  //  fRadioSliceExtent[0]->SetToolTipText("could add tips, but not needed");
  for (Int_t j = 0; j < 2; j++) {
    fRadioSliceExtent[j]->Connect("Pressed()", "TH5SliceDialog", this, "DoSliceExtentRadioButtons()");
  }
  EH5SliceExtentOpt sliceExtent = fParent->GetSliceExtentOpt(); 
  fRadioSliceExtent[sliceExtent]->SetOn();
  fPercentEntry->SetEnabled(sliceExtent == kPercent);
  fLengthEntry->SetEnabled(sliceExtent == kLength);

//////////////////////////////////////////
//slice weighting option
  Double_t oldSigma = fParent->GetSliceSigma();
  fSliceWeightLabel = new TGLabel(fMain, "Slice weighting: ");
  fMain->AddFrame(fSliceWeightLabel, new TGLayoutHints(kLHintsLeft,5,5,15,5)); 

  fRadioSliceWeight[0] = new TGRadioButton(fMain, "Uniform", 0);
  fMain->AddFrame(fRadioSliceWeight[0], 
		  new TGLayoutHints(kLHintsLeft,20,2,0,0)); 

  fSliceGaussianWeight = new TGHorizontalFrame(fMain,600,40);
  fRadioSliceWeight[1] = new TGRadioButton(fSliceGaussianWeight, 
		 "Gaussian", 1); 
  fSliceGaussianWeightLabel = new TGLabel(fSliceGaussianWeight,"  sigma=");
  fSliceSigmaEntry = new TGTextEntry(fSliceGaussianWeight, fSliceSigmaBuffer = new TGTextBuffer(20));
  fSliceSigmaEntry->Resize(40, fSliceSigmaEntry->GetDefaultHeight());
  fSliceSigmaEntry->SetToolTipText("Enter Gaussian sigma to be used in slice weigthing");
  fSliceSigmaEntry->SetEnabled(oldSigma > 0.);
//  fSliceSigmaEntry->Connect("ReturnPressed()", "TH5SliceDialog", this, "DoSet()");
  // write previous sigma value into buffer:
  char bufSliceSigma[20];
  if (oldSigma > 0.) {
    sprintf(bufSliceSigma, "%5.3f", oldSigma);
  } else {
    sprintf(bufSliceSigma, "%5.3f", 0.3);
  }
  fSliceSigmaBuffer->AddText(0, bufSliceSigma);
  fGaussianWeightUnit = new TGComboBox(fSliceGaussianWeight,200);
  fGaussianWeightUnit->AddEntry("mm",0);
  fGaussianWeightUnit->AddEntry("ps",1);
  fGaussianWeightUnit->Select(fParent->GetSliceSigmaUnit());
  fGaussianWeightUnit->Resize(45,20);
  //fGaussianWeightUnit->SetEnabled(oldSigma>0); // only from ROOT 5.12 on
  fSliceGaussianWeight->AddFrame(fRadioSliceWeight[1], new TGLayoutHints(kLHintsLeft,20,2,0,0)); 
  fSliceGaussianWeight->AddFrame(fSliceGaussianWeightLabel, new TGLayoutHints(kLHintsLeft,2,2,2,0)); 
  fSliceGaussianWeight->AddFrame(fSliceSigmaEntry, new TGLayoutHints(kLHintsLeft,2,2,0,0)); 
  fSliceGaussianWeight->AddFrame(fGaussianWeightUnit, new TGLayoutHints(kLHintsLeft,2,2,0,0)); 
  fSliceGaussianWeight->MapSubwindows();
  fMain->AddFrame(fSliceGaussianWeight, new TGLayoutHints(kLHintsLeft|kLHintsExpandX,0,2,0,0));
  
  fRadioSliceWeight[0]->Connect("Pressed()", "TH5SliceDialog", this, "DoSliceWeightRadioButtons()");
  fRadioSliceWeight[1]->Connect("Pressed()", "TH5SliceDialog", this, "DoSliceWeightRadioButtons()");

  if (fParent->GetSliceSigma()>0.) {
    fRadioSliceWeight[1]->SetOn();
    fSliceSigmaEntry->SetEnabled();
  } else {
    fRadioSliceWeight[0]->SetOn();
  }

//////////////////////////////////////////
//update option
  fUpdateLabel = new TGLabel(fMain,"Update slice options for... ");
  fMain->AddFrame(fUpdateLabel, new TGLayoutHints(kLHintsLeft,5,5,20,5)); 
  fMain->AddFrame(fRadioUpdateOpt[0] = new TGRadioButton(fMain, 
		 "...all loaded datafiles", kAll), 
		  new TGLayoutHints(kLHintsLeft,20,2,0,0)); 
  fMain->AddFrame(fRadioUpdateOpt[1] = new TGRadioButton(fMain, 
		 "...the selected datafiles only", kSel), 
 		  new TGLayoutHints(kLHintsLeft,20,2,0,0)); 
  fRadioUpdateOpt[0]->Connect("Pressed()", "TH5SliceDialog", this, "DoUpdateOptRadioButtons()");
  fRadioUpdateOpt[1]->Connect("Pressed()", "TH5SliceDialog", this, "DoUpdateOptRadioButtons()");
  fRadioUpdateOpt[0]->SetOn();

////////////////////////////////////////////////
  fBottomFrame = new TGHorizontalFrame(fMain, 60, 20, kFixedWidth);
  
  fSetButton = new TGTextButton(fBottomFrame, "  &Set  ", 1);
  fSetButton->Connect("Clicked()", "TH5SliceDialog", this, "DoSet()");
  fCancelButton = new TGTextButton(fBottomFrame, "&Cancel", 2);
  fCancelButton->Connect("Clicked()", "TH5SliceDialog", this, "DoCancel()");
  
  fBottomFrame->AddFrame(fSetButton,   new TGLayoutHints(kLHintsTop | kLHintsRight ,10, 10, 5, 5));
  fBottomFrame->AddFrame(fCancelButton, new TGLayoutHints(kLHintsTop | kLHintsRight ,10, 10, 5, 5));
 
  fMain->AddFrame(fBottomFrame, new TGLayoutHints(kLHintsBottom | kLHintsExpandX, 0, 0, 10, 0));
  
  fMain->MapSubwindows();

  fMain->Resize();
  
  // position relative to the parent's window
  fMain->CenterOnParent();
  
  fMain->SetWindowName("H5PartROOT Slice Options");
  
  fMain->MapWindow();
  //gClient->WaitFor(fMain);    // otherwise canvas contextmenu does not work
}

TH5SliceDialog::~TH5SliceDialog()
{
  // Delete load dialog widgets.
  fMain->DeleteWindow();  // deletes fMain
}

void TH5SliceDialog::CloseWindow()
{
  // Called when window is closed via the window manager.
  delete this;
}

void TH5SliceDialog::DoSlicePosRadioButtons()
{
  
  TGButton *btn = (TGButton *) gTQSender;
  Int_t id = btn->WidgetId();
  
  switch (id) {
  case kMeanZ:  
    for (Int_t j = 0; j<6; j++) {
      if (j!=kMeanZ) fRadioSlicePos[j]->SetState(kButtonUp);
    }
    fBinsEntry->SetEnabled(kFALSE);
    break;
  case kMedianZ:  
    for (Int_t j = 0; j<6; j++) {
      if (j!=kMedianZ) fRadioSlicePos[j]->SetState(kButtonUp);
    }
    fBinsEntry->SetEnabled(kFALSE);
    break;
  case kHalfZ:  
    for (Int_t j = 0; j<6; j++) {
      if (j!=kHalfZ) fRadioSlicePos[j]->SetState(kButtonUp);
    }
    fBinsEntry->SetEnabled(kFALSE);
    break;
  case kStartZ:  
    for (Int_t j = 0; j<6; j++) {
      if (j!=kStartZ) fRadioSlicePos[j]->SetState(kButtonUp);
    }
    fBinsEntry->SetEnabled(kFALSE);
    break;
  case kEndZ:  
    for (Int_t j = 0; j<6; j++) {
      if (j!=kEndZ) fRadioSlicePos[j]->SetState(kButtonUp);
    }
    fBinsEntry->SetEnabled(kFALSE);
    break;
  case kPeakZ:
    for (Int_t j = 0; j<6; j++) {
      if (j!=kPeakZ) fRadioSlicePos[j]->SetState(kButtonUp);
    }
    fBinsEntry->SetEnabled(kTRUE);
    break;
  default:
    break;
  }
}

void TH5SliceDialog::DoSliceExtentRadioButtons()
{
  
  TGButton *btn = (TGButton *) gTQSender;
  Int_t id = btn->WidgetId();
  
  switch (id) {
  case kPercent:
    for (Int_t j = 0; j<2; j++) {
      if (j!=kPercent) fRadioSliceExtent[j]->SetState(kButtonUp);
    }
    fLengthEntry->SetEnabled(kFALSE);
    fPercentEntry->SetEnabled(kTRUE);
    break;
  case kLength:
    for (Int_t j = 0; j<2; j++) {
      if (j!=kLength) fRadioSliceExtent[j]->SetState(kButtonUp);
    }
    fLengthEntry->SetEnabled(kTRUE);
    fPercentEntry->SetEnabled(kFALSE);
    break;
  default:
    break;
  }
}

void TH5SliceDialog::DoSliceWeightRadioButtons()
{
  
  TGButton *btn = (TGButton *) gTQSender;
  Int_t id = btn->WidgetId();
  
  switch (id) {
  case 0:
    fRadioSliceWeight[1]->SetState(kButtonUp);
    fSliceSigmaEntry->SetEnabled(kFALSE);
    //    fGaussianWeightUnit->SetEnabled(kFALSE); // only from ROOT 5.12 on
    break;
  case 1:
    fRadioSliceWeight[0]->SetState(kButtonUp);
    fSliceSigmaEntry->SetEnabled(kTRUE);
    //    fGaussianWeightUnit->SetEnabled(kTRUE);  // only from ROOT 5.12 on
    break;
  default:
    break;
  }
}void TH5SliceDialog::DoUpdateOptRadioButtons()
{
  
  TGButton *btn = (TGButton *) gTQSender;
  Int_t id = btn->WidgetId();
  
  switch (id) {
  case kAll:
    fRadioUpdateOpt[kSel]->SetState(kButtonUp);
    break;
  case kSel:
    fRadioUpdateOpt[kAll]->SetState(kButtonUp);
    break;
  default:
    break;
  }
}

void TH5SliceDialog::DoSet()
{
  EH5SlicePosOpt posOpt;
  Int_t nSlicePartPerBin=100;
  if (kButtonDown == fRadioSlicePos[kPeakZ]->GetState()) {
    posOpt = kPeakZ; 
    if (fBinsBuffer->GetTextLength() != 0) {
      nSlicePartPerBin = atoi(fBinsBuffer->GetString()); 
    }
  }
  if (kButtonDown == fRadioSlicePos[kEndZ]->GetState()) posOpt = kEndZ; 
  if (kButtonDown == fRadioSlicePos[kStartZ]->GetState()) posOpt = kStartZ; 
  if (kButtonDown == fRadioSlicePos[kHalfZ]->GetState()) posOpt = kHalfZ; 
  if (kButtonDown == fRadioSlicePos[kMedianZ]->GetState()) posOpt = kMedianZ; 
  if (kButtonDown == fRadioSlicePos[kMeanZ]->GetState()) posOpt = kMeanZ; 
  
  EH5SliceExtentOpt extOpt;
  Double_t lengthValue(1.0);
  EH5SliceUnit lengthUnit;
  Double_t slicePercent(20.);
  if (kButtonDown == fRadioSliceExtent[1]->GetState()) {
    extOpt = kLength;
  }
  if (fLengthBuffer->GetTextLength() != 0) {
    lengthValue = atof(fLengthBuffer->GetString());
  }
  lengthUnit = static_cast<EH5SliceUnit>(fSliceLengthUnit->GetSelected());
  if (kButtonDown == fRadioSliceExtent[0]->GetState()) {
    extOpt = kPercent;
  }
  if (fPercentBuffer->GetTextLength() != 0) {
    slicePercent = atof(fPercentBuffer->GetString()); 
  }
 
  Double_t sigmaValue(0.);
  EH5SliceUnit sigmaUnit=kmm;
  if (kButtonDown == fRadioSliceWeight[1]->GetState()) {
    if (fSliceSigmaBuffer->GetTextLength() != 0) {
      sigmaValue = atof(fSliceSigmaBuffer->GetString()); 
    }
    sigmaUnit = static_cast<EH5SliceUnit>(fGaussianWeightUnit->GetSelected());
  }

  EH5SliceUpdateOpt updOpt;
  if (kButtonDown == fRadioUpdateOpt[1]->GetState()) updOpt = kSel; 
  if (kButtonDown == fRadioUpdateOpt[0]->GetState()) updOpt = kAll; 

  fParent->UpdateSliceOptions(posOpt,nSlicePartPerBin,extOpt,slicePercent,lengthValue,lengthUnit,
			      sigmaValue,sigmaUnit,updOpt);
  fMain->SendCloseMessage();
}

void TH5SliceDialog::DoCancel()
{
  fMain->SendCloseMessage();
}
